###########################################################################
#!make
#
#!Makefile Name: MOD_PR06CR.mk
#
#!Description: Makefile for compiling the V2 Cloud production structure
#              file creation (MOD_PR06CR).
#
#!Variables:
#
#       Internal
#        Macros           Definition
#       ~~~~~~~~          ~~~~~~~~~~
#       ADD_F77FLAGS      Process specific FORTRAN 77 compiler options
#       HDF5OPT_LIB       Path name of HDF5 dynamically shared library
#       INC               Accumulated include file search path
#       INC_FILES_SCIENCE Include files developed by science team
#       LIB               Accumulated library search path
#       OBJ               Accumulated set of object files
#       LDFLAGS           Linker flag
#       PATH_SRC_SDST     Directory path to L2 atmosphere shared source
#       PATH_SRC_UW       Directory path to UW-Madison shared source
#       TARGET            Executable file name
#
#!Env Variables
#
#         System
#       Environment
#        Variables    Definition
#       ~~~~~~~~~~~   ~~~~~~~~~~
#       API_INC       Directory path to M-API include files
#       API_LIB       Directory path to M-API library
#       BRAND         Compiler brand. Current valids are "sgi64/sgi32/linux".
#                     "linux" denotes PGI compilers for FORTRAN code and GNU
#                     compilers for C. 
#       F77           FORTRAN 77 compiler definition
#       F77FLAGS      Common MODIS FORTRAN 77 compiler options
#       HDFEOS_INC    Directory path to HDFEOS include files
#       HDFEOS_LIB    Directory path to HDFEOS library
#       HDFINC        Directory path to HDF include files
#       HDFLIB        Directory path to HDF library
#       HDF5LIB       Directory path to HDF5 libraries
#       PGSINC        Directory path to SDPTK include files
#       PGSLIB        Directory path to SDPTK library
#
#        User
#       Environment
#        Variables    Definition
#       ~~~~~~~~~~~   ~~~~~~~~~~
#       MODIS_STORE   Home directory of MODIS atmosphere PGEs.  Differs 
#                     by sensor and satellite.
#
# 
#!Team-unique Header:
#       This software was developed by MODIS Science Data Support Team
#       for the National Aeronautics and Space Administration,
#       Goddard Space Flight Center, under contract NAS5-32373.
#
#!Revision History:
#
#!Note:
#       1) must follow MAPI, SDP TK, HDF, and other header/libs order
#       2) HDFLIB must be explicit and in order of -lmfhdf, -ldf, -ljpeg, -lz.
#       3) system environments, such as API_INC, PGSINC, HDFINC, PGSLIB,
#          API_LIB etc. should be defined before executing this makefile.
#
#!End
###########################################################################

# Define internal macros
PATH_SRC_SDST = $(MODIS_STORE)/shared_src/atmos_src/src_L2
PATH_SRC_UW = $(MODIS_STORE)/shared_src/atmos_src/src_UW

# Objects
OBJ = CONCATENATE.o\
      create_hdf.o\
      get_spec_info.o\
      message.o\
      mod06_Create_Get_Metadata.o\
      MOD_PR06CR.o\
      MODIS_SMF_SETDYNAMICMSG_V2.1.o\
      setup_create_hdf.o\
      strcompress.o\
      STRING_LOC.o\
      strlen.o

# Include file search path
INC = -I$(API_INC)\
      -I$(HDFINC)\
      -I$(PGSINC)\
      -I$(HDFEOS_INC)\
      -I$(PATH_SRC_SDST)\
      -I$(PATH_SRC_UW)

# Include files developed by ST
INC_FILES_SCIENCE = mod06_pcfnum.inc\
                    mod06_create.inc

# Library files
# Libraries
# Define library search path; it includes libhdf5.so for SDPTK 5.2.7.3 and above.
# Note the use of make substitution operator: $(variable:pattern=replacement). If
# environment variable $(HDF5LIB) is undefined, HDF5OPT_LIB is set NULL.

HDF5OPT_LIB = /$(HDF5LIB)/libhdf5.so
LIB         = -L$(API_LIB) -lmapi\
              -L$(PGSLIB) -lPGSTK\
              -L$(HDFEOS_LIB) -lhdfeos \
              $(HDF5OPT_LIB://libhdf5.so=) \
              -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz

# Compiler options are set based on IRIX/LINUX environment. On Linux, 
# FORTRAN codes are compiled using PGI compilers; C codes use the GNU 
#compilers
target :
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
            $(MAKE) $(TARGET) -f MOD_PR06CR.mk \
                    ADD_F77FLAGS=" -c -O2 -bytereclen -col120" \
            ;; \
          linux ) \
            $(MAKE) $(TARGET) -f MOD_PR06CR.mk \
                    ADD_F77FLAGS=" -c -O2" \
            ;; \
          * ) \
            echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
            exit 1 ;; \
        esac


TARGET = MOD_PR06CR.exe

# Rules
$(TARGET): $(OBJ)
	$(F77) $(OBJ) $(LIB) -o $@


MOD_PR06CR.o: MOD_PR06CR.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR06CR.f

mod06_Create_Get_Metadata.o: mod06_Create_Get_Metadata.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_Create_Get_Metadata.f

#--------------------------------------------------------------------
# Compile objects derived from UW-Madison shared source library files
#--------------------------------------------------------------------

create_hdf.o: $(PATH_SRC_UW)/create_hdf.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/create_hdf.f

get_spec_info.o: $(PATH_SRC_UW)/get_spec_info.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/get_spec_info.f

message.o: $(PATH_SRC_UW)/message.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/message.f

setup_create_hdf.o: $(PATH_SRC_UW)/setup_create_hdf.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/setup_create_hdf.f

strcompress.o: $(PATH_SRC_UW)/strcompress.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strcompress.f

#--------------------------------------------------------------
# Compile objects derived from SDST shared source library files
#--------------------------------------------------------------

CONCATENATE.o: $(PATH_SRC_SDST)/CONCATENATE.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CONCATENATE.f

MODIS_SMF_SETDYNAMICMSG_V2.1.o: $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f

STRING_LOC.o: $(PATH_SRC_SDST)/STRING_LOC.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/STRING_LOC.f

strlen.o: $(PATH_SRC_SDST)/strlen.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/strlen.f
#********************** END OF MAKEFILE ***************************
