###########################################################################
#!make
#
#!Makefile Name: MOD_PR06CT.mk
#
#!Description: Makefile for compiling the V2 Cloud Top Properties production
#              software (MOD_PR06CT).
#
#!Variables:
#
#       Internal
#        Macros           Definition
#       ~~~~~~~~          ~~~~~~~~~~
#       ADD_CFLAGS        Process specific C compiler options
#       ADD_F77FLAGS      Process specific FORTRAN 77 compiler options
#       HDF5OPT_LIB       Path name of HDF5 dynamically shared library
#       INC               Accumulated include file search path
#       INC_FILES_SCIENCE Include files developed by science team
#       LIB               Accumulated library search path
#       OBJ               Accumulated set of object files
#       PATH_SRC_SDST     Directory path to L2 atmosphere shared source
#       PATH_SRC_UW       Directory path to UW-Madison shared source
#       TARGET            Executable file name
#
#!Env Variables
#
#         System
#       Environment
#        Variables    Definition
#       ~~~~~~~~~~~   ~~~~~~~~~~
#       API_INC       Directory path to M-API include files
#       API_LIB       Directory path to M-API library
#       BRAND         Compiler brand. Current valids are "sgi64/sgi32/linux".
#                     "linux" denotes PGI compilers for FORTRAN code and GNU
#                     compilers for C.
#       CC            C compiler definition
#       CFLAGS        C compiler options
#       F77           FORTRAN 77 compiler definition
#       F77FLAGS      Common MODIS FORTRAN 77 compiler options
#       F90           FORTRAN 90 compiler definition
#       HDFEOS_INC    Directory path to HDFEOS include files
#       HDFEOS_LIB    Directory path to HDFEOS library
#       HDFINC        Directory path to HDF include files
#       HDFLIB        Directory path to HDF library
#       HDF5LIB       Directory path to HDF5 libraries
#       PGSINC        Directory path to SDPTK include files
#       PGSLIB        Directory path to SDPTK library
#
#        User
#       Environment
#        Variables    Definition
#       ~~~~~~~~~~~   ~~~~~~~~~~
#       MODIS_STORE   Home directory of MODIS atmosphere PGEs.  Differs 
#                     by sensor and satellite. 
#
#
#!Team-unique Header:
#       This software was developed by MODIS Science Data Support Team
#       for the National Aeronautics and Space Administration,
#       Goddard Space Flight Center, under contract NAS5-32373.
#
#!Revision History:
# $Id: MOD_PR06CT.mk,v 1.1 2006/02/07 22:45:29 gumley Exp $
#!Note:
#       1) must follow MAPI, SDP TK, HDF, and other header/libs order
#       2) HDFLIB must be explicit and in order of -lmfhdf, -ldf, -ljpeg, -lz.
#       3) system environments, such as API_INC, PGSINC, HDFINC, PGSLIB,
#          API_LIB etc. should be defined before executing this makefile.
#       4) The PGI F77 compiler option "-byteswapio" is used to compile 
#          tran_modisd042.f and read_reynsst.f on the Linux systems 
#
#!End
###########################################################################

##---------------------------------------------------------------------------
##                  UW Development Environment
#
#PATH_SRC_SDST = $(HOME)/OPS/MODIS/shared/src_L2
#PATH_SRC_UW   = $(HOME)/OPS/MODIS/shared/src_UW
#
## Define development compiler options (this is the default mode)
## NOTE: THESE COMPILER FLAGS WERE CAREFULLY BY CHOSEN BY UW TO ASSIST IN
## DEVELOPMENT AND TESTING. UW IS NOT RESPONSIBLE FOR ANY ERRORS
## INTRODUCED IN THE CODE IF THESE FLAGS ARE DISABLED OR MODIFIED.
#CC=cc
#CFLAGS=-c -n32 -fullwarn -xansi -D_POSIX_SOURCE
#ADD_CFLAGS=-g -C
#
#F77=f77
#F77FLAGS=-c -n32 -fullwarn -u -bytereclen -extend_source -woff 2271,2272,2274
#ADD_F77FLAGS=-g -C
#
#F90=f90
#F90FLAGS=-c -n32 -fullwarn -u -bytereclen -extend_source -woff 2271,2272,2274
#ADD_F90FLAGS=-g -C
#
## Define optimized compiler options
## (To optimize the code uncomment these lines only: do nothing else!)
## NOTE: UW HAS TESTED THIS CODE ON SGI SYSTEMS USING '-O2' OPTIMZATION ONLY.
## UW IS NOT RESPONSIBLE FOR RUN-TIME ERRORS WHICH OCCUR AT ANY HIGHER
## LEVEL OF OPTIMIZATION.
# ADD_CFLAGS=-O2
# ADD_F77FLAGS=-O2
# ADD_F90FLAGS=-O2
#
## Libraries for development/debug
## (Note addition of floating point library -lfpe at end of list)
#LIB = -L$(API_LIB) -lmapi\
#      -L$(PGSLIB) -lPGSTK -lm\
#      -L$(HDFEOS_LIB) -lhdfeos -lGctp\
#      -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz\
#      -lfpe
##---------------------------------------------------------------------------

#----------------------------------------------------------------------------
#                  SDST Integration and Test Environment

# Define internal macros
PATH_SRC_SDST = $(MODIS_STORE)/shared_src/atmos_src/src_L2
PATH_SRC_UW   = $(MODIS_STORE)/shared_src/atmos_src/src_UW

# Include file search path
INC = -I$(API_INC) \
      -I$(HDFINC) \
      -I$(PGSINC) \
      -I$(HDFEOS_INC) \
      -I$(PATH_SRC_SDST) \
      -I$(PATH_SRC_UW)

# Define required libraries; it includes libhdf5.so for SDPTK 5.2.7.3 and above.
# Note the use of make substitution operator: $(variable:pattern=replacement). If
# environment variable $(HDF5LIB) is undefined, HDF5OPT_LIB is set NULL.

HDF5OPT_LIB = /$(HDF5LIB)/libhdf5.so

LIB = -L$(API_LIB) -lmapi \
      -L$(PGSLIB) -lPGSTK -lm \
      -L$(HDFEOS_LIB) -lhdfeos -lGctp \
      $(HDF5OPT_LIB://libhdf5.so=) \
      -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz

# Define includes files developed by ST
INC_FILES_SCIENCE = mod06uw_data.inc \
                    mod06uw_debug.inc \
                    mod06uw_meta.inc \
                    mod06uw_pcfnum.inc \
                    $(PATH_SRC_SDST)/Atmos_AncData.inc \
                    $(PATH_SRC_UW)/platform_name.inc

# Objects
OBJ = \
CCSDS_FullTimeFormat_A.o \
CONCATENATE.o \
Copy_cldmask_M.o \
Delete_CommonTempFiles_Atmos_V2.1.o \
Delete_PCF_TempFile.o \
Driver_MOD_PR06UW.o \
Gen_Modis_FileName.o \
Get_date_time.o \
MODIS_IO_GEN_CLOSE.o \
MODIS_IO_GEN_OPEN.o \
MODIS_SMF_SETDYNAMICMSG_V2.1.o \
MOD_PR06UW.o \
Met_Common_Atmos.o \
Parse_ECS_DateTime.o \
Read_CldMsk_V2.o \
Read_GEO_V2.o \
Read_L1B_V2.2.o \
STRING_LOC.o \
Set_ArchMet_MOD06.o \
Set_CoreMetadata_QC.o \
Set_InvMet_MOD06.o \
assign_eco_emis.o \
blint_met.o \
bl_int.o \
bright_m.o \
brite_m.o \
chk_input_L2.o \
clozo101.o \
compare_times.o\
copy_info.o \
date_to_ccsds.o \
extem101.o \
extend_profile.o \
ezlh_convert.o\
fill_common_arrays.o \
fm_modrad_emis.o \
getcoord.o \
getiremis.o \
get_ancillary_m06ct.o \
get_co2cld.o \
get_ct_stats.o \
get_irphase.o \
get_platform_name.o \
ialign.o \
init_granule_var_mod06ct.o \
init_granule_var_mod06ir.o \
init_scan_var_mod06ct.o \
interp.o \
isort.o \
message_1.o \
mod06CT_compute_products.o \
mod06CT_initialize_output.o \
mod06IR_initialize_output.o \
mod06ct_set_qa.o \
mod06ir_set_qa.o \
mod06uw_open_files.o \
mod06uw_stats_out.o \
mod06_Get_Core_Metadata.o \
mod06_Get_Global_Metadata.o \
mod06_Get_Swath_Metadata.o \
mod06_Set_Runtime_Meta.o \
mod06_check_box.o \
mod06_chk_input.o \
mod06_close_files.o \
mod06_get_data.o \
mod06_get_rp.o \
mod06_write_products.o \
modis_bright.o \
modis_grib_driver_cld.o \
modis_grib_read.o \
modis_planck.o \
opena_temp.o \
openr_temp.o \
planck_m.o \
planc_m.o \
ppv.o \
put_irphase_out_arrays.o \
put_proc_var_irphase.o \
read_goode.o \
read_nise.o \
read_reynsst_ct.o \
read_zonbias.o \
set_qa_bit.o \
strcompress.o \
strlen.o \
strpos.o \
tran_modisd101.o


# Debug mode compiler options
#ADD_CFLAGS   = -g -trapuv -C               -c -O2 -r10000
#ADD_F77FLAGS = -g -trapuv -C -check_bounds -c -O2 -r10000 -col120 -bytereclen
#ADD_F90FLAGS = -g -trapuv -C -check_bounds -c -O2 -r10000 -ansi -fullwarn


# Compiler options are set based on IRIX/LINUX environment. On Linux, 
# FORTRAN codes are compiled using PGI compilers; C codes use the GNU compilers
target :
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
            $(MAKE) $(TARGET) -f MOD_PR06CT.mk \
                    ADD_CFLAGS="   -c -O2 -r10000 -OPT:Olimit=0 -bytereclen -D_POSIX_SOURCE" \
                    ADD_F77FLAGS=" -c -O2 -r10000 -OPT:Olimit=0 -bytereclen -u -col120 -woff 2271,2272,2274" \
            ;; \
          linux ) \
            $(MAKE) $(TARGET) -f MOD_PR06CT.mk \
                    ADD_CFLAGS=" -c -O2 -D_POSIX_SOURCE -funsigned-char" \
                    ADD_F77FLAGS=" -c -O2 -Mstandard" \
            ;; \
          * ) \
            echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
            exit 1 ;; \
        esac  

TARGET = MOD_PR06CT.exe

# Make executable
$(TARGET): $(OBJ)
	$(F90) $(OBJ) $(LIB) -o $@


#--------------------------------------------------------------------
# Compile objects derived from UW-Madison shared source library files
#--------------------------------------------------------------------

Read_GEO_V2.o: $(PATH_SRC_UW)/Read_GEO_V2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/Read_GEO_V2.f

Read_L1B_V2.2.o: $(PATH_SRC_UW)/Read_L1B_V2.2.f $(PATH_SRC_UW)/L1B_Reader_V2.2.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/Read_L1B_V2.2.f

assign_eco_emis.o: $(PATH_SRC_UW)/assign_eco_emis.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/assign_eco_emis.f

bl_int.o: $(PATH_SRC_UW)/bl_int.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/bl_int.f

blint_met.o: $(PATH_SRC_UW)/blint_met.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/blint_met.f

bright_m.o: $(PATH_SRC_UW)/bright_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/bright_m.f

brite_m.o: $(PATH_SRC_UW)/brite_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/brite_m.f

clozo101.o: $(PATH_SRC_UW)/clozo101.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/clozo101.f

compare_times.o: $(PATH_SRC_UW)/compare_times.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/compare_times.f

copy_info.o: $(PATH_SRC_UW)/copy_info.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/copy_info.f

date_to_ccsds.o: $(PATH_SRC_UW)/date_to_ccsds.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/date_to_ccsds.f

ezlh_convert.o: $(PATH_SRC_UW)/ezlh_convert.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/ezlh_convert.f

extem101.o: $(PATH_SRC_UW)/extem101.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/extem101.f

extend_profile.o: $(PATH_SRC_UW)/extend_profile.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/extend_profile.f

getcoord.o: $(PATH_SRC_UW)/getcoord.c
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_UW)/getcoord.c

getiremis.o: $(PATH_SRC_UW)/getiremis.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/getiremis.f

get_platform_name.o: $(PATH_SRC_UW)/get_platform_name.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/get_platform_name.f

interp.o: $(PATH_SRC_UW)/interp.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/interp.f

isort.o: $(PATH_SRC_UW)/isort.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/isort.f

modis_bright.o: $(PATH_SRC_UW)/modis_bright.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/modis_bright.f

modis_grib_driver_cld.o: $(PATH_SRC_UW)/modis_grib_driver_cld.c $(PATH_SRC_UW)/modis_grib_read.h
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_UW)/modis_grib_driver_cld.c

modis_grib_read.o: $(PATH_SRC_UW)/modis_grib_read.c $(PATH_SRC_UW)/modis_grib_read.h
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_UW)/modis_grib_read.c

modis_planck.o: $(PATH_SRC_UW)/modis_planck.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/modis_planck.f

opena_temp.o: $(PATH_SRC_UW)/opena_temp.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/opena_temp.f

openr_temp.o: $(PATH_SRC_UW)/openr_temp.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/openr_temp.f

planc_m.o: $(PATH_SRC_UW)/planc_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/planc_m.f

planck_m.o: $(PATH_SRC_UW)/planck_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/planck_m.f

ppv.o: $(PATH_SRC_UW)/ppv.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/ppv.f

read_goode.o: $(PATH_SRC_UW)/read_goode.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/read_goode.f

read_nise.o: $(PATH_SRC_UW)/read_nise.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/read_nise.f

set_qa_bit.o: $(PATH_SRC_UW)/set_qa_bit.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/set_qa_bit.f

strcompress.o: $(PATH_SRC_UW)/strcompress.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strcompress.f

strpos.o: $(PATH_SRC_UW)/strpos.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strpos.f

#--------------------------------------------------------------
# Compile objects derived from SDST shared source library files
#--------------------------------------------------------------

CCSDS_FullTimeFormat_A.o: $(PATH_SRC_SDST)/CCSDS_FullTimeFormat_A.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CCSDS_FullTimeFormat_A.f

CONCATENATE.o: $(PATH_SRC_SDST)/CONCATENATE.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CONCATENATE.f

Delete_CommonTempFiles_Atmos_V2.1.o: $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f $(PATH_SRC_SDST)/Atmos_AncData.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f

Delete_PCF_TempFile.o: $(PATH_SRC_SDST)/Delete_PCF_TempFile.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_PCF_TempFile.f

Get_date_time.o: $(PATH_SRC_SDST)/Get_date_time.f
	$(F90) -c $(PATH_SRC_SDST)/Get_date_time.f

Gen_Modis_FileName.o: $(PATH_SRC_SDST)/Gen_Modis_FileName.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Gen_Modis_FileName.f

MODIS_IO_GEN_OPEN.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f

MODIS_IO_GEN_CLOSE.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f

MODIS_SMF_SETDYNAMICMSG_V2.1.o: $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f

Met_Common_Atmos.o: $(PATH_SRC_SDST)/Met_Common_Atmos.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Met_Common_Atmos.f

Parse_ECS_DateTime.o: $(PATH_SRC_SDST)/Parse_ECS_DateTime.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Parse_ECS_DateTime.f

Read_CldMsk_V2.o: $(PATH_SRC_SDST)/Read_CldMsk_V2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Read_CldMsk_V2.f

Set_ArchMet_MOD06.o: $(PATH_SRC_SDST)/Set_ArchMet_MOD06.f $(PATH_SRC_SDST)/mod06_ECSMET.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_ArchMet_MOD06.f

Set_CoreMetadata_QC.o: $(PATH_SRC_SDST)/Set_CoreMetadata_QC.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_CoreMetadata_QC.f

Set_InvMet_MOD06.o: $(PATH_SRC_SDST)/Set_InvMet_MOD06.f $(PATH_SRC_SDST)/mod06_ECSMET.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_InvMet_MOD06.f

STRING_LOC.o: $(PATH_SRC_SDST)/STRING_LOC.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/STRING_LOC.f

chk_input_L2.o: $(PATH_SRC_SDST)/chk_input_L2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/chk_input_L2.f

message_1.o: $(PATH_SRC_SDST)/message_1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/message_1.f

strlen.o: $(PATH_SRC_SDST)/strlen.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/strlen.f


#----------------------------------------------------------------------
# Suffix rules used for process dependency files
# NOTE:  Prequisites in suffix rules are non-standard - don't use them.
#----------------------------------------------------------------------
## suffixe rules
#.SUFFIXES: .f .c .o
#
#.f.o: $(INC_FILES_SCIENCE)
#	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $?
#
#.c.o:
#	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $<

#--------------------------------------------------------------
# Compile objects for process dependent Fortran 77 files
#--------------------------------------------------------------
Copy_cldmask_M.o: Copy_cldmask_M.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Copy_cldmask_M.f

Driver_MOD_PR06UW.o: Driver_MOD_PR06UW.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Driver_MOD_PR06UW.f

MOD_PR06UW.o: MOD_PR06UW.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR06UW.f

fill_common_arrays.o: fill_common_arrays.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) fill_common_arrays.f

fm_modrad_emis.o: fm_modrad_emis.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) fm_modrad_emis.f

get_ancillary_m06ct.o: get_ancillary_m06ct.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) get_ancillary_m06ct.f

get_co2cld.o: get_co2cld.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) get_co2cld.f

get_ct_stats.o: get_ct_stats.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) get_ct_stats.f

get_irphase.o: get_irphase.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) get_irphase.f

ialign.o: ialign.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) ialign.f

init_granule_var_mod06ct.o: init_granule_var_mod06ct.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) init_granule_var_mod06ct.f

init_granule_var_mod06ir.o: init_granule_var_mod06ir.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) init_granule_var_mod06ir.f

init_scan_var_mod06ct.o: init_scan_var_mod06ct.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) init_scan_var_mod06ct.f

mod06CT_compute_products.o: mod06CT_compute_products.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06CT_compute_products.f

mod06CT_initialize_output.o: mod06CT_initialize_output.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06CT_initialize_output.f

mod06IR_initialize_output.o: mod06IR_initialize_output.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06IR_initialize_output.f

mod06ct_set_qa.o: mod06ct_set_qa.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06ct_set_qa.f

mod06ir_set_qa.o: mod06ir_set_qa.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06ir_set_qa.f

mod06uw_open_files.o: mod06uw_open_files.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06uw_open_files.f

mod06uw_stats_out.o: mod06uw_stats_out.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06uw_stats_out.f

mod06_Get_Core_Metadata.o: mod06_Get_Core_Metadata.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_Get_Core_Metadata.f

mod06_Get_Global_Metadata.o: mod06_Get_Global_Metadata.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_Get_Global_Metadata.f

mod06_Get_Swath_Metadata.o: mod06_Get_Swath_Metadata.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_Get_Swath_Metadata.f

mod06_Set_Runtime_Meta.o: mod06_Set_Runtime_Meta.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_Set_Runtime_Meta.f

mod06_check_box.o: mod06_check_box.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_check_box.f

mod06_chk_input.o: mod06_chk_input.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_chk_input.f

mod06_close_files.o: mod06_close_files.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_close_files.f

mod06_get_data.o: mod06_get_data.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_get_data.f

mod06_get_rp.o: mod06_get_rp.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_get_rp.f

mod06_write_products.o: mod06_write_products.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod06_write_products.f

put_irphase_out_arrays.o: put_irphase_out_arrays.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) put_irphase_out_arrays.f

put_proc_var_irphase.o: put_proc_var_irphase.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) put_proc_var_irphase.f

read_reynsst_ct.o: read_reynsst_ct.f $(INC_FILES_SCIENCE)
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
             $(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) read_reynsst_ct.f ;; \
          linux ) \
             $(F77) $(F77FLAGS) $(ADD_F77FLAGS) -byteswapio $(INC) read_reynsst_ct.f ;; \
          * ) \
          echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
          exit 1 ;; \
        esac

read_zonbias.o: read_zonbias.f $(INC_FILES_SCIENCE)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) read_zonbias.f

tran_modisd101.o: tran_modisd101.f $(INC_FILES_SCIENCE)
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
             $(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) tran_modisd101.f ;; \
          linux ) \
             $(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) -byteswapio tran_modisd101.f ;; \
          * ) \
             echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
             exit 1 ;; \
          esac

#********************** END OF MAKEFILE ***************************
