module MOD06AlbedoEcoModule

  !*****************************************************************************
  ! !F90
  !
  ! !Description:
  !    This module contains the subroutines used to provide albedo and ecosystem
  !     maps for processing a MOD06 granule.
  !    There is only one callable routine, getAlbedoEco, which returns albedo 
  !     values and IGBP ecosystem classification for each pixel of the MOD06 granule,
  !     and for the wavelengths specified.  Also returned are values of snow albedos
  !     by ecosystem class for the wavelengths specified.
  !  
  ! !Callable routines:
  !    getAlbedoEco()
  ! 
  ! !Revision History:
  !
  ! Revision 1.0  2003/12/18  12:43:43  EGMoody
  ! Initial revision.
  !
  ! !Team-Unique Header:
  !   Cloud Retrieval Group, NASA Goddard Space Flight Center
  !
  ! !References and Credits:
  !   Written by
  !    Eric Moody
  !    Climate and Radiation Branch, Code 913
  !    NASA/GSFC
  !    Greenbelt MD 20771
  !    Eric.Moody@gsfc.nasa.gov
  !
  ! !Design Notes:
  !
  ! !END
  !*****************************************************************************

  !Dependencies:
  use hdf
  !use hdf,           only :  MAX_VAR_DIMS, SFstart, SFend,                    &
  !                           SFn2index, SFselect, SFrdata,                    &
  !                           SFdimid, SFendacc,                               &
  !                           SFfinfo, SFginfo, SFgdinfo, SFgainfo,            & 
  !                           SFfattr, SFrcatt, SFscatt, SFrnatt, SFsnatt,     &
  !                           DFACC_CREATE, DFACC_WRITE, DFACC_READ, FAIL,     &
  !                           DFNT_CHAR, DFNT_FLOAT, DFNT_DOUBLE,              &
  !                           DFNT_INT8, DFNT_UINT8,                           &
  !                           DFNT_INT16, DFNT_UINT16,                         &
  !                           DFNT_INT32, DFNT_UINT32
  use GeneralAuxType
  use nonscience_parameters
  use core_arrays, only : cloudmask
  implicit none
  
  private

  public :: getAlbedoEco



  !local variables:
  !counters:
  integer  (kind = integer_fourbyte)                         :: i,j,k,l,m,n

  !HDF variables:
  integer  (kind = integer_fourbyte)                         :: HDFstatus
  integer  (kind = integer_fourbyte  ),  &
             dimension(MAX_VAR_DIMS)                         :: hdfStart, hdfStride, hdfEdge
  integer  (kind = integer_fourbyte)                         :: sds_id, sds_index
  !  HDFstatus: Used for error checking.
  !  hdfstart : Follows HDF conventions. The array element from which to begin reading the
  !              HDF array. Zero based.
  !  hdfstride: Follows HDF conventions. The frequency with which data should be read in
  !              each dimension. Stride 1 means read every element.
  !  hdfedge  : Follows HDF conventions. The number of elements to read in each dimension. 
  !              If start(:) == 0 and stride(:) == 1, setting edge equal to the shape of 
  !              the underlying array reads all elements. 


contains



 

  ! ----------------------------------------------------------------------------------
  ! ----------------------------------------------------------------------------------
  subroutine getAlbedoEco (Lat, Long, JulianDay, AlbedoWavelengthFN, EcosystemFN,           &
                           SnowAlbedoFN, NISEFN, Debug, WavelengthText, ProcessLandAlbedo,  &
                           icec, Wavelength, Albedo, Ecosystem, Status                      )

! EM -- June 17, 2005
!Apparently ncdump does not handle the metadata very well, and there was some binary conversion 
!issue such that the metadata values are mixed up.  If you use something like HDFlook, they are fine.
!Here is how ncdump prints them out:
!        byte IGBP_Land_Cover_Type(NumLatPoints, NumLongPoints) ;
!                IGBP_Land_Cover_Type:long_name = "IGBP Land Cover Classification Type" ;
!                IGBP_Land_Cover_Type:units = "class number" ;
!                IGBP_Land_Cover_Type:valid_range = '\0', '\376' ;
!                IGBP_Land_Cover_Type:_FillValue = '\377' ;
!                IGBP_Land_Cover_Type:water = '\0' ;
!                IGBP_Land_Cover_Type:evergreen needleleaf forest = '\1' ;
!                IGBP_Land_Cover_Type:evergreen broadleaf forest = '\2' ;
!                IGBP_Land_Cover_Type:deciduous needleleaf forest = '\3' ;
!                IGBP_Land_Cover_Type:deciduous broadleaf forest = '\4' ;
!                IGBP_Land_Cover_Type:mixed forests = '\5' ;
!                IGBP_Land_Cover_Type:closed shrubland = '\6' ;
!                IGBP_Land_Cover_Type:open shrublands = '\7' ;
!                IGBP_Land_Cover_Type:woody savannas = '\10' ;
!                IGBP_Land_Cover_Type:savannas = '\11' ;
!                IGBP_Land_Cover_Type:grasslands = '\12' ;
!                IGBP_Land_Cover_Type:permanent wetlands = '\13' ;
!                IGBP_Land_Cover_Type:croplands = '\14' ;
!                IGBP_Land_Cover_Type:urban and built-up = '\15' ;
!                IGBP_Land_Cover_Type:cropland/natural vegetation mosaic = '\16' ;
!                IGBP_Land_Cover_Type:snow and ice = '\17' ;
!                IGBP_Land_Cover_Type:barren or sparsely vegetated = '\20' ;
!                IGBP_Land_Cover_Type:unclassified = '\376' ;
!If you notice it skips 7 and goes to 10 (open shrub to woody savanna) and then 17 to 20 (snow/ice to barren).
!
!Here is how my code actually assigns them:
!
!*****************************************************************************
! Create the Ecosystem_Order array, used in HDF initialization:
!*****************************************************************************
!    Ecosystem_Order(0)  = 'Ocean / Water'
!    Ecosystem_Order(1)  = 'Evergreen Needle Forest'
!    Ecosystem_Order(2)  = 'Evergreen Broad Forest'
!    Ecosystem_Order(3)  = 'Deciduous Needle Forest'
!    Ecosystem_Order(4)  = 'Deciduous Broad Forest'
!    Ecosystem_Order(5)  = 'Mixed Forest'
!    Ecosystem_Order(6)  = 'Closed Shrubs'
!    Ecosystem_Order(7)  = 'Open / Shrubs'
!    Ecosystem_Order(8)  = 'Woody Savanna'
!    Ecosystem_Order(9)  = 'Savanna'
!    Ecosystem_Order(10) = 'Grassland'
!    Ecosystem_Order(11) = 'Wetland'
!    Ecosystem_Order(12) = 'Cropland'
!    Ecosystem_Order(13) = 'Urban'
!    Ecosystem_Order(14) = 'Crop Mosaic'
!    Ecosystem_Order(15) = 'Antarctic / Permanant Snow'
!    Ecosystem_Order(16) = 'Barren / Desert'
!    Ecosystem_Order(17) = 'Tundra'   
!
!Ncdump is quite deficient in this respect and I get asked this all the time.
!But it is not a coding issue. -- E. Moody

    real      (kind = single ),   &
              dimension(:,:),                    intent(in ) :: Lat, Long
    character (len = *),      &
              dimension(:),                      intent(in ) :: AlbedoWavelengthFN
    character (len = *),                         intent(in ) :: EcosystemFN
    character (len = *),                         intent(in ) :: SnowAlbedoFN
    character (len = *),                         intent(in ) :: NISEFN
    integer   (kind = integer_fourbyte ),        intent(in ) :: JulianDay
    logical,                                     intent(in ) :: Debug
    character (len = *),      &
              dimension(:),                      intent(in ) :: WavelengthText
    logical,  dimension(:),                      intent(in ) :: ProcessLandAlbedo
    real      (kind = single), &
              dimension(:,:),                    intent(in ) :: icec
    real      (kind = single ),   &
              dimension(:),                      intent(out) :: Wavelength
    real      (kind = single ),   &
              dimension(:,:,:),                  intent(out) :: Albedo
    integer   (kind = integer_fourbyte),   &
              dimension(:,:),                    intent(out) :: Ecosystem
    integer   (kind = integer_fourbyte),         intent(out) :: Status

    ! !Description:
    !    This returns albedo values and IGBP ecosystem classification for each pixel 
    !     of the MOD06 granule, and for the wavelengths specified.  Also returned are 
    !     values of snow albedos by ecosystem class for the wavelengths specified.
    !  
    ! !Input Parameters:
    !    Lat, Long       : Latitude and Longitude arrays from granule.
    !    WavelengthFN    : The albedo filenames for each specified wavelength to process.
    !    EcosystemFN     : The filename for the IGBP Ecosystem Classification map.
    !    Julian Day      : Julian Day
    !    Debug           : Logical Flag for printing during debugging.
    !    WavelengthText  : Text array of wavelengths to be processed.
    !
    ! !Output Parameters:
    !    Wavelength      : Real values of the Text array.
    !    Albedo          : 3D array containing the albedo values for each lat/lon point
    !                      and for each wavelength specified.
    !    SnowAlbedo      : Snow albedos by ecosystem class for each wavelength specified.
    !    Ecosystem       : IGBP ecosystem classification for each lat/lon point.
    !    Status          : Flag specifying if this routine worked correctly.
    !
    ! !Revision History:
    !   See Module revision history at the beginning of the file.
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Eric Moody
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Eric.Moody@gsfc.nasa.gov
    !
    ! !Design Notes:
    !  Albedos values for each lat/lon point are provided by finding the nearest neighbor
    !    from the ancillary albedo maps.
    !  Wavelengths to be processed are specified through the WavelengthText array and
    !    corresponding WavelengthFN array.
    !  There is no ancillary files for 3.7 micron, so this is assumed to be half the 2.1 value.
    !  The 3.7 micron always has to be last in the WavelengthText array and WavelengthFN array,
    !    however the WavelengthFN for 3.7 is not used, so it can be fill.
    !  As the albedo ancillary data is for land only, water values are assumed to be 5% for
    !    all wavelengths.
    !  Sea ice is approximated with perm snow/ice values for a winter time, and a percentage 
    !    during a melt season.  A transition period is approximated between winter and melt.
    !  
    !  The following status error values are used:
    !  1 = Array Sizes not valid
    !  2 = Invalid min/max long/lat
    !  3 = Invalid global map start/stop x/y
    !  4 = Could not gain access to Albedo HDF file
    !  5 = Could not gain access to Albedo SDS
    !  6 = Could not read Albedo SDS
    !  7 = Could not end access to Albedo SDS
    !  8 = Could not end access to Albedo HDF file
    !  9 = Could not gain access to Ecosystem HDF file
    !  10 = Could not gain access to Ecosystem SDS
    !  11 = Could not read Ecosystem SDS
    !  12 = Could not end access to Ecosystem SDS
    !  13 = Could not end access to Ecosystem HDF file
    !  14 = Trouble with Albedo Snow Statistics
    !  15 = Trouble with Reading NISE data
    !  16 = Pixel's computed global map index invalid
    !  17 = Pixel's geolocation invalid
    !  18 = Wavelengths do not match for snow statistics
    !  19 = Wavelengths do not match for sea ice values
    !
    !
    ! !END

    !Local Variables

    !
    !Size variables:
    !
    !Number of Albedo Wavelengths to be processed.
    integer  (kind = integer_fourbyte)                        :: NumAlbWavesProcess
    !Number of Columns and Rows of the granule to be processed
    integer  (kind = integer_fourbyte)                        :: NumberOfCols, NumberOfRows                                                                                                   
    !Number of Ecosystem classifications
    integer  (kind = integer_fourbyte), parameter             :: NumEcosystems = 18

    !
    !Min/Max Lat/Lon:
    !
    real     (kind = single)                                  :: minLat, maxLat, minLong, maxLong

    ! 
    !Albedo/Ecosystem map variables:
    !
    !Global map dims and maximum number of albedo wavelengths:
    integer  (kind = integer_fourbyte), parameter             :: NumMapCols        = 21600, &
                                                                 NumMapRows        = 10800, &
                                                                 NumSnowTypes      = 2
                                                                 !dry = 1, wet = 2
                                                                 
    !Stores the resolution (in degrees) of the global map
    real     (kind = single)                                  :: Map_Resolution
    !Stores the upper right corner global map coords, used to compute rest of map coords.
    real     (kind = single)                                  :: westernMostLongitude,      &
                                                                 northernMostLatitude
    !Indices for section of global map that is needed for supplying values for granule:
    integer  (kind = integer_fourbyte)                        :: startX, stopX,             &
                                                                 startY, stopY,             &
                                                                 NumberOfXPoints,           &
                                                                 NumberOfYPoints                                                                                
    real     (kind = single)                                  :: startXval, stopXval,       &
                                                                 startYval, stopYval 
    !Stores the real and integer values of the section of the global albedo map.
    real     (kind = single ),   &
             allocatable, dimension(:,:,:)                    :: AlbedoMap
    integer  (kind = integer_twobyte),   &
             allocatable, dimension(:,:)                      :: AlbedoMapInt
    !Used to scale the albedo values read in as integers from the HDF files.
    real     (kind = single ), parameter                      :: ScaleFactor = 0.00100
    !Stores the snow albedo statistics:
    real     (kind = single ),   &
             allocatable, dimension(:,:,:)                    :: SnowAlbedoStats,    &
                                                                 ProcessSnowAlbStats
    integer  (kind = integer_fourbyte), parameter             :: NumSnowAlbStatWaves = 10
    real   (kind = single), dimension(NumSnowAlbStatWaves)    :: PossStatWavelengths
    logical                                                   :: FoundWave
    !Store the NISE snow type:
    INTEGER(integer_fourbyte), allocatable, DIMENSION(:,:)    :: snowIceType
    !Store the section of the global ecosystem map needed for supplying values for granule:
    integer  (kind = integer_onebyte),   &
             allocatable, dimension(:,:)                      :: EcosystemMap                                                                               
    !Dummy index variables:
    integer  (kind = integer_fourbyte)                        :: xIndex, yIndex                                                                          
    
    !HDF variables:
    !File IDs
    integer  (kind = integer_fourbyte)                        :: EcoMapFID,      &
                                                                 AlbMapFID
    !SDS IDs
    integer  (kind = integer_fourbyte)                        :: EcoMapSDSID,    &
                                                                 AlbMapSDSID    
    !Stores the SDS name
    character(len =  MAX_SDS_NAME_LEN)                        :: SDSName

    !Error handling:
    INTEGER(integer_fourbyte)    :: errorLevel

    !Wavelength descriptions used in snow statistics:
    
    !Variables used for sea ice albedos:
    integer  (kind = integer_fourbyte), parameter             :: NumSeaIceAlbWaves = 8
    real     (kind = single), dimension(NumSeaIceAlbWaves)    :: PossSeaIceWavelengths
    real     (kind = single), dimension(NumSeaIceAlbWaves)    :: meltSeaIceAlbedos,        &
                                                                 drySeaIceAlbedos
    real     (kind = single), allocatable, dimension(:)       :: ProcessMeltSeaIceAlbedos, &
                                                                 ProcessDrySeaIceAlbedos,  &
                                                                 TransitionAlbedo,         &
                                                                 Slope, Intercept
    LOGICAL                                                   :: arcticMeltingSeason,      &
                                                                 WinterToMeltTransition,   &
                                                                 MeltToWinterTransition
    integer  (kind = integer_fourbyte), parameter             :: TransitionDays = 10,      &
                                                                 NHMeltBeg = 152,          &
                                                                 NHMeltEnd = 244,          &
                                                                 SHMeltBeg = 334,          &
                                                                 SHMeltEnd = 61
    real     (kind = single), parameter                       :: PercentageOfDry = 0.800d0
    
  

    !Water albedo values:
    real    (kind = single), parameter                        :: WaterAlbedo = 0.050000

    !*****************************************************************************
    ! Determine the size of the arrays:  
    !*****************************************************************************
    !Determine dimensions:
    NumberOfCols      = size( Albedo(:,:,:),   dim = 1 )
    NumberOfRows      = size( Albedo(:,:,:),   dim = 2 )
    NumAlbWavesProcess = size( Albedo(:,:,:),   dim = 3 )
    !Error checking:
    if ( (NumberOfCols < 1)      .or. &
         (NumberOfRows < 1)      .or. &
         (NumAlbWavesProcess < 2) .or. &
         (NumEcosystems < 1)          ) then
       status = error
       call local_message_handler('Problem with albedo array size, contact SDST',status,'getAlbedoEco')
       return     
    end if

    
    !*****************************************************************************
    ! Convert the text wavelength strings to real for future MOD06OD usage:  
    !*****************************************************************************
    do i = 1, NumAlbWavesProcess
      Wavelength(i) = CharToReal(trim(WavelengthText(i)))
    end do


    !*****************************************************************************
    ! Determine the min/max of lat/lon, which will be used to determine what portion
    !  of the albedo/eco maps need to be read in. 
    !*****************************************************************************
    !Determine min and max:
    if (maxval(Lat) == -999. .or. maxval(Long) == -999.) then
      !there is no gelocation data in the scan.
      Albedo(:,:,:) = -999.  
      return
    else
      minLat  = minval( Lat,  mask = ( (Lat  >= -90.0000 .and. Lat  <= 90.00000) ) )
      maxLat  = maxval( Lat,  mask = ( (Lat  >= -90.0000 .and. Lat  <= 90.00000) ) )
      minLong = minval( Long, mask = ( (Long >= -180.000 .and. Long <= 180.0000) ) )
      maxLong = maxval( Long, mask = ( (Long >= -180.000 .and. Long <= 180.0000) ) )
    endif
 
    !!Error checking:
    !if ( .not. (minLat  >= -90.0000 .and. minLat  <= 90.00000) .or. &
    !     .not. (maxLat  >= -90.0000 .and. maxLat  <= 90.00000) .or. &
    !     .not. (minLong >= -180.000 .and. minLong <= 180.0000) .or. &
    !     .not. (maxLong >= -180.000 .and. maxLong <= 180.0000)      ) then
    !   status = error
    !   call local_message_handler('Problem with lat/long array limits, contact SDST',status,'getAlbedoEco')
    !end if


    !*****************************************************************************
    ! Set up the global albedo/ecosystem map resolution and determine the start/stop
    !  points for the portion of the global map to be read in.  This is done by
    !  computing the map indices from the min/max lat/lon values.
    !*****************************************************************************
    !Determine the map resolution:
    Map_Resolution = 360.000d0 / float(NumMapCols)
    
    !Compute the WesternMostLongitude and NothernMostLatitude of the maps:
    westernMostLongitude = Map_Resolution / 2.0 - 180.0
    northernMostLatitude = 90.0 - Map_Resolution / 2.0
    
    !Compute the start/stop x/y indices based on min/max lat/lon values:
    !use anint so that it rounds to nearest index.
    startx = anint( (minLong-westernMostLongitude) / (Map_Resolution) ) + 1
    stopx  = anint( (maxLong-westernMostLongitude) / (Map_Resolution) ) + 1
    starty = anint( (northernMostLatitude-maxLat ) / (Map_Resolution) ) + 1
    stopy  = anint( (northernMostLatitude-minLat ) / (Map_Resolution) ) + 1
    
    !Due to roundoff, geocoordinates of -90, 90, -180, 180 can result in 
    ! indices of one beyond the map bounds, so correct that here:
    if (startx == 0) startx = 1
    if (stopx  == NumMapCols+1 ) stopx = NumMapCols
    if (starty == 0) starty = 1
    if (stopy  == NumMapRows+1 ) stopy = NumMapRows
    
    !Compute the number of points:
    NumberOfXPoints = stopx - startx + 1
    NumberOfYPoints = stopy - starty + 1

    !Error Checking:
    if ( (startx < 1) .or. (stopx > NumMapCols) .or. &
         (starty < 1) .or. (stopy > NumMapRows) .or. &
         (startx > stopx) .or. (starty > stopy)      ) then
       status = error
       call local_message_handler('Problem with loop index, contact SDST',status,'getAlbedoEco')
       return  
    end if

    !*****************************************************************************
    ! Allocate the global albedo and ecosytem map storage arrays based upon these
    !  start and stop values.  Note that the arrays will be indexed using start/stop
    !  variables, instead of 1-NumberOfZPoints, for ease of selecting the nearest
    !  neighbor to the inputted lat/lon grid.
    !*****************************************************************************
    allocate( AlbedoMap    (startx:stopx, starty:stopy, 1:NumAlbWavesProcess) )
    allocate( AlbedoMapInt (startx:stopx, starty:stopy                      ) )
    allocate( EcosystemMap (startx:stopx, starty:stopy                      ) )    
    !*****************************************************************************
    ! Set up the HDF read variables, making note that HDF arrays start at 0
    !  instead of F90's 1:
    !*****************************************************************************
    !Define the starting point
    hdfStart (:) = 0
    hdfStart (1) = startX - 1
    hdfStart (2) = startY - 1
    
    !Define the stride = 1 so that it doesn't skip any data.
    hdfStride(:) = 1
    
    !Define the Number of Points to read:
    hdfEdge  (:) = 1
    hdfEdge  (1) = NumberOfXPoints
    hdfEdge  (2) = NumberOfYPoints


    !*****************************************************************************
    ! Loop over each wavelength, except for 3.7, and read in the specified area
    !  of the albedo map for those wavelengths flagged for land processing.
    ! Also compute the 3.7 from the 2.1.
    !*****************************************************************************
    !Set the albedo map to fill value, so that wavelengths not needed for land 
    ! processing will be set to fill value (i.e. not read in)
    AlbedoMap(:,:,:) = -999.0000

    !Read in the portions of the albedo maps from their corresponding files:
    do i = 1, NumAlbWavesProcess - 1
      !Test to see if this wavelength should be read in:
      if (ProcessLandAlbedo(i) .and.  index(AlbedoWavelengthFN(i),'notUsed') == 0 ) then
       !Define the SDS Name:
       SDSName = "Albedo_Map_" // trim(WavelengthText(i))

       
       !Open the HDF file for reading:
       AlbMapFID = Sfstart(Trim(AlbedoWavelengthFN(i)), Dfacc_Read)
       if (AlbMapFID == Fail) then
          status = failure
          call local_message_handler('Problem detected albedo file SFSTART',status,'getAlbedoEco')
          return
       end if

       !Obtain the data set ID's:
       AlbMapSDSID   = SFselect(AlbMapFID, SFn2index(AlbMapFID, trim(SDSName)))
       if (AlbMapSDSID == Fail) then
          status = failure
          call local_message_handler('Problem detected albedo file SFselect',status,'getAlbedoEco')
          return
       end if

       !Read in the data:
       HDFstatus = SFrdata(AlbMapSDSID, hdfStart, hdfStride, hdfEdge, AlbedoMapInt)
       !Error checking:
       if (HDFstatus == FAIL) then
          status = failure
          call local_message_handler('Problem detected albedo file sfrdata',status,'getAlbedoEco')
          return
       end if

       !End Access to the datasets and files:
       HDFstatus = SFendacc( AlbMapSDSID )
       if (HDFstatus == FAIL) then
          status = failure
          call local_message_handler('Problem detected albedo file sfendacc',status,'getAlbedoEco')
          return
       end if
       HDFstatus = SFend ( AlbMapFID   )   
       if (HDFstatus == FAIL) then
         status = failure
         call local_message_handler('Problem detected albdeo file sfend',status,'getAlbedoEco')
         return
       end if

       !Convert to real, with scale factor applied, and store in appropriate index of AlbedoMap:
       AlbedoMap(:,:,i) = float(AlbedoMapInt(:,:)) * ScaleFactor
       
       !If this is the 2.1 band, then compute the 3.7 from the 2.1, assumed to be last and 
       ! second to last indices, respectively:
       if (i == NumAlbWavesProcess-1) then
          AlbedoMap(:,:,i+1) = float(AlbedoMapInt(:,:)) * ScaleFactor * 0.5000
       end if
       
      end if !Test to see if this wavelength is to be read in.

      !If this is the 2.1 band, then compute the 3.7 from the 2.1, indices are assumed
      ! index 5 == 2.1 um and index 6 = 3.7  um
      if (ProcessLandAlbedo(i) .and. index(WavelengthText(i),'3.7') /= 0) then
          AlbedoMap(:,:,6) = AlbedoMap(:,:,5) *  0.5000 
      endif
    end do
        
    
    !*****************************************************************************
    ! Read in the portion of the ecosystem map from the corresponding file:
    !*****************************************************************************
    !Define the SDS Name:
    SDSName = "IGBP_Land_Cover_Type"
    
    ! Open the HDF file for reading:
    EcoMapFID = Sfstart(Trim(EcosystemFN), Dfacc_Read)
    if (EcoMapFID == Fail) then
       status = failure
       call local_message_handler('Problem detected ecosysyem file sfstart',status,'getAlbedoEco')
       return
    end if
    
    ! Obtain the data set ID's:
    EcoMapSDSID   = SFselect(EcoMapFID, SFn2index(EcoMapFID, trim(SDSName)))
    if (EcoMapSDSID == Fail) then
       status = failure
       call local_message_handler('Problem detected ecosystem file sfselect',status,'getAlbedoEco')
       return
    end if
  
    ! Read in the data:
    HDFstatus = SFrdata(EcoMapSDSID, hdfStart, hdfStride, hdfEdge, EcosystemMap)
    !Error checking:
    if (HDFstatus == FAIL) then
       status = failure
       call local_message_handler('Problem detected ecosystem file sfrdata',status,'getAlbedoEco')
       return
    end if
          
    !End Access to the datasets and files:
    HDFstatus = SFendacc( EcoMapSDSID )
    if (HDFstatus == FAIL) then
       status = failure
       call local_message_handler('Problem detected ecosystem file sfendacc',status,'getAlbedoEco')
       return
    end if
    HDFstatus = SFend ( EcoMapFID   )   
    if (HDFstatus == FAIL) then
       status = failure
       call local_message_handler('Problem detected ecossystem file sfend',status,'getAlbedoEco')
       return
    end if
    

    !*****************************************************************************
    ! Allocate the Snow Albedo stats and types from 1 to NumberofRows and Cols:
    !*****************************************************************************
    allocate( SnowAlbedoStats    (1:NumSnowAlbStatWaves, 0:NumEcosystems-1, 1:NumSnowTypes) )
    allocate( ProcessSnowAlbStats(1:NumAlbWavesProcess,  0:NumEcosystems-1, 1:NumSnowTypes) )
    allocate( snowIceType        (1:NumberOfCols, 1:NumberOfRows                   ) ) 

    !*****************************************************************************
    ! Read in the snow albedo statistics from the corresponding file:
    !*****************************************************************************
    call ReadSnowAlbStats  ( SnowAlbedoFN, NumSnowTypes, NumSnowAlbStatWaves,  &
                             NumEcosystems, SnowAlbedoStats, errorLevel        )
    if (errorLevel == FAIL) then
       status = error
       call local_message_handler('Problem reported from ReadSnowAlbStats',status,'getAlbedoEco')
       return
    end if

    !*****************************************************************************
    ! For each wavelength to be processed, store the albedo snow statistic in the
    !  cooresponding index in a temp array. Note that 8-10 are broad band, and 
    !  not included.
    !*****************************************************************************
    !Load in the order of the statistical wavelengths from the file:
    PossStatWavelengths(1)  = 0.470000
    PossStatWavelengths(2)  = 0.555000
    PossStatWavelengths(3)  = 0.659000
    PossStatWavelengths(4)  = 0.858000
    PossStatWavelengths(5)  = 1.240000
    PossStatWavelengths(6)  = 1.640000
    PossStatWavelengths(7)  = 2.130000
    PossStatWavelengths(8)  = 0.0 
    PossStatWavelengths(9)  = 0.0
    PossStatWavelengths(10) = 0.0
    
    !Loop over the wavelengths to be processed, storing the cooresponding
    ! data from the complete snowAlbedoStats array.  Do not process 3.7,
    ! the last index as there are no cooresponding values, instead compute
    ! from the 2.1m at the end:
    ProcessSnowAlbStats= 0.
    do i = 1, NumAlbWavesProcess-5
      FoundWave = .false.

      do j = 1, NumSnowAlbStatWaves-3
         if ( mod(Wavelength(i),PossStatWavelengths(j)) < 0.05 ) then
            ProcessSnowAlbStats(i,:,:) = SnowAlbedoStats(j,:,:)
            FoundWave = .true.
            goto 100
         end if
      end do
      100 continue
      if (.not. FoundWave) then
         status = error
         call local_message_handler('Problem detected in wavelength loop',status,'getAlbedoEco')
         return
      end if
    end do
    
    !compute the 3.7m (assume 6 = 3.6 and 5 = 2.1m):
    ProcessSnowAlbStats(6,:,:) = &
       ProcessSnowAlbStats(5,:,:) * 0.5000
                        
    !*****************************************************************************
    ! For each wavelength to be processed, store the albedo sea ica albedo in the
    !  cooresponding index in a temp array. 
    !*****************************************************************************
    !Load in the order of the statistical wavelengths from the file:
    PossSeaIceWavelengths(1) = 0.470000
    PossSeaIceWavelengths(2) = 0.555000
    PossSeaIceWavelengths(3) = 0.659000
    PossSeaIceWavelengths(4) = 0.858000
    PossSeaIceWavelengths(5) = 1.240000
    PossSeaIceWavelengths(6) = 1.640000
    PossSeaIceWavelengths(7) = 2.130000
    PossSeaIceWavelengths(8) = 3.700000
    
    !Set the sea ice values to be equal to the dry perm snow/ice values or percentage of them:
    drySeaIceAlbedos(1:7)  = SnowAlbedoStats(1:7,15,1)
    meltSeaIceAlbedos(1:7) = SnowAlbedoStats(1:7,15,1) * PercentageOfDry
    !3.7 is half of 2.1:
    drySeaIceAlbedos(8)  =  drySeaIceAlbedos(7) * 0.5000d0
    meltSeaIceAlbedos(8) = meltSeaIceAlbedos(7) * 0.5000d0
                         
                                       
    !Allocate the processing albedos:
    allocate( ProcessDrySeaIceAlbedos (1:NumAlbWavesProcess) )
    allocate( ProcessMeltSeaIceAlbedos(1:NumAlbWavesProcess) )
    allocate( TransitionAlbedo        (1:NumAlbWavesProcess) )
    allocate( Slope                   (1:NumAlbWavesProcess) )
    allocate( Intercept               (1:NumAlbWavesProcess) )

    !Loop over the wavelengths to be processed, storing the cooresponding
    ! data from the complete snowAlbedoStats array.  Process 3.7m for this
    ! sea ice case
    do i = 1, NumAlbWavesProcess
      FoundWave = .false.
      do j = 1, NumSeaIceAlbWaves
         if ( mod(Wavelength(i),PossSeaIceWavelengths(j)) < 0.05 ) then
            ProcessDrySeaIceAlbedos(i)  = drySeaIceAlbedos(j)
            ProcessMeltSeaIceAlbedos(i) = meltSeaIceAlbedos(j)
            FoundWave = .true.
            goto 101
         end if
      end do
      101 continue
      if (.not. FoundWave) then
!         status = 19
!         return
      end if
    end do
    

    !*****************************************************************************
    ! Read in the NISE snow type from the corresponding file:
    !*****************************************************************************
    call GetNISEType(NISEFN, long, lat, snowIceType, errorLevel)  
    if (errorLevel == FAIL) then
       status = failure
       call local_message_handler('Problem reported from GetNISEType',status,'getAlbedoEco')
       return
    end if
                                     
    !*****************************************************************************
    ! Loop over each pixel in the lat/lon grid, compute the global coordinates and
    !  set the pixel's albedo and ecosystem values to the corresponding global
    !  albedo and ecosystem map values.  This is essentially a nearest neighbor 
    !  calculation.
    !*****************************************************************************
    do j = 1, NumberOfRows
       do i = 1, NumberOfCols
       
        !Check to see if the geolocation is valid:
        if ( (Lat (i,j) >= -90.0000) .and. &
             (Lat (i,j) <=  90.0000) .and. &
             (Long(i,j) >= -180.000) .and. &
             (Long(i,j) <=  180.000)       ) then
         
          !Valid geolocation, so proceed.
          !Compute the x,y indices from this pixel's lat/long values.
          xIndex = anint( (Long(i,j)-westernMostLongitude) / (Map_Resolution) ) + 1
          yIndex = anint( (northernMostLatitude-Lat (i,j)) / (Map_Resolution) ) + 1

          !Due to roundoff, geocoordinates of -90, 90, -180, 180 can result in 
          ! indices of one beyond the map bounds, so correct that here:
          if (xIndex == 0) xIndex = 1
          if (xIndex  == NumMapCols+1 ) stopx = NumMapCols
          if (yIndex == 0) starty = 1
          if (yIndex  == NumMapRows+1 ) yIndex = NumMapRows
          
          !Checks to make sure indices are within bounds of the map:
          if (      (xIndex < 1) .or. (xIndex > NumMapCols) &
               .or. (yIndex < 1) .or. (yIndex > NumMapRows) ) then
             status = error
             call local_message_handler('Problem detected, bad array indexes (main loop) ',status,'getAlbedoEco')
             return
          end if

          !Implement Rich Frey (UWisc) interpolation scheme in which he sets the map
          ! to 200 if it should be set to snow/ice.  For sea ice, he also uses the icec data:
          if (Lat(i,j) >= 40.0 .or. Lat(i,j) <= -40.0) then
            if (EcosystemMap(xIndex,yIndex) == 0) then
              !This is water, so check if the interpolation flagged as 200, or the icec data
              ! says this is ice, or the NISE has misidentified it as Perm Snow:
              if (     (icec(i,j) > 0.5 .and. icec(i,j) <= 1.0) &
                  .or. (snowIceType(i,j) == 101) &
                  .or. (snowIceType(i,j) == 200) ) then
                 !This is water, so this will be sea ice.  Set snowIceType to 95%
                 ! so that the sea ice flag is triggered
                 snowIceType(i,j) = 95
              end if
            else
              !This is land, so check if the interpolation flagged as 200, or it is
              ! misidentified as sea ice:
              if (     (snowIceType(i,j) >= 1 .and. snowIceType(i,j) <= 100) &
                  .or. (snowIceType(i,j) == 200) ) then
                 !This is snow covered land, so set to dry snow:
                 snowIceType(i,j) = 103
              end if
            end if
          end if

          !Store the ecosystem value:
          Ecosystem(i,j) = EcosystemMap(xIndex,yIndex)
          !Set the albedo to either land values, water, or snow/ice, depending
          ! on the scenario:
          if (      (snowIceType(i,j) >= 1)   &
              .and. (snowIceType(i,j) <= 100) &
              .and. (Ecosystem(i,j) == 0)     ) then

              Ecosystem(i,j) = 15

              !This is sea ice, so set the either wet or dry scenario:
              ! Determine arctic melting season and set albedo values:
              if (lat(i,j) >= 0.0) then
                ! Arctic melting season: June 1st (day 152) to September 1st (day 244):
                SELECT CASE (JulianDay)
                  CASE ( (NHMeltBeg-TransitionDays):NHMeltBeg-1)
                    !transition period between winter and melt, so linear fit between them
                    ! to get albedo value:
                    Slope (:) = (ProcessMeltSeaIceAlbedos(:)-ProcessDrySeaIceAlbedos(:))   &
                              / real(TransitionDays) 
                    Intercept(:) = ProcessDrySeaIceAlbedos(:) - Slope(:)  &
                                 * real(NHMeltBeg-TransitionDays) 
                    TransitionAlbedo(:) = Slope(:) * real(JulianDay) + Intercept(:)    
                    Albedo(i,j,:) = (TransitionAlbedo(:) * real(snowIceType(i,j))/100.000)         &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                  CASE ( NHMeltBeg:NHMeltEnd )
                    !Melt Sea Ice Values:
                    Albedo(i,j,:) = (ProcessMeltSeaIceAlbedos(:) * real(snowIceType(i,j))/100.000) &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                  CASE ( NHMeltEnd+1:(NHMeltEnd+TransitionDays) )
                    !transition period between melt and winter, so linear fit between them
                    ! to get albedo value:
                    Slope (:) = (ProcessDrySeaIceAlbedos(:)-ProcessMeltSeaIceAlbedos(:))   &
                              / real(TransitionDays) 
                    Intercept(:) = ProcessMeltSeaIceAlbedos(:) - Slope(:)  &
                                 * real(NHMeltEnd) 
                    TransitionAlbedo(:) = Slope(:) * real(JulianDay) + Intercept(:)    
                    Albedo(i,j,:) = (TransitionAlbedo(:) * real(snowIceType(i,j))/100.000)         &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                  CASE DEFAULT
                    !Dry Sea Ice values::
                    Albedo(i,j,:) = (ProcessDrySeaIceAlbedos(:) * real(snowIceType(i,j))/100.000) &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                END SELECT
                
              else           
                ! Determine southern hemisphere melting season:
                ! Arctic melting season: December 1st (day 334) to March 1st (day 61):
                SELECT CASE (JulianDay)
                  CASE ( (SHMeltBeg-TransitionDays):SHMeltBeg-1)
                    !transition period between winter and melt, so linear fit between them
                    ! to get albedo value:
                    Slope (:) = (ProcessMeltSeaIceAlbedos(:)-ProcessDrySeaIceAlbedos(:))   &
                              / real(TransitionDays) 
                    Intercept(:) = ProcessDrySeaIceAlbedos(:) - Slope(:)  &
                                 * real(SHMeltBeg-TransitionDays) 
                    TransitionAlbedo(:) = Slope(:) * real(JulianDay) + Intercept(:)    
                    Albedo(i,j,:) = (TransitionAlbedo(:) * real(snowIceType(i,j))/100.000)         &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                    
                  CASE ( SHMeltBeg:366 )
                    !Melt Sea Ice Values:
                    Albedo(i,j,:) = (ProcessMeltSeaIceAlbedos(:) * real(snowIceType(i,j))/100.000) &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                  CASE ( 1:SHMeltEnd )
                    !Melt Sea Ice Values:
                    Albedo(i,j,:) = (ProcessMeltSeaIceAlbedos(:) * real(snowIceType(i,j))/100.000) &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                  CASE (SHMeltEnd+1:(SHMeltEnd+TransitionDays) )
                    !transition period between melt and winter, so linear fit between them
                    ! to get albedo value:
                    Slope (:) = (ProcessDrySeaIceAlbedos(:)-ProcessMeltSeaIceAlbedos(:))   &
                              / real(TransitionDays) 
                    Intercept(:) = ProcessMeltSeaIceAlbedos(:) - Slope(:)  &
                                 * real(SHMeltEnd) 
                    TransitionAlbedo(:) = Slope(:) * real(JulianDay) + Intercept(:)    
                    Albedo(i,j,:) = (TransitionAlbedo(:) * real(snowIceType(i,j))/100.000)         &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
	
                  CASE DEFAULT
                    !Dry Sea Ice values::
                    Albedo(i,j,:) = (ProcessDrySeaIceAlbedos(:) * real(snowIceType(i,j))/100.000) &
                                  + (WaterAlbedo * real(100-snowIceType(i,j))/100.000)
                END SELECT
              end if

          else if ( Ecosystem(i,j) == 15 .or. snowIceType(i,j) == 101 ) then
              !This is permanent snow/ice, so set to either map or stat value:
              if ((Ecosystem(i,j) == 15) .and. &
                  (      AlbedoMap(xIndex,yIndex,1) > 0.00d0 &
                   .and. AlbedoMap(xIndex,yIndex,1) <= 1.00d0) ) then
                !This pixel is perm snow class and has a valid map value,
                ! so set to the map's value:
                Albedo(i,j,:) = AlbedoMap(xIndex,yIndex,:)
              else
                !This pixel isn't flag as perm ice eco, so set it to the
                ! statistical value for perm ice:
                Ecosystem(i,j) = 15
                Albedo(i,j,:) = ProcessSnowAlbStats(:, Ecosystem(i,j), 1)
              end if
          else if ( snowIceType(i,j) == 103 ) then
              !This is dry snow, so set the pixel to dry snow stats:
              Albedo(i,j,:) = ProcessSnowAlbStats(:, Ecosystem(i,j), 1) 
              Ecosystem(i,j)= 15
          else if ( snowIceType(i,j) == 104 ) then
              !This is wet snow, so set the pixel to wet snow stats:
              Albedo(i,j,:) = ProcessSnowAlbStats(:, Ecosystem(i,j), 2) 
              Ecosystem(i,j)= 15
          else if (Ecosystem(i,j) == 0) then
              !Water, so set all wavelengths to the water albedo value, 5%:
              Albedo(i,j,:) = WaterAlbedo
          else
              !This is a snow-free, ocean free, and not perm snow/ice pixel,
              ! so just store the value from the albedo map:
              Albedo(i,j,:) = AlbedoMap(xIndex,yIndex,:)
          end if

        else
          
          !Bad geolocation, so set albedos to fill value
          Albedo(i,j,:) = -999.
   
        end if
      

       end do
   
    end do
    

    !*****************************************************************************
    ! Deallocate arrays:
    !*****************************************************************************
    deallocate( AlbedoMap        )
    deallocate( AlbedoMapInt     )
    deallocate( EcosystemMap     ) 
    deallocate( ProcessSnowAlbStats )
    deallocate( SnowAlbedoStats  )
    deallocate( ProcessDrySeaIceAlbedos )
    deallocate( ProcessMeltSeaIceAlbedos ) 
    deallocate( snowIceType      )
    deallocate( TransitionAlbedo )
    deallocate( Slope            )
    deallocate( Intercept        )

    if (status > 0 )then
      status = error
      call local_message_handler('Undetermined problem detected in getAlbedoEco',status,'getAlbedoEco')
    endif
  end subroutine getAlbedoEco

  ! ----------------------------------------------------------------------------------
  ! ----------------------------------------------------------------------------------

  SUBROUTINE GetNISEType(nise_file, lon, lat, snowIceType, errorLevel)
  !...............................................................................
  !
  !...............................................................................
  ! !F90
  !
  !   !DESCRIPTION: 
  !        return snow ice type using "Near Real-Time SSM/I EASE-Grid Daily 
  !        Global Ice Concentration and Snow Extent (NISE)" data file from NSIDC.
  !        If NISE is not available, leave the snowIceType array untouched. 
  ! 
  !
  !   !INPUT PARAMETERS:
  !        
  !        lat: latitudes  (in degrees:  -90 to  90)
  !        lon: longitudes (in degrees: -180 to 180)
  !
  !   !OUTPUT PARAMETERS:
  !        snowIceType: 
  !                  0          snow-free land
  !              1-100          sea ice concentration percentage
  !                101          permanent ice (Greenland, Antarctica)
  !                102          not used
  !                103          dry snow
  !                104          Not Used / 2000-March 2002 data Wet Snow
  !            105-251          not used
  !                252          mixed pixels at coastlines
  !                253          suspect ice value
  !                254          corners (undefined)
  !                255          ocean
  ! 
  !        errorLevel: return error code, 0 (success) or 1 (fail)
  !...............................................................................
  !   !Revision History:
  !
  !...............................................................................
  !   !Team-unique Header:
  !    Cloud Retrieval Group, NASA/GSFC
  !
  !   !PROGRAMMER:
  !            Jason Li (SM&A)
  !            Climate and Radiation Branch
  !            NASA Goddard Space Flight Center
  !            Greenbelt, Maryland 20771, U.S.A.
  !
  !-------------------------------------------------------------------------------
  !   !END

 
  REAL(single),        DIMENSION(:,:) :: lat, lon
  INTEGER(integer_fourbyte),    DIMENSION(:,:) :: snowIceType
  INTEGER(integer_fourbyte),    INTENT(OUT)    :: errorLevel

  INTEGER, PARAMETER                              :: gridsize = 721  
  CHARACTER(LEN=1), DIMENSION(gridsize, gridsize) :: niseNorth, niseSouth

  INTEGER(integer_fourbyte) :: version, xdim, ydim, iret, nise
  REAL(single)     :: x, y, pixel_lon, pixel_lat

  CHARACTER(*), intent(in)    :: nise_file
  CHARACTER(*), PARAMETER     :: routine = 'MakeModisAuxData: GetSnowIceMask_NISE'


  ! Liam Gumley's NISE reader:
  INTEGER, external :: EZLH_Convert


  !**********

  !Set error level
  errorLevel = 0

  !Determine x, y sizes:
  xdim = SIZE(snowIceType, dim=1)
  ydim = SIZE(snowIceType, dim=2)

  !Read in the NISE data:
  iret = Read_NISE(nise_file, gridsize, niseNorth, niseSouth)
  IF ( iret /= 0 ) THEN
       errorLevel = 1
       GOTO 9999
  ENDIF

  ! Get cell coordinates for southern or northern hemisphere.
  DO n = 1, ydim
  DO m = 1, xdim
  
        pixel_lon = lon(m,n)
        pixel_lat = lat(m,n)
  
        x = min( max( pixel_lon, -179.99 ),  179.99 )
        y = min( max( pixel_lat,  -89.99 ),   89.99 )
  
        IF ( y < 0.0 ) THEN
             iret = EZLH_Convert( 'Sl', y, x, i, j )
        ELSE
             iret = EZLH_Convert( 'Nl', y, x, i, j )
        ENDIF
 
        !  Save output NISE data for southern or northern hemisphere
        ! NISE data grid value definitions:
        !
        !             Data Grid Value     Meaning
        !                  0          snow-free land
        !              1-100          sea ice concentration percentage
        !                101          permanent ice (Greenland, Antarctica)
        !                102          not used
        !                103          dry snow
        !                104          Not Used / 2000-March 2002 data Wet Snow
        !            105-251          not used
        !                252          mixed pixels at coastlines
        !                253          suspect ice value
        !                254          corners (undefined)
        !                255          ocean
  
        IF ( iret /= 0 ) THEN
             errorLevel = 1
        ELSE
             IF( y < 0.0 ) THEN
                 nise = ICHAR( niseSouth( i, j ) )
             ELSE
                 nise = ICHAR( niseNorth( i, j ) )
             ENDIF
             
             !Set the snow/ice type and flag:             
             SELECT CASE (nise)
                    CASE(1:100)
                         !sea ice:
                         snowIceType(m,n) = nise
                    CASE(101)
                         !perm ice:
                         snowIceType(m,n) = 101
                    CASE(103)
                         !dry snow:
                         snowIceType(m,n) = 103
                    CASE(104)
                         !wet snow:
                         !For the 2000-March 2002 data, the snow is further divided by
                         ! wet and dry snow.  Per the NISE documentation, the wet class
                         ! was inaccurate and thereby no longer used after March 2002.
                         ! For reprocesing of 2000-March 2002 data, set this value to 
                         ! dry snow so that the collection has consistent snow albedo
                         ! values.  This is the change for collection 5.
                         snowIceType(m,n) = 103
                    CASE(252)
                         !mixed scene pixels at coasts
                         snowIceType(m,n) = 252
                    CASE(200)
                         !Fill scene pixels:
                         snowIceType(m,n) = 200
                    CASE DEFAULT                  
                         !no snow/ice, so set this type to 0:
                         snowIceType(m,n) = 0
             END SELECT
             
        ENDIF
  END DO
  END DO


  9999 RETURN

  END SUBROUTINE GetNISEType

  ! ----------------------------------------------------------------------------------
  ! ----------------------------------------------------------------------------------

  subroutine ReadSnowAlbStats  ( StatsFN, NumSnowTypes, NumAlbBnds, numEco, &
                                 AlbedoMean,  errorLevel                    )
                                 
    character (len = *),                 intent( in)  :: StatsFN
    real      (kind = single),  &
              dimension(:,0:,:),         intent(inout):: AlbedoMean
    integer   (kind = integer_fourbyte), intent( in)  :: NumAlbBnds,       &
                                                         numEco,           &
                                                         NumSnowTypes
    INTEGER(integer_fourbyte),            INTENT(OUT) :: errorLevel
                                                         
    ! !Description:
    !   This routine will output only a single band of albedo, for the specified amount
    !    of data, to the specified position within the hdf file.
    !  
    ! !Input Parameters:
    !    StatsFN : The name, only, of the new HDF file.
    !    Albedo          : Contains the single band of albedo to be stored.
    !    NumSnowTypes    : 2, Dry or wet, via NISE classes.
    !    NumAlbBnds      : Number of Albedo wavelengths.
    !    numEco          : Number of Ecosystem Classes.
    !
    ! !Output Parameters:
    !    AlbedoMean      : Mean statistic
    !
    ! !Revision History:
    !   See Module revision history at the beginning of the file.
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Eric Moody
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Eric.Moody@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    !local variables: 
    !HDF variables:
    integer  (kind = integer_fourbyte)               :: status
    integer  (kind = integer_fourbyte  ),  &
               dimension(MAX_VAR_DIMS)  :: hdfStart, hdfStride, hdfEdge
    integer  (kind = integer_fourbyte)               :: sds_id, newHDFID, sds_index
    character(len =  MAX_SDS_NAME_LEN)                :: SDSName
    real      (kind =single),  &
              dimension(1:2,1:NumAlbBnds,0:numEco,1:1,1:2)   :: DummyAlb
    !  status    : Used for error checking.
    !  hdfstart  : Follows HDF conventions. The array element from which to begin reading the
    !               HDF array. Zero based.
    !  hdfstride : Follows HDF conventions. The frequency with which data should be read in
    !               each dimension. Stride 1 means read every element.
    !  hdfedge   : Follows HDF conventions. The number of elements to read in each dimension. 
    !               If start(:) == 0 and stride(:) == 1, setting edge equal to the shape of 
    !               the underlying array reads all elements.
    !  sds_id,
    !   newHDFID : HDF SDS ID.
    !  sds_index : Index of the SDS in the HDF file.
    ! SDSName    : Stores the name of the SDS being procesed.


    !************************************************************************************
    ! Open the input file:
    !************************************************************************************
    !Open the albedo HDF file:
    newHDFID = SFstart( trim(StatsFN), DFACC_READ )
    if (newHDFID == Fail) then
       Print *, "Can't Reopen the NEW HDF File", trim(StatsFN)
       errorLevel = fail
       return
    end if


    !************************************************************************************
    ! Input the Albedo mean statistic:
    !************************************************************************************   
    !Set up the output hdf variables, in this case we are writing a Belt:
    ! hdfStart, note that we are storing the entire Longitude, Albedo Bands and Ecosystems
    ! so these start at 0, however we are storing a box of Latitude, so this starts at the
    ! starty counter.
    hdfStart ( : ) = 0
    hdfStart ( 5 ) = 0  !Latitude, 0 = NH, 1 = SH
    hdfStart ( 4 ) = 0  !Longitude, only 1, so 0
    hdfStart ( 3 ) = 0  !Eco
    hdfStart ( 2 ) = 0  !AlbBands
    hdfStart ( 1 ) = 0  !NumSnowTypes

    !Strides = 1, since we are not skiping points
    hdfStride( : ) = 1
    
    !Edge is the total number of points being read:
    ! It is the number of Albedo bands, the number of Ecosystems, the Number of Longitude
    ! boxes, and 1 lat box:
    hdfEdge  ( : ) = 1
    hdfEdge  ( 5 ) = 2
    hdfEdge  ( 4 ) = 1
    hdfEdge  ( 3 ) = numEco
    hdfEdge  ( 2 ) = NumAlbBnds
    hdfEdge  ( 1 ) = NumSnowTypes
    
    !Read the Mean data:
    SDSName = 'Snow_Albedo_Year_Mean'
    !determine the sds_index for the SDS:
    sds_index = SFn2index( newHDFID, trim(SDSName) )
    !get access to this SDS:
    sds_id = SFselect(newHDFID,sds_index)
        
    !Read the data:
    status = SFrdata(sds_id, hdfStart, hdfStride, hdfEdge, DummyAlb)

    !Store the dummy data in the final array:
    do i = 1, NumAlbBnds
      do j = 0, numEco-1
        do k = 1, NumSnowTypes
           AlbedoMean(i,j,k) = DummyAlb(k,i,j,1,1)
        end do
      end do
    end do

    !Error Checking:
    if (sds_index == fail .or. &
        sds_id    == fail .or. &
        status    == fail)     then
        errorLevel = fail
        return
     end if
    
    !End Access to the SDS
    status = sfendacc(sds_id)
    if (status  == fail) then
       errorLevel = fail
       return
    end if

    !************************************************************************************
    ! Close the file:
    !************************************************************************************
    !Close the  HDF file:
    status = SFend(newHDFID)
    if (status == fail) then
       Print *, "Can't End access to the NEW HDF File", trim(StatsFN)
       errorLevel = fail
       return
    end if


  end subroutine ReadSnowAlbStats

  ! ----------------------------------------------------------------------------------
  ! ----------------------------------------------------------------------------------

  function CharToReal(inputString)
    character(len = *), intent( in) :: inputString
    real (kind = single)                :: CharToReal
    ! !F90
    !
    ! !Description:
    !   Reads a character string into an integer variable 
    !  
    ! !Input Parameters:
    !    inputString: character string. Should contain the character representation
    !      of a single value. Traling blanks are allowed. Example: " 30   "
    !
    ! !Output Parameters:
    !    CharToInt returns a single integer value. 
    ! 
    ! !Revision History:
    !   See module revision history in data section of module. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    No internal error checking is done - if an incorrect string
    !      is passed in the entire program will fail with a Fortran runtime error. 
    !
    ! !END
    
    ! Local variables
    character(len = len_trim(inputString)) tempString

    ! Remove trailing blanks, which might be read as 0s on some processors
    ! Use list-directed read. 
    tempString = trim(AdjustL(inputString))
    read(tempString, *) CharToReal
  end function CharToReal

  ! --------------------------------------------------------------------------------
  ! --------------------------------------------------------------------------------

      INTEGER FUNCTION READ_NISE( FILENAME, GRIDSIZE, NISE_NORTH, NISE_SOUTH )
      
!-----------------------------------------------------------------------
! !F77
!
! !DESCRIPTION:
!    To read a "Near Real-Time SSM/I EASE-Grid Daily Global Ice
!    Concentration and Snow Extent (NISE)" data file from NSIDC.
!    Information on these files is available at
!    http://www-nsidc.colorado.edu/NSIDC/CATALOG/catalog_index.html
!
!    This function reads the northern and southern hemisphere azimuthal
!    equal area grids which are stored at 25 km resolution in HDF-EOS.
!
!    To obtain NISE files (which are updated daily), contact NSIDC at
!    nsidc@kryos.colorado.edu
!
! !INPUT PARAMETERS:
!    FILENAME      Name of the NISE file
!    GRIDSIZE      Dimension for output arrays NISE_N and NISE_S
!                  (GRIDSIZE=721 for 25 km azimuthal grid)
!
! !OUTPUT PARAMETERS:
!    READ_NISE     Success flag
!                   0 => Success
!                  -1 => Error opening FILENAME
!                  -2 => Error reading northern hemisphere grid
!                  -3 => Error reading southern hemisphere grid
!    NISE_NORTH    Northern hemisphere data grid
!    NISE_SOUTH    Southern hemisphere data grid
!
!    Data grid value     Meaning
!             0          snow-free land
!         1-100          sea ice concentration percentage
!           101          permanent ice (Greenland, Antarctica)
!           102          not used
!           103          dry snow
!           104          wet snow
!       105-251          not used
!           252          mixed pixels at coastlines
!           253          suspect ice value
!           254          corners (undefined)
!           255          ocean
!           
! !REVISION HISTORY:
!
! !TEAM-UNIQUE HEADER:
!     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
!
! !END
!-----------------------------------------------------------------------

      IMPLICIT NONE

! ... Input arguments

      CHARACTER filename*(*)
      INTEGER gridsize

! ... Output arguments

      CHARACTER*1 nise_north( gridsize, gridsize )
      CHARACTER*1 nise_south( gridsize, gridsize )

! ... Local variables
      
      INTEGER file_id, grid_id, status
      
      INTEGER xsize, ysize
! rhucek 052500: resized upleft and lowrgt to 2 elements
!     DOUBLE PRECISION upleft, lowrgt
      DOUBLE PRECISION upleft(2), lowrgt(2)
      
      INTEGER start( 2 ), stride( 2 ), edge( 2 )
      
! ... HDF-EOS functions
      
      INTEGER gdopen, gdattach, gddetach, gdclose, gdgridinfo, gdrdfld
      EXTERNAL gdopen, gdattach, gddetach, gdclose, gdgridinfo, gdrdfld

      INTEGER DFACC_READ
      PARAMETER ( DFACC_READ = 1 )
      
!-----------------------------------------------------------------------
!     OPEN FILE
!-----------------------------------------------------------------------

! ... Open file

      file_id = gdopen( filename, DFACC_READ )
      if ( file_id .eq. -1 ) then
        read_nise = -1
        return
      endif
      
!-----------------------------------------------------------------------
!     READ NORTHERN HEMISPHERE GRID
!-----------------------------------------------------------------------

! ... Open northern hemisphere grid

      grid_id = gdattach( file_id, 'Northern Hemisphere' )
      
! ... Get grid information

      status = gdgridinfo( grid_id, xsize, ysize, upleft, lowrgt )

! ... Read grid data

      start( 1 ) = 0
      start( 2 ) = 0
      stride( 1 ) = 1
      stride( 2 ) = 1
      edge( 1 ) = xsize
      edge( 2 ) = ysize
      status = gdrdfld( grid_id, 'Extent', start, stride, edge, nise_north )
      if ( status .lt. 0 ) then
        read_nise = -2
        return
      endif
      
!     Call routine to fill in snow/ice info for coastal regions as 
!     much as possible (northern hemisphere).
      call massage_snowice(nise_north,xsize,ysize)

! ... Close northern hemisphere grid

      status = gddetach( grid_id )

!-----------------------------------------------------------------------
!     READ SOUTHERN HEMISPHERE GRID
!-----------------------------------------------------------------------

! ... Open southern hemisphere grid

      grid_id = gdattach( file_id, 'Southern Hemisphere' )

! ... Get grid information

      status = gdgridinfo( grid_id, xsize, ysize, upleft, lowrgt )

! ... Read grid data

      start( 1 ) = 0
      start( 2 ) = 0
      stride( 1 ) = 1
      stride( 2 ) = 1
      edge( 1 ) = xsize
      edge( 2 ) = ysize
      status = gdrdfld( grid_id, 'Extent', start, stride, edge, nise_south )
      if ( status .lt. 0 ) then
        read_nise = -3
        return
      endif
      
!     Call routine to fill in snow/ice info for coastal regions as 
!     much as possible (southern hemisphere).
      call massage_snowice(nise_south,xsize,ysize)

! ... Close southern hemisphere grid

      status = gddetach( grid_id )

!-----------------------------------------------------------------------
!     CLOSE FILE
!-----------------------------------------------------------------------

! ... Close file and return success flag

      status = gdclose( file_id )
      read_nise = 0

      END FUNCTION READ_NISE

!---------------------------------------------------------------------
!---------------------------------------------------------------------
!---------------------------------------------------------------------


      subroutine massage_snowice(map_nise,xsize,ysize)

!---------------------------------------------------------------------
!!F77
!
!!Description:
!     Subroutine for filling in snow and ice information in coastal
!     regions.  For all coast pixels in the NISE data, search both
!     latitudinally and longitudinally for non-coast pixels which
!     indicate snow or ice.  Stop search when non-snow/ice, non-coast
!     pixels are found.  For those coast pixels which have snow/ice
!     pixels indicated on their boundaries, set flag to indicate 
!     snow/ice.
!
!!Input Parameters:
! map_nise    Input/output snow/ice data grid
! xsize       Grid size in the horizontal direction
! ysize       Grid size in the vertical direction
!
!!Output Parameters:
! map_nise    Input/output snow/ice data grid
!
!!Revision History:
! Original subroutine:
!     03-09-04  R. Frey
!
!!Team-Unique Header:
!
!!References and Credits:
! See Cloud Mask ATBD-MOD-06.
!
!!End
!----------------------------------------------------------------------

      implicit none
      save

      !Arguments.
      integer xsize, ysize
      character*1 map_nise(xsize,ysize)

      !Local arrays.
      integer k(4)

      !Local scalars.
      integer temp_nise, i, j, m, n

      !Loop through map data.
 
      do j = 1, xsize

        do i = 1, ysize

          temp_nise = 0

         !Check for coast pixel (value = 252).

          if( ichar( map_nise(j,i) ) .eq. 252 ) then

           !Check if on map edges.  Cannot search on all four sides.
           !No data in corners.

            if(j .eq. 1) then
              k(1) = ichar(map_nise(j+1,i))
              k(2) = ichar(map_nise(j,i-1))
              k(3) = ichar(map_nise(j,i+1))
              k(4) = 0
         
            else if(i .eq. 1) then
              k(1) = ichar(map_nise(j,i+1))
              k(2) = ichar(map_nise(j+1,i))
              k(3) = ichar(map_nise(j-1,i))
              k(4) = 0

            else if(j .eq. xsize) then
              k(1) = ichar(map_nise(j-1,i))
              k(2) = ichar(map_nise(j,i-1))
              k(3) = ichar(map_nise(j,i+1))
              k(4) = 0
         
            else if(i .eq. ysize) then
              k(1) = ichar(map_nise(j,i-1))
              k(2) = ichar(map_nise(j+1,i))
              k(3) = ichar(map_nise(j-1,i))
              k(4) = 0

            else

             !Get all four adjacent values.
              k(1) = ichar(map_nise(j,i-1))
              k(2) = ichar(map_nise(j,i+1))
              k(3) = ichar(map_nise(j-1,i))
              k(4) = ichar(map_nise(j+1,i))

            end if

           !Fill in missing snow/ice values (=200).
            do m = 1,4

              n = k(m)
              if(n .ne. 0) then

                if((n .eq. 103 .or. n .eq. 104 .or. n .eq. 200) .or. &
                                  (n .gt. 25 .and. n .lt. 102)) then
                  temp_nise = 200
                end if

              end if
            enddo

            if (temp_nise .eq. 200) map_nise(j,i) = char(temp_nise)

          end if

        enddo

      enddo

      return
      end subroutine massage_snowice

!---------------------------------------------------------------------
!---------------------------------------------------------------------
!---------------------------------------------------------------------


end module MOD06AlbedoEcoModule
