###########################################################################
#!make
#
#!Makefile Name: MOD_PR06OD.mk
#
#!Description: Makefile for compiling the V2 Cloud Optical Properties
#              production software (MOD_PR06OD).
#
#!Variables:
#
#       Internal
#        Macros           Definition
#       ~~~~~~~~          ~~~~~~~~~~
#       ADD_F77FLAGS      Process specific FORTRAN 77 compiler options
#       ADD_F90FLAGS      Process specific FORTRAN 90 compiler options
#       ADD_CFLAGS        Process specific C compiler options
#       HDF5OPT_LIB       Path name of HDF5 dynamically shared library
#       INC               Accumulated include file search path
#       INC_FILES_SCIENCE Process specific include files
#       LIB               Accumulated library search path
#       OBJ               Accumulated set of object files
#       PATH_SRC_SDST     Directory path to L2 atmosphere shared source
#       PATH_SRC_L3       Directory path to L3 atmosphere shared source
#       PATH_SRC_UW       Directory path to UW-Madison shared source
#       TARGET            Executable file name
#
#!Env Variables
#
#         System
#       Environment
#        Variables    Definition
#       ~~~~~~~~~~~   ~~~~~~~~~~
#       API_INC       Directory path to M-API include files
#       API_LIB       Directory path to M-API library
#       BRAND         Compiler configuration. Current valids are "sgi64/sgi32/linux". 
#                     "linux" denotes PGI compilers for FORTRAN code and GNU 
#                     compilers for C.
#       CC            C compiler definition
#       CFLAGS        Common MODIS C compiler options
#       F77           FORTRAN 77 compiler definition
#       F77FLAGS      Common MODIS FORTRAN 77 compiler options
#       F90           FORTRAN 90 compiler definition
#       F90FLAGS      Common MODIS FORTRAN 90 compiler options
#       HDFEOS_INC    Directory path to HDFEOS include files
#       HDFEOS_LIB    Directory path to HDFEOS library
#       HDFINC        Directory path to HDF include files
#       HDFLIB        Directory path to HDF library
#       HDF5LIB       Directory path to HDF5 libraries
#       PGSINC        Directory path to SDPTK include files
#       PGSLIB        Directory path to SDPTK library
#
#        User
#       Environment
#        Variables    Definition
#       ~~~~~~~~~~~   ~~~~~~~~~~
#       MODIS_STORE   Home directory of MODIS atmosphere shared source code
#
#
#!Team-unique Header:
#       This software was developed by MODIS Science Data Support Team
#       for the National Aeronautics and Space Administration,
#       Goddard Space Flight Center, under contract NAS5-32373.
#
#!Revision History:
#
# 08/10/2004 mag
# added clean directive
#
# 08/05/1998 fhliang
# changed to comply with MODIS makefile standards.
#
#!Note:
#       1) must follow MAPI, SDP TK, HDF, and other header/libs order
#       2) HDFLIB must be explicit and in order of -lmfhdf, -ldf, -ljpeg, -lz.
#       3) system environments, such as API_INC, PGSINC, HDFINC, PGSLIB,
#          API_LIB etc. should be defined before executing this makefile.
#
#!End
###########################################################################

# Define internal macros
PATH_SRC_SDST = $(MODIS_STORE)/shared_src/atmos_src/src_L2
PATH_SRC_L3   = $(MODIS_STORE)/shared_src/atmos_src/src_L3
PATH_SRC_UW   = $(MODIS_STORE)/shared_src/atmos_src/src_UW

# Define object files
OBJ =       \
            MODIS_SMF_SETDYNAMICMSG_V2.1.o \
            CONCATENATE.o \
            STRING_LOC.o \
            strcompress.o \
            message_1.o \
            MODIS_IO_GEN_OPEN.o \
            MODIS_IO_GEN_CLOSE.o \
            modis_io_gen_temp_openf.o \
            modis_grib_driver_1.o \
            modis_grib_read.o \
            ezlh_convert.o \
            typeSizes.o \
            ErrorHandling.o \
            Read_CldMsk_V2.o \
            Read_L1B_V2.1.o \
            Delete_CommonTempFiles_Atmos_V2.1.o \
            Delete_PCF_TempFile.o \
            strlen.o \
            Get_date_time.o \
            Met_Common_Atmos.o \
            Gen_Modis_FileName.o \
            Set_ArchMet_MOD06.o \
            Set_InvMet_MOD06.o \
            chk_input_L2.o \
            GeneralAuxType.o \
            science_parameters.o \
            hdf.o \
            libraryarrays.o \
            modis_sciencestructure.o \
            modis_cloudstructure.o \
            core_arrays.o \
            nonscience_parameters.o \
            local_message_handler.o \
            mod06_run_settings.o \
            global_model_grids.o \
            get_platform_name.o \
            modis_cloudhandler.o\
            openr_temp_SDST.o \
            svpwat.o \
            modis_io_module.o\
            MOD06AlbedoEcoModule.o \
            local_read_reynsst.o \
            ancillary_module.o\
            libraryinterpolates.o \
            bright_m.o \
            brite_m.o \
            planck_m.o \
            planc_m.o \
            modis_bright.o \
            sciencemodule_interface.o \
            modis_numerical_module.o\
            spline_module.o \
            modis_planck.o \
            multi_layer_clouds.o \
            get_retrieval_uncertainty.o\
            clear_sky_restoral.o \
            remove_edge_scenes.o\
            corescience_module.o\
            modis_frontend_module.o\
            modis_science_module.o\
            mod_pr06od.o \
            Driver_MOD_PR06OD.o

# Define library search path; it includes libhdf5.so for SDPTK 5.2.7.3 and above.
# Note the use of make substitution operator: $(variable:pattern=replacement). If
# environment variable $(HDF5LIB) is undefined, HDF5OPT_LIB is set NULL.

HDF5OPT_LIB = /$(HDF5LIB)/libhdf5.so

LIB = -L$(API_LIB) -lmapi \
      -L$(PGSLIB) -lPGSTK -lm \
      -L$(HDFEOS_LIB) -lhdfeos -lGctp \
      $(HDF5OPT_LIB://libhdf5.so=) \
      -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz


# Include file search path
INC = -I$(API_INC)\
      -I$(PGSINC)\
      -I$(HDFINC)\
      -I$(HDFEOS_INC)\
      -I$(PATH_SRC_SDST)\
      -I$(PATH_SRC_UW)\
      -I$(PATH_SRC_L3)

# Include files developed by ST
INC_FILES_SCIENCE = mod06.inc

###########################################################################
#                           debug flags and environment
#
# -trapuv:        identifies usage of undefined variables as "INVALID" 
# -C:             terminates program at point of subscript out-of-range. 
# -check_bounds:  same as -C
# -g:             used to run in UNIX debugger, dbx 
# -lfpe:          collects floating point statistics as defined in 
#                 environment variable "TRAP_FPE".  E.g., set TRAP_FPE 
#                 setenv TRAP_FPE "ALL=COUNT,TRACE(1)"
#
# test 1:  run from UNIX command line: perform subscript range checking, 
#          count occurrences of floating point exception (fpe) types including
#          INVALID, DIVZERO, UNDERFL, OVERFL.  Trace first occurrence of 
#          each fpe type and print to standard output. 
#ADD_CFLAGS   = -O0               -trapuv -C               -c -r10000 -lfpe
#ADD_F77FLAGS = -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -col120 -bytereclen -lfpe
#ADD_F90FLAGS = -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -ansi -fullwarn -lfpe
#setenv TRAP_FPE "ALL=COUNT,TRACE(1)"
#
# test 2: like test 1, but with full access to UNIX debugger features available.
#ADD_CFLAGS   = -g -O0               -trapuv -C               -c -r10000 -lfpe
#ADD_F77FLAGS = -g -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -col120 -bytereclen -lfpe
#ADD_F90FLAGS = -g -O0 -OPT:Olimit=0 -trapuv -C -check_bounds -c -r10000 -ansi -fullwarn -lfpe
#setenv TRAP_FPE "ALL=COUNT,TRACE(1)"
###########################################################################


# Compiler options are set based on IRIX/LINUX environment. On Linux,
# FORTRAN codes are compiled using LINUX PGI compilers; C codes use the
# GNU compilers 
target :
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
            $(MAKE) $(TARGET) -f MOD_PR06OD.mk \
                    ADD_CFLAGS="   -c -O2" \
                    ADD_F77FLAGS=" -c -O2 -col120 -bytereclen" \
                    ADD_F90FLAGS=" -c -O2 -ansi -bytereclen" \
            ;; \
          linux ) \
            $(MAKE) $(TARGET) -f MOD_PR06OD.mk \
                    ADD_CFLAGS="   -c -O2 -funsigned-char" \
                    ADD_F77FLAGS=" -c -O2 -Mstandard" \
                    ADD_F90FLAGS=" -c -O2 -Mstandard" \
            ;; \
          * ) \
            echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
            exit 1 ;; \
        esac

# Executable name
TARGET = MOD_PR06OD.exe

# Target file (executable)
$(TARGET): $(OBJ)
	$(F90) $(OBJ) $(LIB) -o $@

#----------------------------------------------------------------------
# Suffix rules used for process dependency files
# NOTE:  Prequisites in suffix rules are non-standard - don't use them.
#----------------------------------------------------------------------
#.SUFFIXES: .o .f .f90

#.f.o: $(INC_FILES_SCIENCE)
#	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $<

#.f90.o: $(INC_FILES_SCIENCE)
#	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) $<

#--------------------------------------------------------------------
# Compile objects derived from UW-Madison shared source library files
#--------------------------------------------------------------------

planck_m.o: $(PATH_SRC_UW)/planck_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/planck_m.f

planc_m.o: $(PATH_SRC_UW)/planc_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/planc_m.f

bright_m.o: $(PATH_SRC_UW)/bright_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/bright_m.f

brite_m.o: $(PATH_SRC_UW)/brite_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/brite_m.f

ezlh_convert.o: $(PATH_SRC_UW)/ezlh_convert.f
	$(F90) $(F90FLAGS) -c $(PATH_SRC_UW)/ezlh_convert.f

get_platform_name.o: $(PATH_SRC_UW)/get_platform_name.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/get_platform_name.f

modis_io_gen_temp_openf.o: $(PATH_SRC_UW)/modis_io_gen_temp_openf.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/modis_io_gen_temp_openf.f

modis_grib_read.o: $(PATH_SRC_UW)/modis_grib_read.c $(PATH_SRC_UW)/modis_grib_read.h
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_UW)/modis_grib_read.c

strcompress.o: $(PATH_SRC_UW)/strcompress.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strcompress.f

svpwat.o: $(PATH_SRC_UW)/svpwat.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/svpwat.f


#--------------------------------------------------------------
# Compile objects derived from SDST shared source library files
#--------------------------------------------------------------

modis_grib_driver_1.o: $(PATH_SRC_SDST)/modis_grib_driver_1.c $(PATH_SRC_UW)/modis_grib_read.h
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_SDST)/modis_grib_driver_1.c

chk_input_L2.o: $(PATH_SRC_SDST)/chk_input_L2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/chk_input_L2.f

CONCATENATE.o: $(PATH_SRC_SDST)/CONCATENATE.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CONCATENATE.f

Delete_CommonTempFiles_Atmos_V2.1.o: $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f

Delete_PCF_TempFile.o: $(PATH_SRC_SDST)/Delete_PCF_TempFile.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_PCF_TempFile.f

ErrorHandling.o: $(PATH_SRC_L3)/ErrorHandling.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) $(PATH_SRC_L3)/ErrorHandling.f90

Get_date_time.o: $(PATH_SRC_SDST)/Get_date_time.f
	$(F90) $(F90FLAGS) -c  $(PATH_SRC_SDST)/Get_date_time.f

Gen_Modis_FileName.o: $(PATH_SRC_SDST)/Gen_Modis_FileName.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Gen_Modis_FileName.f

Met_Common_Atmos.o: $(PATH_SRC_SDST)/Met_Common_Atmos.f $(PATH_SRC_SDST)/Atmos_ECSMET.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Met_Common_Atmos.f

MODIS_IO_GEN_OPEN.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f

MODIS_IO_GEN_CLOSE.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f

MODIS_SMF_SETDYNAMICMSG_V2.1.o: $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f

Read_CldMsk_V2.o: $(PATH_SRC_SDST)/Read_CldMsk_V2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Read_CldMsk_V2.f

Read_L1B_V2.1.o: $(PATH_SRC_SDST)/Read_L1B_V2.1.f $(PATH_SRC_SDST)/L1B_Reader_V2.1.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Read_L1B_V2.1.f

Set_ArchMet_MOD06.o: $(PATH_SRC_SDST)/Set_ArchMet_MOD06.f $(PATH_SRC_SDST)/mod06_ECSMET.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_ArchMet_MOD06.f

Set_InvMet_MOD06.o: $(PATH_SRC_SDST)/Set_InvMet_MOD06.f $(PATH_SRC_SDST)/mod06_ECSMET.inc
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_InvMet_MOD06.f

strlen.o: $(PATH_SRC_SDST)/strlen.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/strlen.f

STRING_LOC.o: $(PATH_SRC_SDST)/STRING_LOC.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/STRING_LOC.f

message_1.o: $(PATH_SRC_SDST)/message_1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/message_1.f

openr_temp_SDST.o: $(PATH_SRC_SDST)/openr_temp_SDST.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/openr_temp_SDST.f

typeSizes.o: $(PATH_SRC_L3)/typeSizes.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(PATH_SRC_L3)/typeSizes.f90

#--------------------------------------------------------------
# Compile objects for MOD_PR06OD specific Fortran 90 files
#--------------------------------------------------------------
mod_pr06od.o: mod_pr06od.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) mod_pr06od.f90

GeneralAuxType.o: GeneralAuxType.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) GeneralAuxType.f90

clear_sky_restoral.o: clear_sky_restoral.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) clear_sky_restoral.f90

core_arrays.o: core_arrays.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) core_arrays.f90

nonscience_parameters.o: nonscience_parameters.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) nonscience_parameters.f90

mod06_run_settings.o: mod06_run_settings.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) mod06_run_settings.f90

modis_sciencestructure.o: modis_sciencestructure.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) modis_sciencestructure.f90

modis_cloudstructure.o:modis_cloudstructure.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) modis_cloudstructure.f90

local_message_handler.o: local_message_handler.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) local_message_handler.f90

hdf.o: hdf.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) hdf.f90

libraryarrays.o: libraryarrays.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) libraryarrays.f90

science_parameters.o: science_parameters.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) science_parameters.f90

modis_cloudhandler.o: modis_cloudhandler.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) modis_cloudhandler.f90

global_model_grids.o: global_model_grids.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) global_model_grids.f90

local_read_reynsst.o: local_read_reynsst.f90
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
		$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) \
			local_read_reynsst.f90 ;; \
          linux ) \
		$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) \
			-byteswapio local_read_reynsst.f90 ;; \
          * ) \
          echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
          exit 1 ;; \
        esac

ancillary_module.o: ancillary_module.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) ancillary_module.f90

MOD06AlbedoEcoModule.o: MOD06AlbedoEcoModule.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) MOD06AlbedoEcoModule.f90

libraryinterpolates.o: libraryinterpolates.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) libraryinterpolates.f90

modis_numerical_module.o: modis_numerical_module.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) modis_numerical_module.f90

spline_module.o: spline_module.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) spline_module.f90

corescience_module.o: corescience_module.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) corescience_module.f90

# ===================================================================
# Note reduction in optimization flag to "-O1" for this routine only,
# under Linux only.
# ===================================================================
multi_layer_clouds.o: multi_layer_clouds.f90
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
		$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) \
			multi_layer_clouds.f90 ;; \
          linux ) \
		$(F90) $(F90FLAGS) -c -O1 -Mstandard $(INC) \
			multi_layer_clouds.f90 ;; \
          * ) \
          echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
          exit 1 ;; \
        esac

remove_edge_scenes.o: remove_edge_scenes.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) remove_edge_scenes.f90

sciencemodule_interface.o: sciencemodule_interface.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) sciencemodule_interface.f90

get_retrieval_uncertainty.o: get_retrieval_uncertainty.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) get_retrieval_uncertainty.f90

modis_io_module.o: modis_io_module.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) modis_io_module.f90

modis_frontend_module.o: modis_frontend_module.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) modis_frontend_module.f90

modis_science_module.o: modis_science_module.f90
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) modis_science_module.f90

#--------------------------------------------------------------
# Compile objects for MOD_PR06OD specific Fortran 77 files
#--------------------------------------------------------------
modis_bright.o: modis_bright.f 
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) modis_bright.f

modis_planck.o: modis_planck.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) modis_planck.f

Driver_MOD_PR06OD.o: Driver_MOD_PR06OD.f $(INC_FILES_SCIENCE) \
                     $(PATH_SRC_SDST)/Atmos_AncData.inc 
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) Driver_MOD_PR06OD.f

clean:
	-rm -f $(OBJ) *.mod
#********************** END OF MAKEFILE ***************************
