 module clear_sky_restoral
!-------------------------------------------------------------------------------
! DESCRIPTION:
!        this module contains the routines used to perform spatial variability
!        tests and 250m cloudiness evaluation.
!
!
! PROGRAMMER:
!            Gala Wind (L3 GSI)
!            Climate and Radiation Branch
!            Code 913, NASA Goddard Space Flight Center
!            Greenbelt, Maryland 20771, U.S.A.
!
!            Mark Gray (L3 GSI)
!            Climate and Radiation Branch
!            Code 913, NASA Goddard Space Flight Center
!            Greenbelt, Maryland 20771, U.S.A.
!            gray@climate.gsfc.nasa.gov
!------------------------------------------------------------------------------
! REVISION:
!            10.22.04  integrated with MOD_PR06OD, substantial style changes.
!            10.07.04  Initial revision
!            10.12.04  added more inputs and tests to be potentially included
!------------------------------------------------------------------------------

 use GeneralAuxType
 use modis_cloudstructure
 use mod06_run_settings

 implicit none
 
 private

 public :: cloudiness_test

 contains


!=====================================
subroutine cloudiness_test_alternate ( cloudmask,       &
                             process_summary, &
                             measurement,    &
                             reflectance_box, &
                             ir_cloudphase,   &
                              sunglint_dust,   &
                             cloudiness_degree, &
                             lowvariability_confidence_test, &
                             CSR_QA)            

!=====================================
!-----------------------------------------------------------------------
! !f90
!
! !Description:
!    This subroutine performs  spatial variability and reflectance tests 
!    in an attempt to eliminate sunglint pixels falsely flagged by cloud mask as cloud. 
!    Examples of this include bright ocean sunglint, smoke, dust and other heavy aerosols.
!
! !Input Parameters:
!     cloudmask       -- structure          -- all cloud mask tests
!     process_summary -- structure          -- process flags, surface type etc
!     measurements    -- 4 byte real vector -- reflectances/ radiances for all bands
!     reflectance_box -- 4 byte real array  -- .86um reflectances for 3x2 box
!     ir_cloudphase   -- structure          -- cloud phase result from IR tests (Baum et al)
!
! !Output Parameters:
!     sunglint_dust     -- logical -- true indicates the presence of sunglint/dust
!     cloudiness_degree -- integer -- degree of cloudiness: number of cloudy 250m pixels from 0 to 16
!
! !Revision History:
!     2004/10/23 mag
!     integrated into MOD_PR06OD
!       - added logical structures, variable types used by MOD_PR06OD
!       - cleaned up variable naming, code formatting, style.
!
!     2004/10/07 wind: Gala Wind
!     Revision 1.0 Initial revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!    Written by:
!    Gala Wind
!      L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      wind@climate.gsfc.nasa.gov
!
!    Mark Gray
!      L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      gray@climate.gsfc.nasa.gov
!
!
! !END
!
!-----------------------------------------------------------------------

  type(processflag), intent(in)            :: process_summary
  type(cloudmask_type), intent(in)         :: cloudmask
  type(cloudphase)                         :: ir_cloudphase
  real(single), dimension(set_number_of_bands), intent(in)   :: measurement
  real(single), dimension(:,:), intent(in) :: reflectance_box

  integer*1, intent(inout) :: CSR_QA
  logical,    intent(out):: lowvariability_confidence_test
  integer(integer_fourbyte),    intent(out):: cloudiness_degree
  logical,                      intent(out):: sunglint_dust
  
  real                 :: average, ratio_1, ratio_2, ratio_3, &
                          scaled_sd_086, standard_deviation_086
 real ::  r21_r138, r124_r21, r086_r124 

! initialize core outputs
  cloudiness_degree = 0
  sunglint_dust = .false.
  lowvariability_confidence_test = .false.

! don't run this code if there is no cloud observed. 
  if (.not. process_summary%cloudobserved) return


  r21_r138 = measurement(band_0213) - measurement(band_0138)
  r124_r21 = measurement(band_0124) - measurement(band_0213)
  r086_r124 = measurement(band_0086) - measurement(band_0124)



! compute the degree of cloudiness from the cloud mask bytes for this pixel
! of course we need to check (for each test) that the test was actually run
!
! the cloudiness degree is a number between 0 and 16. The number represents the number of
! cloudy pixels in the 250m box corresponding to a particular 1km pixel. The box is 4x4, so
! if the 1km pixel is truly cloud-filled, then the degree of cloudiness would be 16.

  if (cloudmask%visible_cloudtest_11 .and. &
      cloudmask%applied_visible_cloudtest_11) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_12 .and. &
      cloudmask%applied_visible_cloudtest_12) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_13 .and. &
      cloudmask%applied_visible_cloudtest_13) cloudiness_degree = cloudiness_degree+1            
  if (cloudmask%visible_cloudtest_14 .and. &
      cloudmask%applied_visible_cloudtest_14) cloudiness_degree = cloudiness_degree+1      
  
  if (cloudmask%visible_cloudtest_21 .and. &
      cloudmask%applied_visible_cloudtest_21) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_22 .and. &
      cloudmask%applied_visible_cloudtest_22) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_23 .and. &
      cloudmask%applied_visible_cloudtest_23) cloudiness_degree = cloudiness_degree+1            
  if (cloudmask%visible_cloudtest_24 .and. &
      cloudmask%applied_visible_cloudtest_24) cloudiness_degree = cloudiness_degree+1 
      
  if (cloudmask%visible_cloudtest_31 .and. &
      cloudmask%applied_visible_cloudtest_31) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_32 .and. &
      cloudmask%applied_visible_cloudtest_32) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_33 .and. &
      cloudmask%applied_visible_cloudtest_33) cloudiness_degree = cloudiness_degree+1            
  if (cloudmask%visible_cloudtest_34 .and. &
      cloudmask%applied_visible_cloudtest_34) cloudiness_degree = cloudiness_degree+1 

  if (cloudmask%visible_cloudtest_41 .and. &
      cloudmask%applied_visible_cloudtest_41) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_42 .and. &
      cloudmask%applied_visible_cloudtest_42) cloudiness_degree = cloudiness_degree+1
  if (cloudmask%visible_cloudtest_43 .and. &
      cloudmask%applied_visible_cloudtest_43) cloudiness_degree = cloudiness_degree+1            
  if (cloudmask%visible_cloudtest_44 .and. &
      cloudmask%applied_visible_cloudtest_44) cloudiness_degree = cloudiness_degree+1 

 

! now we can compute the spatial variability. 

! Early testing indicated that the 3x3 variability test is a more effective filter
! than the 2x2 variability test. But we have no choice in this matter, 
! so we have to resort to certain restrictions over what kind of pixels to process. 

! we will apply the test only over the desert surface
 
  if (process_summary%land_surface   .and. measurement(band_0065) > 0.45)  return 
  if (process_summary%desert_surface .and. measurement(band_0065) > 0.55)  return 
  if ( (process_summary%ocean_surface .or. process_summary%coastal_surface) &
                                     .and. measurement(band_0086) > 0.35) return 
! added don't bother for ice surface G.Wind 4.26.05
  if (process_summary%snowice_surface .and. measurement(band_0124) > 0.65) return

!  if (process_summary%desert_surface) then 

     average = array_mean(reflectance_box)
     standard_deviation_086 = standard_deviation(reflectance_box, average)
     if (average > 0.) then 
        scaled_sd_086 = standard_deviation_086 / average * 100.
     else
        scaled_sd_086 = 0.
     endif

     if (scaled_sd_086 >= 0. .and.  scaled_sd_086 <= 3.5 .and. &
         (.not. ir_cloudphase%icecloud) .and. measurement(band_0138) < 0.1 ) then 

        sunglint_dust = .true.
        CSR_QA = 2
        

! part III : spectral behavior
        if (measurement(band_0138) > 0.015) then
           
           if ( (1.0 > r21_r138 .and. r21_r138 > r124_r21 .and. r124_r21 > r086_r124) .or. &
                (r21_r138 < r124_r21 .and. r124_r21 < r086_r124 .and. r086_r124 < 1.0)) then 
              
              sunglint_dust = .true. ! RESTORE PIXEL TO CLEAR
              CSR_QA = 2
              
           else
              ! probably thin cirrus, don't restore
              sunglint_dust = .false.
              CSR_QA = 0
           endif
           
        else
           sunglint_dust = .TRUE. ! RESTORE PIXEL TO CLEAR
           CSR_QA = 2
        endif

     else
        sunglint_dust = .false.
        CSR_QA = 0
     endif


!  endif

! since the cloudiness degree is not used anywhere in the code, then we should just put it in here. 
  if (process_summary%ocean_surface .and. cloudiness_degree <= 8 .and. (.not. sunglint_dust) ) then 
     sunglint_dust = .true.
     CSR_QA = 3 ! restored by 250m
  endif


! maybe cloud and maybe not, but to play safe, keep it and make the QA less confident
! added the framework for absolute sdev tests G.Wind 4.26.05
  if (process_summary%ocean_surface .or. process_summary%coastal_surface) then 
     if ( ( (measurement(band_0086) > 0.25 .and. measurement(band_0086) <= 0.35) &
          .or. ( standard_deviation_086 > 0.003 .and. standard_deviation_086 < 0.003) ) &
          .and. sunglint_dust) then 

        lowvariability_confidence_test = .true.
        sunglint_dust = .false. 
        CSR_QA = 0
 
    endif
  endif


!  if (debugPRN) then 
!     print*, process_summary%ocean_surface, cloudiness_degree, sunglint_dust
!  endif

end subroutine cloudiness_test_alternate

!====================================================================
 subroutine cloudiness_test              &
                           ( cloudmask,        &
                             process_summary,  &
                             measurement,      &
                             reflectance_box,  &
                             ir_cloudphase,    &
                             not_cloud,        &
                             lowvariability_confidence_test, CSR_QA )
!====================================================================

! NOTES TO Gala: 
!
! This subroutine name should be changed to "cloudiness_test", while the above routine could be something
! like "cloudiness_test_alternate"; similar changes in modis_science_module are also needed, of course. 
! This routine has not for a long time been equivalent to the original Riedi work, but includes contributions
! from all of us and so is a misnomer. Furthermore, I have added absolute variability calculations for testing
! purposes only (presently commented out) so this routine now essentially contains the "alternate" structure anyway.

!-----------------------------------------------------------------------
! !f90
!
! !Description:
!    This subroutine performs  Clear Sky Restoral (CSR) tests 
!    in an attempt to eliminate sunglint or heavy aerosol pixels falsely flagged as
!    cloudy by the cloud mask. Examples of this include bright ocean sunglint, 
!    smoke, and dust.
!
! !Input Parameters:
!     cloudmask       -- structure          -- all cloud mask tests
!     process_summary -- structure          -- process flags, surface type etc
!     measurements    -- 4 byte real vector -- reflectances/ radiances for all bands
!     reflectance_box -- 4 byte real array  -- all reflectances for 2x2 box
!     ir_cloudphase   -- structure          -- cloud phase result from IR tests (Baum et al)
!
! !Output Parameters:
!     not_cloud     -- logical -- true indicates the presence of sunglint/dust or other non-cloud situation
!     lowvariability_confidence_test - Indicates went with a "cloudy" determination but are unsure so set confidence QA to a lower value
!     CSR_QA        --
!       CSR_flag_array=0 => cloudy, all CSR tests negative
!       CSR_flag_array=1 => edge test positive (determined in subroutine remove_edge_scenes.f90)
!       CSR_flag_array=2 => spatial, etc. tests positive
!       CSR_flag_array=3 => 250 m test positive
!
! !Revision History:
!
!     2005/09/14, S. Plantick: 
!        An error was found in the implementation of the CSR flow chart. The 250 cloudiness test
!        was not being used for clouds brighter than the reflectance threshold. This was fixed, along with:
!        (1) Additional comments and edits of existing comments.
!        (2) Local variable name changes and additional variables to implement correct 250 test logic.
!        (3) For testing/comparison purposes - the calculation of an absolute spatial variability metric.
!
!     2005, May-June: several iterations with a number of authors.
!
!     2005/02/08 wind
!        implemented Riedi algorithm as per the flow chart from Dec.10.2004
!     2004/10/23 mag
!     integrated into MOD_PR06OD
!       - added logical structures, variable types used by MOD_PR06OD
!       - cleaned up variable naming, code formatting, style.
!
!     2004/10/07 wind: Gala Wind
!     Revision 1.0 Initial revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!    Written by:
!    Gala Wind
!      L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      wind@climate.gsfc.nasa.gov
!
!    Mark Gray
!      L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      gray@climate.gsfc.nasa.gov
!
!
! !END
!
!-----------------------------------------------------------------------

  type(processflag), intent(in)              :: process_summary
  type(cloudmask_type), intent(in)           :: cloudmask
  type(cloudphase)                           :: ir_cloudphase
  real(single), dimension(:), intent(in)     :: measurement
  real(single), dimension(:,:,:), intent(in) :: reflectance_box
  integer*1, intent(inout)                   :: CSR_QA
  logical, intent(out)                       :: lowvariability_confidence_test
  logical, intent(out)                       :: not_cloud

! local variables
  logical                                    :: cloudy_250
  integer(integer_fourbyte)                  :: cloudiness_degree 
  real                                       :: average, r21_r138, r124_r21, r086_r124, &
                                                standard_deviation_val, Rbox(5,2), th_Ri, th_Ri_QA, &
                                                dispersion_val, dispersion_Ri, dispersion_Ri_QA, &
                                                Rmeas, Rth_ocean, Rth_land, Rth_desert, Rth_snowice


! Initialize outputs for no restoral, and 250m cloudiness variables
  CSR_QA = 0 ; not_cloud = .false. ; lowvariability_confidence_test = .false.
  cloudiness_degree = 0 ; cloudy_250 = .true.




! Con't run this code if there is no cloud observed.
  if (.not. process_summary%cloudobserved) return

  r21_r138  = measurement(band_0213) - measurement(band_0138)
  r124_r21  = measurement(band_0124) - measurement(band_0213)
  r086_r124 = measurement(band_0086) - measurement(band_0124)


! For use in Part V, compute the degree of cloudiness from the 250m cloud mask bytes for this pixel
!  of course we need to check (for each test) that the test was actually run
!
! The cloudiness degree is a number between 0 and 16, representing the number of
!  cloudy pixels in the 250m box corresponding to a particular 1km pixel. The box is 4x4, so
!  if the 1km pixel is completely overcaset, the degree of cloudiness would be 16.

  If (process_summary%ocean_surface) then

    if (cloudmask%visible_cloudtest_11 .and. &
        cloudmask%applied_visible_cloudtest_11) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_12 .and. &
        cloudmask%applied_visible_cloudtest_12) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_13 .and. &
        cloudmask%applied_visible_cloudtest_13) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_14 .and. &
        cloudmask%applied_visible_cloudtest_14) cloudiness_degree = cloudiness_degree+1
  
    if (cloudmask%visible_cloudtest_21 .and. &
        cloudmask%applied_visible_cloudtest_21) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_22 .and. &
        cloudmask%applied_visible_cloudtest_22) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_23 .and. &
        cloudmask%applied_visible_cloudtest_23) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_24 .and. &
        cloudmask%applied_visible_cloudtest_24) cloudiness_degree = cloudiness_degree+1
      
    if (cloudmask%visible_cloudtest_31 .and. &
        cloudmask%applied_visible_cloudtest_31) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_32 .and. &
        cloudmask%applied_visible_cloudtest_32) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_33 .and. &
        cloudmask%applied_visible_cloudtest_33) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_34 .and. &
        cloudmask%applied_visible_cloudtest_34) cloudiness_degree = cloudiness_degree+1

    if (cloudmask%visible_cloudtest_41 .and. &
        cloudmask%applied_visible_cloudtest_41) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_42 .and. &
        cloudmask%applied_visible_cloudtest_42) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_43 .and. &
        cloudmask%applied_visible_cloudtest_43) cloudiness_degree = cloudiness_degree+1
    if (cloudmask%visible_cloudtest_44 .and. &
        cloudmask%applied_visible_cloudtest_44) cloudiness_degree = cloudiness_degree+1

    if (cloudiness_degree <= 8) then
         cloudy_250 = .false.
    endif

  End if

! TO TURN OFF 250m TEST, UNCOMMENT THE FOLLOWING LINE:
! cloudy_250 = .true.

! ENTER PART I: PERFORM SPATIAL VARIABLILITY TESTS 

! original:
! Rth_ocean   = 0.35; Rth_land = 0.45; Rth_desert = NA;    Rth_snowice = 0.65

! threshold test 1: USED FOR COLLECTION 5 OPERATIONAL VERSION
  Rth_ocean   = 0.65; Rth_land = 0.65; Rth_desert = 0.65; Rth_snowice = 0.65

! threshold test 2:
! Rth_ocean   = 0.50; Rth_land = 0.50; Rth_desert = 0.55; Rth_snowice = 0.65

  If ( &
       (  process_summary%land_surface    .and. (measurement(band_0065) > Rth_land)    ) .OR. & 
       (  process_summary%desert_surface  .and. (measurement(band_0065) > Rth_desert)  ) .OR. &
       (  process_summary%snowice_surface .and. (measurement(band_0124) > Rth_snowice) ) .OR. &
       ( (process_summary%ocean_surface .or. process_summary%coastal_surface) &
                                     .and. (measurement(band_0086) > Rth_ocean) )   ) Then

     ! PART V (via PART I):
     if (.not. cloudy_250) then
        not_cloud = .true.
        CSR_QA = 3 ! restored by 250m test
     end If
     return
     
  End if
  
! Pick the reflectance box according to the surface type
  if (process_summary%land_surface .or. process_summary%desert_surface) then 
      Rbox = reflectance_box(:,band_0065,:)
      Rmeas = measurement(band_0065)
  endif
      
  if (process_summary%ocean_surface .or. process_summary%coastal_surface) then
      Rbox = reflectance_box(:,band_0086,:)
      Rmeas = measurement(band_0086)
  endif

  if (process_summary%snowice_surface) then
      Rbox = reflectance_box(:,band_0124,:)
      Rmeas = measurement(band_0124)
  endif

  average = array_mean(Rbox)
  standard_deviation_val = standard_deviation(Rbox, average)
  if(average > 0.001) then
     dispersion_val = 100 * standard_deviation_val/average
  else
     dispersion_val = -1.
  end if


! *** For spatial varobility clear sky restoral based on box absolute sdev: TO BE USED FOR COLLECTION 5 OPERATIONAL VERSION

! 1km spatial variability (sdev) thresholds

!  early version of sdev thresholds:
!  if (process_summary%ocean_surface) then 
!     th_Ri = 0.004
!     th_Ri_QA = 0.003
!  else
!     th_Ri = 0.005
!     th_Ri_QA = 0.004
!  endif

! sdev threshold test 1:
!  if (process_summary%ocean_surface) then 
!     th_Ri = 0.005
!     th_Ri_QA = 0.004
!  else
!     th_Ri = 0.006
!     th_Ri_QA = 0.005
!  endif

! sdev threshold test 2: TO BE USED FOR COLLECTION 5 OPERATIONAL VERSION
  if (process_summary%ocean_surface) then 
     th_Ri    = 0.006
     th_Ri_QA = 0.005
  else
     th_Ri = 0.007
     th_Ri_QA = 0.006
  endif

  IF (Rmeas > 0. .and. standard_deviation_val < th_Ri) THEN ! candidate for restoral

! *** For spatial variability clear sky restoral based on box dispersion: FOR TESTING PURPOSES

! dispersion = 100*sdev/avg thresholds:
!  if (process_summary%ocean_surface) then 
!     dispersion_Ri    = 3.5
!     dispersion_Ri_QA = 3.0
!  else
!     dispersion_Ri    = 4.0
!     dispersion_Ri_QA = 3.5
!  endif

! IF (Rmeas > 0. .and. dispersion_val > 0. .and. dispersion_val < dispersion_Ri) THEN ! candidate for restoral


! ENTER PART II: ALTITUDE INDICATOR

! sep, 17May: previous version used IR phase, this version is using "not ice " from the decision tree phase 
     If ( (.not. process_summary%icecloud) & !.and. measurement(band_0138) > -1.0 &
          .and. measurement(band_0138) < 0.1) Then ! candidate for restoral

! ENTER PART III: SPECTRAL BEHAVIOR

        if (measurement(band_0138) > 0.015) then
           
           if ( (1.0 > r21_r138 .and. r21_r138 > r124_r21 .and. r124_r21 > r086_r124) .or. &
                (r21_r138 < r124_r21 .and. r124_r21 < r086_r124 .and. r086_r124 < 1.0)) then 
              
              !  Restore pixel and thereby pass through PART IV (branch 2 in flow chart)
              not_cloud = .true.
              CSR_QA = 2
           else
              !  PART V (via PART III): probaby thin cirrus
              if (.not. cloudy_250) then
                 not_cloud = .true.
                 CSR_QA = 3 ! restored by 250m test
              end if
              return
           endif

        else
           !  Restore pixel and thereby pass through PART IV (branch 1 in flow chart)
           not_cloud = .TRUE.
           CSR_QA = 2
        endif

     Else

        !  PART V (via PART II): probably cirrus or high cloud
        if (.not. cloudy_250) then
           not_cloud = .true.
           CSR_QA = 3 ! restored by 250m test
        end if
        return
        
     End If

  ELSE

     !  PART V (via PART I):
     if (.not. cloudy_250) then
        not_cloud = .true.
        CSR_QA = 3 ! restored by 250m test
     end if
     return
     
  END IF

! ENTER PART IV: check variability again, if not_cloud is .true. but variability is still
!          fairly high then process as cloud but only lower the retrieval confidence QA

! *** For spatial varobility clear sky restoral based on box absolute sdev: TO BE USED FOR COLLECTION 5 OPERATIONAL VERSION
  if ( not_cloud .and. standard_deviation_val > th_Ri_QA ) then

! *** For spatial variability clear sky restoral based on box dispersion: FOR TESTING PURPOSES
! if ( not_cloud .and. dispersion_val > dispersion_Ri_QA ) then

         not_cloud = .FALSE.                      ! RESET PIXEL TO CLOUDY
         CSR_QA = 0
         lowvariability_confidence_test = .TRUE.  ! Indicates that we must set confidence QA to a lower value
                                                  ! Info is propagated using processflag by the "cloudsuspicious" 
                                                  ! logical flag
         ! PART V (via PART IV):
         if (.not. cloudy_250) then
            not_cloud = .true.
            CSR_QA = 3 ! restored by 250m test
         end If

  endif

end subroutine cloudiness_test

function array_mean(array)
  real*4 :: array_mean, temp
  real*4, dimension(:,:), intent(in) :: array
  integer :: N1, N2, i, j, cnt

  N1 = size(array, 1)
  N2 = size(array, 2)

  temp = 0.
  cnt = 0
  do i=1, N1
     do j=1, N2
        if (array(i,j) > 0.) then 
           temp = temp + array(i,j)
           cnt = cnt + 1
        endif 
     end do
  end do

  if (cnt > 0) then 
     array_mean = temp / real(cnt)
  else
     array_mean = 0.
  endif 

end function array_mean

!=====================================
function standard_deviation(array, average)
!=====================================
!-----------------------------------------------------------------------
! !f90
!
! !Description:
!       This function computes standard deviation of a 2D array
!
! !Input Parameters:
!     array -- real*4, dimension(:,:) -- an array of data
!     average -- real*4 -- mean value of the array
!
! !Output Parameters:
!     standard_deviation -- real*4 -- the standard deviation value
!
! !Revision History:
!     2004/06/02 wind: Gala Wind
!     Revision 1.0 clean-up
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!      Written by:
!      Gala Wind
!      L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      wind@climate.gsfc.nasa.gov
!
! !Design Notes:
!       NONE
!
! !END
!
!-----------------------------------------------------------------------
  real, intent(in) :: average
  real, dimension(:,:), intent(in) :: array
  real :: standard_deviation, temp
  integer :: i,j, cnt, N1, N2

  N1 = size(array, 1)
  N2 = size(array, 2)
  temp = 0.
  cnt = 0
  do i=1, N1
    do j=1, N2
      if ( array(i,j) > 0. ) then 
        temp = temp + (average - array(i,j))**2
        cnt = cnt + 1
      endif
    end do
  end do

  if ( (cnt-1) > 0) then 
     standard_deviation= sqrt ( temp / real(cnt - 1) )
  else
     standard_deviation = 0.
  endif

end function standard_deviation


end module clear_sky_restoral

