module get_retrieval_uncertainty

 implicit none
 private

 public ::  getUncertainties, getradiibounds

 contains


subroutine getUncertainties  (thickness,               &
                              re,                      &
                              water_path,              &
                              phase,                   &
                              tauRegimeThreshold,      &
                              R1R2wavelengthIdx,       &
                              surface_albedo,          &
                              transmittance_w1,        &
                              transmittance_w2,        &
                              delta_transmittance_w1,  &
                              delta_transmittance_w2,  &
                              transmittance_stddev_w1, &
                              transmittance_stddev_w2, &
                              uTau,uRe,uWaterPath)
!-----------------------------------------------------------------------
! !f90
!
! !Description:
!     This subroutine computes the final uncertainties in thickness, re and water vapor
!
! !Input Parameters:
!     NONE
!
! !Output Parameters:
!     NONE
!
! !Revision History:
!     2004/06/01 bwind: Brad Wind
!     Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!    Written by:
!    Bradley P. Wind
!    L3 GSI
!    Code 913, NASA/GSFC
!    Greenbelt, MD 20771
!    bwind@climate.gsfc.nasa.gov
! 
!    Mark Gray
!    L3-GSI
!    Climate and Radiation Branch, Code 913
!    NASA/GSFC
!    Greenbelt MD 20771
!    gray@climate.gsfc.nasa.gov
!
! !Design Notes:
!   NONE
!
! !END
!
!-----------------------------------------------------------------------

! 
! S. Platnick, 2-23-05: 
!
!  Wrote variance calculations in terms of matrix formulation
!  Added covariance(tau,re) to water path uncertainty
!

  use GeneralAuxType
  use science_parameters

  implicit none

  real(single), intent(in) :: thickness, re, water_path,            &
                              surface_albedo(2),     &
                              tauRegimeThreshold(2), &
                              transmittance_w1,transmittance_w2, &
                              delta_transmittance_w1,delta_transmittance_w2, &
                              transmittance_stddev_w1, transmittance_stddev_w2   

   
  integer, intent(in)       :: R1R2wavelengthIdx(2)
  character(10), intent(in) :: phase
  real(single), intent(out) :: uTau, uRe,uWaterPath

  real(single) :: meanCloudTopReflW1, meanCloudTopReflW2
  real(single) :: partialDerivTauWrtR1AtConstR2, partialDerivTauWrtR2AtConstR1 
  real(single) :: partialDerivReWrtR2AtConstR1 , partialDerivReWrtR1AtConstR2
  real(single) :: meanDeltaR1trans, meanDeltaR2trans
  real(single) :: deltacloudtopmean(2), density

  real(single) :: correlation_trans, correlation_WP

! declare matrices
  real(single), dimension(2,2) :: S, S_transpose, retrieval_covariance, dummy,  &
  								  refl_cov_total, refl_cov_sfc_albedo, &
  								  refl_cov_trans, refl_cov_trans_PW, refl_cov_trans_table, &
  								  refl_cov_calibration




! -----------------------------------------------------------------------
! Calculate change in cloud top reflectance due to sfc albedo uncertainty
! -----------------------------------------------------------------------

    call getctref_albedodiff(tauRegimeThreshold, re, thickness, phase,     &
                           R1R2wavelengthIdx,  &
                           surface_albedo,     &
                           deltacloudtopmean, meanCloudTopReflW1, meanCloudTopReflW2)

! -----------------------------------------------------------------------
! Calculate sensitivity derivatives and generate the sensitivity matrix S    
! -----------------------------------------------------------------------

    call sensitivityPartialDerivatives( tauRegimeThreshold, R1R2wavelengthIdx, &
                                      re, thickness, phase,  &
                                      surface_albedo, &
                                      partialDerivTauWrtR1AtConstR2, &
                                      partialDerivTauWrtR2AtConstR1, &
                                      partialDerivReWrtR1AtConstR2,  &
                                      partialDerivReWrtR2AtConstR1 )

    S(1,1) = partialDerivTauWrtR1AtConstR2
    S(1,2) = partialDerivTauWrtR2AtConstR1 
    S(2,1) = partialDerivReWrtR1AtConstR2
    S(2,2) = partialDerivReWrtR2AtConstR1

	S_transpose = Transpose(S)

! ----------------------------------------------------------------------
! Calculate the uncertainties attributable to each source of uncertainty 
! ----------------------------------------------------------------------

! (1) Reflectance covariance matrix due to Surface albedo uncertainty


	refl_cov_sfc_albedo(1,1) = deltacloudtopmean(1)**2
	refl_cov_sfc_albedo(2,2) = deltacloudtopmean(2)**2
	refl_cov_sfc_albedo(1,2) = 0.
	refl_cov_sfc_albedo(2,1) = refl_cov_sfc_albedo(1,2)

! (2) Reflectance covariance matrix due to atmospheric transmittance uncertainty

!  2a. Calculate covariance matrix due to percipitable water (PW) first:

!  deltaR due to PW uncertainty only:
   meanDeltaR1trans =  meanCloudTopReflW1 * abs(delta_transmittance_w1)/transmittance_w1
   meanDeltaR2trans =  meanCloudTopReflW2 * abs(delta_transmittance_w2)/transmittance_w2

   correlation_trans = 1.0
   refl_cov_trans_PW(1,1) = meanDeltaR1trans**2
   refl_cov_trans_PW(2,2) = meanDeltaR2trans**2
   refl_cov_trans_PW(1,2) = abs(meanDeltaR1trans*meanDeltaR2trans)*correlation_trans
   refl_cov_trans_PW(2,1) = refl_cov_trans_PW(1,2)

                
!  2b. Calculate covariance matrix due to table uncertainty next:

!  deltaR due to table uncertainty only:
   meanDeltaR1trans =  meanCloudTopReflW1 * transmittance_stddev_w1/transmittance_w1
   meanDeltaR2trans =  meanCloudTopReflW2 * transmittance_stddev_w2/transmittance_w2

   refl_cov_trans_table(1,1) = meanDeltaR1trans**2
   refl_cov_trans_table(2,2) = meanDeltaR2trans**2
   refl_cov_trans_table(1,2) = 0.
   refl_cov_trans_table(2,1) = refl_cov_trans_table(1,2)

!  2c. Combined covariance matrix:

   refl_cov_trans = refl_cov_trans_PW + refl_cov_trans_table

! (3) Reflectance covariance matrix due to calibration uncertainty

!  sep: addition to see effect of increasing relative measurement error:
!  refl_cov_calibration(1,1) = (2 * measurement_error * meanCloudTopReflW1)**2
!  refl_cov_calibration(2,2) = (2 * measurement_error * meanCloudTopReflW2)**2

   refl_cov_calibration(1,1) = (measurement_error * meanCloudTopReflW1)**2
   refl_cov_calibration(2,2) = (measurement_error * meanCloudTopReflW2)**2
   refl_cov_calibration(1,2) = 0.
   refl_cov_calibration(2,1) = refl_cov_calibration(1,2)

! Total tau & re covariance matrix

   refl_cov_total = refl_cov_sfc_albedo + refl_cov_trans + refl_cov_calibration

! ------------------------------------------------------------------------------
! Calculate retrieval covariance matrix = S * retrieval_covariance * S_transpose
! ------------------------------------------------------------------------------

   dummy = MATMUL(refl_cov_total, S_transpose)
   retrieval_covariance = MATMUL(S, dummy)

! ---------------------
! Tau, re uncertainties
! ---------------------

   uTau = SQRT( retrieval_covariance(1,1) ) 
   uRe  = SQRT( retrieval_covariance(2,2) ) 

! ----------------------
! Water path uncertainty
! ----------------------


   if ((100.*utau/thickness) >= 200. .OR. (100.*ure/re) >= 200.  ) then
        uWaterPath = 200. 

   else
      if (phase == 'water') then
         density = liquid_water_density
      else
         density = ice_water_density
      endif

      uWaterPath = (re**2)*(uTau**2) + (thickness**2)*(uRe**2) + (uTau**2)*(uRe**2) + &
                    2*retrieval_covariance(1,2)*uTau*uRe + retrieval_covariance(1,2)**2

      if (uWaterPath < 0.) then
         uWaterPath = 200.
      else
         uWaterPath= sqrt ( density**2 * uWaterPath )
      end if

   endif

! ----------------------
! Relative uncertainties
! ----------------------

   utau = 100.*utau/thickness
   ure  = 100.*ure/re
   uwaterpath = 100.*uwaterpath/water_path

! ------------------------------------------------------
! Limit answer to a maximum of 200% relative uncertainty
! ------------------------------------------------------

!  sep: Was told by mag that max value should be 200 (or 255) 
!       because of the scaling value used in the filespec

   if (utau > 200.  ) utau = 200.
   if (ure > 200. ) ure = 200.
   if (uWaterPath > 200.) uWaterPath = 200.

end subroutine getUncertainties

subroutine getctref_albedodiff(tauRegimeThreshold, &
                              radius,      &
                             optical_thickness, &
                             phase,             &
                             wave_index,        &
                             surface_albedo,    &
                             reflectancediff, meancloudtopreflw1, meancloudtopreflw2)
  use GeneralAuxType
  use libraryinterpolates
  use libraryarrays
  use science_parameters
  implicit none

  real(single), intent(in)  :: radius, tauRegimeThreshold(2)
  integer, intent(in)       :: wave_index(2)
  real(single), intent(in)  :: optical_thickness, surface_albedo(2)
  character(10),intent(in)  :: phase
  real(single), intent(out) :: reflectancediff(2), meancloudtopreflw1, meancloudtopreflw2

  integer :: radii_size , radius_upperbound, radius_lowerbound, radius_index, i,j, vectorsize
  real(single) :: reflectance_lower, reflectance_middle, reflectance_upper
  real(single) :: opticalthick_vector(6)
  integer :: dummy
  reflectance_upper = 0.
  reflectance_lower = 0.
  reflectance_middle = 0.


  if (phase == 'water') then
    call getclosestradius(number_waterradii,  &
                   water_radii,radius,  &
                   i)

  else
    call getclosestradius(number_iceradii,  &
                        ice_radii,radius,  &
                        i)
  endif
  if (phase == 'water') then
   do j = 1,2
     if (optical_thickness < tauRegimeThreshold(j)) then
      opticalthick_vector(1) = 0.
      opticalthick_vector(2:4) = library_taus(1:3)
      opticalthick_vector(5)= 1.8
      opticalthick_vector(6) = 2.
      opticalthick_vector = opticalthick_vector/(1-asymmetry_water(wave_index(j),i))
      call nonasymptotic_albedovar(opticalthick_vector, &
                              optical_thickness,                         &
                              asymmetry_water(wave_index(j),i),                   &
                              singlescattering_water(wave_index(j),i),            &
                              ak_water(wave_index(j),i),                          &
                              el_water(wave_index(j),i),                          &
                              em_water(wave_index(j),i),                          &
                              en_water(wave_index(j),i),                          &
                              spherical_albedo_water(:,wave_index(j),i),&
                              int_reflectance_water(:,wave_index(j),i), &
                              int_escapefuncwater_solar(wave_index(j),i),         &
                              int_escapefuncwater_sensor(wave_index(j),i),        &
                              int_fluxdownwater_sensor(:,wave_index(j),i), &
                              int_fluxdownwater_solar(:,wave_index(j),i), & 
                              surface_albedo(j),                              &
                              reflectance_lower, &
                              reflectance_middle, &
                              reflectance_upper)
 
     else
       call asymptotic_albedovar(optical_thickness,                         &
                              asymmetry_water(wave_index(j),i),                   &
                              singlescattering_water(wave_index(j),i),            &
                              ak_water(wave_index(j),i),                          &
                              el_water(wave_index(j),i),                          &
                              em_water(wave_index(j),i),                          &
                              en_water(wave_index(j),i),                          &
                              spherical_albedo_water(number_taus,wave_index(j),i),&
                              int_reflectance_water(number_taus,wave_index(j),i), &
                              int_escapefuncwater_solar(wave_index(j),i),         &
                              int_escapefuncwater_sensor(wave_index(j),i),        &
                              surface_albedo(j),                              &
                              reflectance_lower, &
                              reflectance_middle, &
                              reflectance_upper)
    endif
    reflectancediff(j) = (abs(reflectance_lower - reflectance_middle) +  &
                         abs(reflectance_upper - reflectance_middle))/2.     
    if(j .eq. 1) then
       meancloudtopreflw1 = reflectance_middle
    else
       meancloudtopreflw2 = reflectance_middle
    endif
   enddo 
  else 
   do j = 1,2
     if (optical_thickness < tauRegimeThreshold(j)) then
      opticalthick_vector(1) = 0.
      opticalthick_vector(2:4) = library_taus(1:3)
      opticalthick_vector(5)= 1.8
      opticalthick_vector(6) = 2.
      opticalthick_vector = opticalthick_vector/(1-asymmetry_ice(wave_index(j),i))
      call nonasymptotic_albedovar(opticalthick_vector, &
                              optical_thickness,                         &
                              asymmetry_ice(wave_index(j),i),                   &
                              singlescattering_ice(wave_index(j),i),            &
                              ak_ice(wave_index(j),i),                          &
                              el_ice(wave_index(j),i),                          &
                              em_ice(wave_index(j),i),                          &
                              en_ice(wave_index(j),i),                          &
                              spherical_albedo_ice(:,wave_index(j),i),&
                              int_reflectance_ice(:,wave_index(j),i), &
                              int_escapefuncice_solar(wave_index(j),i),         &
                              int_escapefuncice_sensor(wave_index(j),i),        &
                              int_fluxdownice_sensor(:,wave_index(j),i), &
                              int_fluxdownice_solar(:,wave_index(j),i), &
                              surface_albedo(j),                              &
                              reflectance_lower, &
                              reflectance_middle, &
                              reflectance_upper)

     else
       call asymptotic_albedovar(optical_thickness,                         &
                            asymmetry_ice(wave_index(j),i),                   &
                            singlescattering_ice(wave_index(j),i),            &
                            ak_ice(wave_index(j),i),                          &
                            el_ice(wave_index(j),i),                          &
                            em_ice(wave_index(j),i),                          &
                            en_ice(wave_index(j),i),                          &
                            spherical_albedo_ice(number_taus,wave_index(j),i),&
                            int_reflectance_ice(number_taus,wave_index(j),i), &
                            int_escapefuncice_solar(wave_index(j),i),         &
                            int_escapefuncice_sensor(wave_index(j),i),        &
                            surface_albedo(j),                            &
                            reflectance_lower, &
                            reflectance_middle, &
                            reflectance_upper)
     endif
     reflectancediff(j) = (abs(reflectance_lower - reflectance_middle) + &
                           abs(reflectance_upper - reflectance_middle))/2. 
     if(j .eq. 1) then
        meancloudtopreflw1 = reflectance_middle
     else
        meancloudtopreflw2 = reflectance_middle
     endif
    enddo
   endif

end subroutine getctref_albedodiff

subroutine getclosestradius(numberradii, radiidata, radius, index)

  integer, intent(in):: numberradii
  real, intent(in)   :: radiidata(numberradii), radius
  integer, intent(out):: index
  integer :: index1(1)
  index1 = MINLOC(abs(radiidata-radius))
  index=index1(1)

end subroutine getclosestradius
subroutine nonasymptotic_calcu(library_taus,tc,g,w,ak,el,em,en, &
                            sfr, rf, fk0,fk1,ft1,ft0, &
                            albedo,rf_calculated)
  use GeneralAuxType
  use spline_module
  use modis_numerical_module

  implicit none

  real(single), intent(in)    :: library_taus(6),tc,albedo
  real(single), intent(in)    :: g,w,ak,el,em,en, fk1, fk0
  real(single), intent(in)    :: rf(4), ft0(4), ft1(4), sfr(4)
  real(single), intent(out)   :: rf_calculated

  real(single) :: reflectance_vector(6), yy2(6)
  integer      :: total_taus, points

  total_taus = 4
  reflectance_vector(1) =  albedo

  reflectance_vector(2:4) = rf(1:3) + ft1(1:3) * ft0(1:3) * albedo /( 1. - albedo*sfr(1:3))

  call asymptotic_calcu(library_taus(5),  &
                 g,w,      &
                 ak,el,    &
                 em,en,    &
                 sfr(total_taus),  &
                 rf(total_taus),fk1,       &
                 fk0,albedo,reflectance_vector(5))

  call asymptotic_calcu(library_taus(6),  &
                 g,w,      &
                 ak,el,    &
                 em,en,    &
                 sfr(total_taus),  &
                 rf(total_taus),fk1,       &
                 fk0,albedo,reflectance_vector(6))


!     computing reflectance function at
!     tau(lambda) using spline fitting.
      points = 6
      call spline(points, library_taus,reflectance_vector,yy2)
      rf_calculated = splint(points, tc ,library_taus,reflectance_vector,yy2)

end subroutine nonasymptotic_calcu
subroutine nonasymptotic_albedovar(library_taus,tc,g,w,ak,el,em,en, &
                            sfr, rf, fk0,fk1,ft1,ft0, &
                            albedo,rf_lower, rf_middle, rf_upper)
  use GeneralAuxType
  use science_parameters
  implicit none

  real(single), intent(in)    :: library_taus(6),tc,albedo
  real(single), intent(in)    :: g,w,ak,el,em,en, fk1, fk0
  real(single), intent(in)    :: rf(4), ft0(4), ft1(4), sfr(4)
  real(single), intent(out)   :: rf_middle, rf_lower, rf_upper
  real(single) :: albedoUpper

  call nonasymptotic_calcu(library_taus,tc,g,w,ak,el,em,en, &
                            sfr, rf, fk0,fk1,ft1,ft0, &
                            albedo,rf_middle)

  call nonasymptotic_calcu(library_taus,tc,g,w,ak,el,em,en, &
                            sfr, rf, fk0,fk1,ft1,ft0, &
                            (albedo * (1.0 - albedo_error)),rf_lower)

  albedoUpper = albedo * (1.0 + albedo_error)
  if(albedoUpper .gt. 0.99) albedoUpper = 0.99
  call nonasymptotic_calcu(library_taus,tc,g,w,ak,el,em,en, &
                            sfr, rf, fk0,fk1,ft1,ft0, &
                            albedoUpper,rf_upper)

end subroutine nonasymptotic_albedovar
subroutine asymptotic_albedovar(tc,g,w,ak,el,em,en, &
                            sfr, rfi, fk1,fk0,ag,rf_lower, rf_middle, rf_upper)
  use GeneralAuxType
  use science_parameters
  implicit none

  real(single), intent(in)    :: tc,ag, rfi
  real(single), intent(in)    :: g,w,ak,el,em,en,sfr, fk1, fk0
  real(single), intent(out)   :: rf_middle, rf_lower, rf_upper
  real(single)                :: agU
  real(single)                :: q1, rf0, ft1, sfr1, x, fkk, exc, ft0
  parameter (q1=0.712)

  fkk = fk1*fk0

  if (w > 0.999991) then

!    conservative case
     x = 4.0/3.0/ ((1.-g)*tc+2.0*q1)      
     ft1 = fk1*x                          !this is eq.(17)
     ft0 = fk0*x                          !this is eq.(17)
     sfr1 = 1.0- x                        !this is eq.(19)
     rf0 = rfi - fkk*x                    !this is eq.(4)
  else

!    non-conservative case
     exc = ak*tc
     if (exc <= 50.0) then
        exc = exp(-exc)
     else
        exc = 0.
     end if

     x = em*exc/ (1.0 - (el*exc)**2)      
     ft1 = en*fk1*x                       !this is eq.(20)
     ft0 = en*fk0*x                       !this is eq.(20)
     sfr1 = sfr - el*en**2*exc*x          !this is eq.(22)
     rf0 = rfi - el*fkk*exc*x             !this is eq.(6)
  end if

  rf_lower = rf0 + ft1*ft0*ag*(1.-albedo_error)/ (1.-ag*sfr1*(1.-albedo_error))    !this is eq.(11)
  rf_middle = rf0 + ft1*ft0*ag/ (1.-ag*sfr1)
  agU = ag*(1.+albedo_error)
  if(agU .gt. 0.99) agU = 0.99
  rf_upper= rf0 + ft1*ft0*agU/ (1.-agU*sfr1)
end subroutine asymptotic_albedovar
  subroutine sensitivityPartialDerivatives(tauRegimeThreshold, &
                                           R1R2wavelengthIdx,&
                                           re, tau,  &
                                           phase, surface_albedo, &
                                           partialDerivTauWrtR1AtConstR2,&
                                           partialDerivTauWrtR2AtConstR1,&
                                           partialDerivReWrtR1AtConstR2, &
                                           partialDerivReWrtR2AtConstR1)
!-----------------------------------------------------------------------
! !f90
!
! !Description:
!   This subroutine computes the partial derivatives of tau and re with 
!   respect to reflectance in one band as the other one is held constant
!
!
! !Input Parameters:
!
! !Output Parameters:
!   NONE
!
! !Revision History:
!   2004/05/28 bwind: Brad Wind
!   Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!   Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!   Written by:
!   Bradley P. Wind
!   L3 GSI
!   Code 913, NASA/GSFC
!   Greenbelt, MD 20771
!   bwind@climate.gsfc.nasa.gov
! 
!   Mark Gray
!   L3-GSI
!   Climate and Radiation Branch, Code 913
!   NASA/GSFC
!   Greenbelt MD 20771
!   gray@climate.gsfc.nasa.gov
!
! !Design Notes:
!   NONE
!
! !END
!
!-----------------------------------------------------------------------
  use GeneralAuxType
  use science_parameters
  use libraryarrays
  implicit none

  integer, intent(in)      :: R1R2wavelengthIdx(2)
  real(single), intent(in) :: re, tau
  real(single), intent(in) :: surface_albedo(2)
  real(single), intent(in) :: tauRegimeThreshold(2)
  character(10), intent(in):: phase
  real(single), intent(out) :: partialDerivTauWrtR1AtConstR2, &
                               partialDerivTauWrtR2AtConstR1, &
                               partialDerivReWrtR1AtConstR2,  &
                               partialDerivReWrtR2AtConstR1

  real(single) :: partialDerivR1wrtTauAtConstRe, &
                  partialDerivR2wrtTauAtConstRe, &
                  partialDerivR1wrtReAtConstTau, &
                  partialDerivR2wrtReAtConstTau

  integer :: i
  integer :: wrtParam

  real(single) :: partialDerivativesTau, partialDerivativesRe
  real(single) :: upperLimit, lowerLimit,abscissaintervalstartingpoint
  real(single) :: halfAbscissaInterval
  real(single), parameter :: halfAbscissaIntervalTau = 1., halfAbscissaIntervalRe = 1.
  real(single), parameter :: tauUpperLimit = 1000.
  real(single), parameter :: tauLowerLimit = 0.

  integer, parameter :: tauParamPosition = 4
  integer, parameter :: reParamPosition = 5

! for tau, determine abscissa interval taking into account the 
! extrema and potentially rough transition point(s)

  halfAbscissaInterval = halfAbscissaIntervalTau
  abscissaIntervalStartingPoint = tau
  wrtParam = tauParamPosition


! loop over wavelengths
  do i =1, 2
!   for tau must determine which regime
    if (tau > tauRegimeThreshold(i) ) then
      upperLimit=tauUpperLimit
      lowerLimit=tauRegimeThreshold(i)
    else
      upperLimit=tauRegimeThreshold(i)
      lowerLimit=tauLowerLimit
    endif

!   Compute derivative for dR1/dT )re, and dR2/dT )re 
    call reflPartialDerivative(tauRegimeThreshold(i), &
                               upperLimit, lowerLimit, &
                               halfAbscissaInterval, & 
                               abscissaIntervalStartingPoint, &
                               re, tau,phase,  &
                               surface_albedo(i), &
                               wrtParam,                                &
                               R1R2wavelengthIdx(i), &
                               partialDerivativesTau)
    if (i .eq. 1) then
!     dR1/dT | re
      partialDerivR1wrtTauAtConstRe = partialDerivativesTau
    else
!     dR2/dT | re 
      partialDerivR2wrtTauAtConstRe = partialDerivativesTau
    endif
  end do

! for re, determine abscissa interval taking into account the extrema
  halfAbscissaInterval = halfAbscissaIntervalRe
  abscissaIntervalStartingPoint = re

! for re must determine which phase 
  if(phase == 'water') then
    upperLimit = water_radii(number_waterradii)
    lowerLimit = water_radii(1)
  else
    upperLimit = ice_radii(number_iceradii)
    lowerLimit = ice_radii(1)
  endif

! Compute derivative for dR1/dre )T, and dR2/dre)T
  wrtParam = reParamPosition

  do i = 1, 2
   call reflPartialDerivative(tauRegimeThreshold(i),   &
                              upperLimit, lowerLimit,  &
                              halfAbscissaInterval,    &
                              abscissaIntervalStartingPoint, &
                              re, tau,phase,           &
                              surface_albedo(i),       &
                              wrtParam,                &
                              R1R2wavelengthIdx(i),    &
                              partialDerivativesRe)

    if (i == 1 ) then
!     dR1 / dre | T
      partialDerivR1wrtReAtConstTau = partialDerivativesRe
    else
!     dR2 / dre | T
      partialDerivR2wrtReAtConstTau = partialDerivativesRe
    endif
  enddo

! head-off divide by 0
  if(abs(partialDerivR1wrtTauAtConstRe) .lt. epsilon(partialDerivR1wrtTauAtConstRe)) partialDerivR1wrtTauAtConstRe = &
     epsilon(partialDerivR1wrtTauAtConstRe)
  if(abs(partialDerivR2wrtTauAtConstRe) .lt. epsilon(partialDerivR2wrtTauAtConstRe)) partialDerivR2wrtTauAtConstRe = &
     epsilon(partialDerivR2wrtTauAtConstRe)
  if(abs(partialDerivR1wrtReAtConstTau) .lt. epsilon(partialDerivR1wrtReAtConstTau)) partialDerivR1wrtReAtConstTau = &
     epsilon(partialDerivR1wrtReAtConstTau)
  if(abs(partialDerivR2wrtReAtConstTau) .lt. epsilon(partialDerivR2wrtReAtConstTau)) partialDerivR2wrtReAtConstTau = &
     epsilon(partialDerivR2wrtReAtConstTau)

! Compute the derivatives
  partialDerivTauWrtR1AtConstR2 = 1. / &
                                  (partialDerivR1wrtTauAtConstRe - &
                                   partialDerivR1wrtReAtConstTau * &
                                   (partialDerivR2wrtTauAtConstRe/partialDerivR2wrtReAtConstTau) )
    
  partialDerivTauWrtR2AtConstR1 = 1. / &
                                   (partialDerivR2wrtTauAtConstRe - &
                                    partialDerivR2wrtReAtConstTau * &
                                    (partialDerivR1wrtTauAtConstRe/partialDerivR1wrtReAtConstTau) )
   
  partialDerivReWrtR1AtConstR2 = 1. / &
                                  (partialDerivR1wrtReAtConstTau - &
                                   partialDerivR1wrtTauAtConstRe * &
                                   (partialDerivR2wrtReAtConstTau/partialDerivR2wrtTauAtConstRe) )
    
  partialDerivReWrtR2AtConstR1 = 1. / &
                                  (partialDerivR2wrtReAtConstTau - &
                                   partialDerivR2wrtTauAtConstRe * &
                                   (partialDerivR1wrtReAtConstTau/partialDerivR1wrtTauAtConstRe) )

if(debugPRN) then
print*, 'partialDerivTauWrtR1AtConstR2 = ', partialDerivTauWrtR1AtConstR2
print*, 'partialDerivTauWrtR2AtConstR1 = ', partialDerivTauWrtR2AtConstR1
print*, 'partialDerivReWrtR1AtConstR2 = ', partialDerivReWrtR1AtConstR2
print*, 'partialDerivReWrtR2AtConstR1 = ', partialDerivReWrtR2AtConstR1
endif

end subroutine sensitivityPartialDerivatives
  subroutine reflPartialDerivative(tauRegimeThreshold,upperLimit, lowerLimit, &
                                   halfAbscissaInterval, abscissaIntervalStartingPoint, &
                                   re, tau, phase, &
                                   surface_albedo, &
                                   wrtParam, wavelengthIdx, &
                                   partialDerivative)
!-----------------------------------------------------------------------
! !f90
!
! !Description:
!     This subroutine computes a partial derivative of tau or re using the
!     central difference method. Special cases such as crossing to asymptotic regime
!     and being at libraries' ends are handled also. 
!
! !Input Parameters:
!     wrtParam -- integer -- flag controlling whether or not to compute the derivatives
!
! !Output Parameters:
!     partialDerivatives -- real*4, dimension(:) -- array of partial derivatives
!
! !Revision History:
!     2004/05/28 bwind: Brad Wind
!     Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!    Written by:
!    Bradley P. Wind
!    L3 GSI
!    Code 913, NASA/GSFC
!    Greenbelt, MD 20771
!    bwind@climate.gsfc.nasa.gov
!
!    Mark Gray
!    L3-GSI
!    Climate and Radiation Branch, Code 913
!    NASA/GSFC
!    Greenbelt MD 20771
!    gray@climate.gsfc.nasa.gov
!
! !Design Notes:
!      NONE
!
! !END
!
!-----------------------------------------------------------------------
  use GeneralAuxType
  use science_parameters
  implicit none

  integer, intent(in)       :: wrtParam, wavelengthIdx
  real(single), intent(in)  :: re, tau, tauRegimeThreshold
  real(single), intent(in)  :: surface_albedo
  real(single), intent(in)  :: upperLimit, lowerLimit
  character(10),intent(in)  :: phase
  real(single), intent(out) :: partialDerivative

  real(single) :: Rb, Ra
  real(single) :: tmpTau, tmpRe
  
  real(single) :: halfAbscissaInterval, abscissaIntervalStartingPoint
  real(single) :: abscissaIntervalLower, abscissaIntervalUpper
  integer, parameter :: tauParamPosition = 4
  integer, parameter :: reParamPosition = 5

  integer :: idxJ
  call abscissaInterval(upperLimit, lowerLimit, &
                        halfAbscissaInterval, abscissaIntervalStartingPoint, &
                        abscissaIntervalLower, abscissaIntervalUpper)

  Rb = 0.
  Ra = 0.

! get reflectances
  select case (wrtParam)
    case(tauParamPosition)
!     in tau case
!     ...abscissaIntervalUpper/Lower are upper and lower optical thicknesses

      call getctref_constradius(tauRegimeThreshold,         &
                                abscissaIntervalUpper,      &
                                abscissaIntervalLower,      &
                                re, phase,                  &
                                surface_albedo,             &
                                wavelengthIdx, Rb, Ra)
    case(reParamPosition)
!     in re case
!     ...abscissaIntervalUpper/Lower are upper and lower effective radii

        call getCTref_consttau(tauRegimeThreshold, &
                               abscissaIntervalUpper,      &
                               abscissaIntervalLower,      &
                               tau, phase,                  &
                               surface_albedo,             &
                               wavelengthIdx, Rb, Ra)      
    case default
  end select
  
  ! perform the division

  partialDerivative = (Rb - Ra) 
  partialDerivative = partialDerivative / (abscissaIntervalUpper - abscissaIntervalLower)
  end subroutine reflPartialDerivative


subroutine abscissainterval(upperLimit, lowerLimit, &
                            halfAbscissaInterval, abscissaIntervalStartingPoint, &
                            abscissaIntervalLower, abscissaIntervalUpper)

!-----------------------------------------------------------------------
! !f90
!
! !Description:
!   This subroutine figures out the interval for use in the
!   central difference method. Special cases such as crossing to asymptotic regime
!   and being at libraries' ends are handled also. 
!
! !Input Parameters:
!   NONE
!
! !Output Parameters:
!   NONE
!
! !Revision History:
!   2004/05/28 bwind: Brad Wind
!   Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!   Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!   Written by:
!   Bradley P. Wind
!   L3 GSI
!   Code 913, NASA/GSFC
!   Greenbelt, MD 20771
!   bwind@climate.gsfc.nasa.gov
!
!   Mark Gray
!   L3-GSI
!   Climate and Radiation Branch, Code 913
!   NASA/GSFC
!   Greenbelt MD 20771
!   gray@climate.gsfc.nasa.gov
!
! !Design Notes:
!   NONE
!
! !END
!
!-----------------------------------------------------------------------\
  use GeneralAuxType
  use science_parameters
  implicit none

  real(single), intent(in) :: upperLimit, lowerLimit, &
                              halfAbscissaInterval, abscissaIntervalStartingPoint
  real(single), intent(out) :: abscissaIntervalLower, abscissaIntervalUpper

  ! Reminder: halfAbscissaIntervalTau = 1., halfAbscissaIntervalRe = 1.
   
  if((abscissaIntervalStartingPoint+halfAbscissaInterval) > upperLimit) then 
      abscissaIntervalUpper = abscissaIntervalStartingPoint
  else
     abscissaIntervalUpper = (abscissaIntervalStartingPoint+halfAbscissaInterval)
  end if
   
  if((abscissaIntervalStartingPoint-halfAbscissaInterval) < lowerLimit) then 
     abscissaIntervalLower = abscissaIntervalStartingPoint 
  else
     abscissaIntervalLower = (abscissaIntervalStartingPoint-halfAbscissaInterval)
  end if

end subroutine abscissainterval
subroutine getctref_constradius(tauRegimeThreshold,      &
                                optical_thickness_upper, &
                                optical_thickness_lower, &
                                radius,                  &
                                phase,                   &
                                surface_albedo,          &
                                wave_index,              &
                                reflectance_upper,       &
                                reflectance_lower)

  use GeneralAuxType
  use libraryinterpolates
  use libraryarrays
  use spline_module
  use modis_numerical_module
  implicit none

  real(single), intent(in)  :: optical_thickness_upper, optical_thickness_lower
  integer, intent(in)       :: wave_index
  real(single), intent(in)  :: radius, surface_albedo, tauRegimeThreshold
  character(10),intent(in)  :: phase
  real(single), intent(out) :: reflectance_upper, reflectance_lower
 
  integer :: radii_size , radius_upperbound, radius_lowerbound, radius_index, i,j, points
  real(single) :: opticalthick_vector(6)
  real(single) :: reflectance_vector_upper(3), reflectance_vector_lower(3), yy2(3)

  if ((optical_thickness_lower < tauRegimeThreshold) .or. (optical_thickness_upper < tauRegimeThreshold)) then

     opticalthick_vector(1) = 0.
     opticalthick_vector(2:4) = library_taus(1:3)
     opticalthick_vector(5)= 1.8
     opticalthick_vector(6) = 2.
     if (phase == 'water') then
       call getclosestradius(number_waterradii,  &
                             water_radii,radius,  &
                             i)
       opticalthick_vector = opticalthick_vector/(1-asymmetry_water(wave_index,i))
       if(optical_thickness_lower < tauRegimeThreshold) then
         call nonasymptotic_calcu(opticalthick_vector, optical_thickness_lower, asymmetry_water(wave_index,i), &
                                  singlescattering_water(wave_index,i), ak_water(wave_index,i), &
                                  el_water(wave_index,i), em_water(wave_index,i), en_water(wave_index,i), &
                                  spherical_albedo_water(:,wave_index,i), int_reflectance_water(:,wave_index,i), &
                                  int_escapefuncwater_solar(wave_index,i), int_escapefuncwater_sensor(wave_index,i), &
                                  int_fluxdownwater_sensor(:,wave_index,i), int_fluxdownwater_solar(:,wave_index,i), & 
                                  surface_albedo, reflectance_lower)
       endif
       if(optical_thickness_upper < tauRegimeThreshold) then
         call nonasymptotic_calcu(opticalthick_vector, optical_thickness_upper, asymmetry_water(wave_index,i), &
                                  singlescattering_water(wave_index,i), ak_water(wave_index,i), &
                                  el_water(wave_index,i), em_water(wave_index,i), en_water(wave_index,i), &
                                  spherical_albedo_water(:,wave_index,i), int_reflectance_water(:,wave_index,i), &
                                  int_escapefuncwater_solar(wave_index,i), int_escapefuncwater_sensor(wave_index,i), &
                                  int_fluxdownwater_sensor(:,wave_index,i), int_fluxdownwater_solar(:,wave_index,i), & 
                                  surface_albedo, reflectance_upper)
       endif
     else
       call getclosestradius(number_iceradii,  &
                             ice_radii,radius,  &
                             i)
       opticalthick_vector = opticalthick_vector/(1-asymmetry_ice(wave_index,i))
       if(optical_thickness_lower < tauRegimeThreshold) then
         call nonasymptotic_calcu(opticalthick_vector, optical_thickness_lower, asymmetry_ice(wave_index,i), &
                                  singlescattering_ice(wave_index,i), ak_ice(wave_index,i), &
                                  el_ice(wave_index,i), em_ice(wave_index,i), en_ice(wave_index,i), &
                                  spherical_albedo_ice(:,wave_index,i), int_reflectance_ice(:,wave_index,i), &
                                  int_escapefuncice_solar(wave_index,i), int_escapefuncice_sensor(wave_index,i), &
                                  int_fluxdownice_sensor(:,wave_index,i), int_fluxdownice_solar(:,wave_index,i), & 
                                  surface_albedo, reflectance_lower)
       endif
       if(optical_thickness_upper < tauRegimeThreshold) then
         call nonasymptotic_calcu(opticalthick_vector, optical_thickness_upper, asymmetry_ice(wave_index,i), &
                                  singlescattering_ice(wave_index,i), ak_ice(wave_index,i), &
                                  el_ice(wave_index,i), em_ice(wave_index,i), en_ice(wave_index,i), &
                                  spherical_albedo_ice(:,wave_index,i), int_reflectance_ice(:,wave_index,i), &
                                  int_escapefuncice_solar(wave_index,i), int_escapefuncice_sensor(wave_index,i), &
                                  int_fluxdownice_sensor(:,wave_index,i), int_fluxdownice_solar(:,wave_index,i), & 
                                  surface_albedo, reflectance_upper)
       endif
     endif

  endif


  if ((optical_thickness_lower .ge. tauRegimeThreshold) .or. (optical_thickness_upper .ge. tauRegimeThreshold)) then

     reflectance_upper = 0.
     reflectance_lower = 0.
   
   ! get radii bounds
   
     if (phase == 'water') then
       call getradiibounds(number_waterradii,  &
                           water_radii,  &
                           radius,       &
                           radius_upperbound, &
                           radius_lowerbound)
       if (radius_upperbound == number_waterradii )then
         radius_lowerbound = radius_lowerbound-1
       else
         radius_upperbound = radius_upperbound+1
       endif
     else
       call getradiibounds(number_iceradii,  &
                           ice_radii,  &
                           radius,       &
                           radius_upperbound, &
                           radius_lowerbound)
   
       if (radius_upperbound == number_iceradii )then
         radius_lowerbound = radius_lowerbound-1
       else
         radius_upperbound = radius_upperbound+1
       endif
   
     endif
   
     j = 0
     if (phase == 'water') then
       do i= radius_lowerbound, radius_upperbound
         j = j+1
         if (optical_thickness_upper .ge. tauRegimeThreshold) then
           call asymptotic_calcu(optical_thickness_upper,                       &
                               asymmetry_water(wave_index,i),                   &
                               singlescattering_water(wave_index,i),            &
                               ak_water(wave_index,i),                          &
                               el_water(wave_index,i),                          &
                               em_water(wave_index,i),                          &
                               en_water(wave_index,i),                          &
                               spherical_albedo_water(number_taus,wave_index,i),&
                               int_reflectance_water(number_taus,wave_index,i), &
                               int_escapefuncwater_sensor(wave_index,i),        &
                               int_escapefuncwater_solar(wave_index,i),         &
                               surface_albedo,                                  &
                               reflectance_vector_upper(j))
         endif
         if (optical_thickness_lower .ge. tauRegimeThreshold) then
           call asymptotic_calcu(optical_thickness_lower,                       &
                               asymmetry_water(wave_index,i),                   &
                               singlescattering_water(wave_index,i),            &
                               ak_water(wave_index,i),                          &
                               el_water(wave_index,i),                          &
                               em_water(wave_index,i),                          &
                               en_water(wave_index,i),                          &
                               spherical_albedo_water(number_taus,wave_index,i),&
                               int_reflectance_water(number_taus,wave_index,i), &
                               int_escapefuncwater_sensor(wave_index,i),        &
                               int_escapefuncwater_solar(wave_index,i),         &
                               surface_albedo,                                  &
                               reflectance_vector_lower(j))
         endif
       enddo
     else
       do i= radius_lowerbound, radius_upperbound
         j = j+1
         if (optical_thickness_upper .ge. tauRegimeThreshold) then
           call asymptotic_calcu(optical_thickness_upper,                     &
                               asymmetry_ice(wave_index,i),                   &
                               singlescattering_ice(wave_index,i),            &
                               ak_ice(wave_index,i),                          &
                               el_ice(wave_index,i),                          &
                               em_ice(wave_index,i),                          &
                               en_ice(wave_index,i),                          &
                               spherical_albedo_ice(number_taus,wave_index,i),&
                               int_reflectance_ice(number_taus,wave_index,i), &
                               int_escapefuncice_sensor(wave_index,i),        &
                               int_escapefuncice_solar(wave_index,i),         &
                               surface_albedo,                                &
                               reflectance_vector_upper(j))
         endif
         if (optical_thickness_lower .ge. tauRegimeThreshold) then
           call asymptotic_calcu(optical_thickness_lower,                     &
                               asymmetry_ice(wave_index,i),                   &
                               singlescattering_ice(wave_index,i),            &
                               ak_ice(wave_index,i),                          &
                               el_ice(wave_index,i),                          &
                               em_ice(wave_index,i),                          &
                               en_ice(wave_index,i),                          &
                               spherical_albedo_ice(number_taus,wave_index,i),&
                               int_reflectance_ice(number_taus,wave_index,i), &
                               int_escapefuncice_sensor(wave_index,i),        &
                               int_escapefuncice_solar(wave_index,i),         &
                               surface_albedo,                                &
                               reflectance_vector_lower(j))
         endif
        enddo
      endif
   
   !  get final cloud top temperature at requested radii for wavelengths requested
      points = 3
   
      if(phase == 'water')then
        if (optical_thickness_upper .ge. tauRegimeThreshold) then
          call spline(points,water_radii(radius_lowerbound:radius_upperbound), reflectance_vector_upper, yy2)
          reflectance_upper = splint(points, radius, water_radii(radius_lowerbound:radius_upperbound), &
                                     reflectance_vector_upper, yy2)
        endif
        if (optical_thickness_lower .ge. tauRegimeThreshold) then
          call spline(points, water_radii(radius_lowerbound:radius_upperbound), reflectance_vector_lower, yy2)
          reflectance_lower = splint(points, radius, water_radii(radius_lowerbound:radius_upperbound), &
                                     reflectance_vector_lower, yy2)
        endif
      else
        if (optical_thickness_upper .ge. tauRegimeThreshold) then
          call spline(points, ice_radii(radius_lowerbound:radius_upperbound), reflectance_vector_upper, yy2)
          reflectance_upper = splint(points, radius, ice_radii(radius_lowerbound:radius_upperbound), &
                                     reflectance_vector_upper, yy2)
        endif
        if (optical_thickness_lower .ge. tauRegimeThreshold) then
          call spline(points, ice_radii(radius_lowerbound:radius_upperbound), reflectance_vector_lower, yy2)
          reflectance_lower = splint(points, radius, ice_radii(radius_lowerbound:radius_upperbound), &
                                     reflectance_vector_lower, yy2)
        endif
      endif
  endif
end subroutine getctref_constradius
subroutine getctref_consttau(tauRegimeThreshold,&
                             radius_upper,      &
                             radius_lower,      &
                             optical_thickness, &
                             phase,             &
                             surface_albedo,    &
                             wave_index,        &
                             reflectance_upper, &
                             reflectance_lower)
  use GeneralAuxType
  use libraryinterpolates
  use libraryarrays
  use spline_module
  use modis_numerical_module
  implicit none

  real(single), intent(inout)  :: radius_upper, radius_lower
  real(single), intent(in)  :: tauRegimeThreshold
  integer, intent(in)       :: wave_index
  real(single), intent(in)  :: optical_thickness, surface_albedo
  character(10),intent(in)  :: phase
  real(single), intent(out) :: reflectance_upper, reflectance_lower

  integer :: radii_size , radius_upperbound, radius_lowerbound, radius_index, i,j, vectorsize
  real(single) :: opticalthick_vector(6), opticalthick_vector2(6)
  real(single),dimension(20):: reflectance_vector, yy2

  integer :: dummy
  if (optical_thickness < tauRegimeThreshold) then

     opticalthick_vector(1) = 0.
     opticalthick_vector(2:4) = library_taus(1:3)
     opticalthick_vector(5)= 1.8
     opticalthick_vector(6) = 2.
     opticalthick_vector2 = opticalthick_vector
     if (phase == 'water') then
       call getclosestradius(number_waterradii,  &
                             water_radii,radius_lower,  &
                             i)
       call getclosestradius(number_waterradii,  &
                             water_radii,radius_upper,  &
                             j)
       if (i == j) then
         if (i == number_waterradii ) then
           i  = j - 1
         else
           j = i + 1
         endif
       endif
       radius_lower = water_radii(i)
       radius_upper = water_radii(j)
       opticalthick_vector = opticalthick_vector/(1-asymmetry_water(wave_index,i))
       call nonasymptotic_calcu(opticalthick_vector, optical_thickness, asymmetry_water(wave_index,i), &
                                singlescattering_water(wave_index,i), ak_water(wave_index,i), &
                                el_water(wave_index,i), em_water(wave_index,i), en_water(wave_index,i), &
                                spherical_albedo_water(:,wave_index,i), int_reflectance_water(:,wave_index,i), &
                                int_escapefuncwater_solar(wave_index,i), int_escapefuncwater_sensor(wave_index,i), &
                                int_fluxdownwater_sensor(:,wave_index,i), int_fluxdownwater_solar(:,wave_index,i), & 
                                surface_albedo, reflectance_lower)
       opticalthick_vector2 = opticalthick_vector2/(1-asymmetry_water(wave_index,j))
       call nonasymptotic_calcu(opticalthick_vector2, optical_thickness, asymmetry_water(wave_index,j), &
                                singlescattering_water(wave_index,j), ak_water(wave_index,j), &
                                el_water(wave_index,j), em_water(wave_index,j), en_water(wave_index,j), &
                                spherical_albedo_water(:,wave_index,j), int_reflectance_water(:,wave_index,j), &
                                int_escapefuncwater_solar(wave_index,j), int_escapefuncwater_sensor(wave_index,j), &
                                int_fluxdownwater_sensor(:,wave_index,j), int_fluxdownwater_solar(:,wave_index,j), & 
                                surface_albedo, reflectance_upper)
     else
       call getclosestradius(number_iceradii,  &
                             ice_radii,radius_lower,  &
                             i)
       call getclosestradius(number_iceradii,  &
                             ice_radii,radius_upper,  &
                             j)
       if (i == j) then
         if (i == number_iceradii ) then
           i  = j - 1
         else
           j = i + 1
         endif
       endif
       radius_lower = ice_radii(i)
       radius_upper = ice_radii(j)
       opticalthick_vector = opticalthick_vector/(1-asymmetry_ice(wave_index,i))
       call nonasymptotic_calcu(opticalthick_vector, optical_thickness, asymmetry_ice(wave_index,i), &
                                singlescattering_ice(wave_index,i), ak_ice(wave_index,i), &
                                el_ice(wave_index,i), em_ice(wave_index,i), en_ice(wave_index,i), &
                                spherical_albedo_ice(:,wave_index,i), int_reflectance_ice(:,wave_index,i), &
                                int_escapefuncice_solar(wave_index,i), int_escapefuncice_sensor(wave_index,i), &
                                int_fluxdownice_sensor(:,wave_index,i), int_fluxdownice_solar(:,wave_index,i), & 
                                surface_albedo, reflectance_lower)
       opticalthick_vector2 = opticalthick_vector2/(1-asymmetry_ice(wave_index,j))
       call nonasymptotic_calcu(opticalthick_vector2, optical_thickness, asymmetry_ice(wave_index,j), &
                                singlescattering_ice(wave_index,j), ak_ice(wave_index,j), &
                                el_ice(wave_index,j), em_ice(wave_index,j), en_ice(wave_index,j), &
                                spherical_albedo_ice(:,wave_index,j), int_reflectance_ice(:,wave_index,j), &
                                int_escapefuncice_solar(wave_index,j), int_escapefuncice_sensor(wave_index,j), &
                                int_fluxdownice_sensor(:,wave_index,j), int_fluxdownice_solar(:,wave_index,j), & 
                                surface_albedo, reflectance_upper)
     endif

  else
     reflectance_upper = 0.
     reflectance_lower = 0.
     yy2 = 0.
     reflectance_vector = 0.
     if (phase == 'water') then
       call getradiibounds(number_waterradii,  &
                      water_radii,  &
                      radius_lower,       &
                      dummy, &
                      radius_lowerbound)
       call getradiibounds(number_waterradii,  &
                      water_radii,  &
                      radius_upper,       &
                      radius_upperbound, &
                      dummy)
       if ((radius_upperbound - radius_lowerbound) == 1) then
         if (radius_upperbound  == number_waterradii ) then
           radius_lowerbound  = radius_lowerbound -1
         else
           radius_upperbound = radius_upperbound + 1
         endif
       endif
     else
       call getradiibounds(number_iceradii,  &
                           ice_radii,  &
                           radius_lower,       &
                           dummy, &
                           radius_lowerbound)
       call getradiibounds(number_iceradii,  &
                           ice_radii,  &
                           radius_upper,       &
                           radius_upperbound, &
                           dummy)
       if ((radius_upperbound - radius_lowerbound) == 1) then
         if (radius_upperbound  == number_iceradii ) then
           radius_lowerbound  = radius_lowerbound -1
         else
           radius_upperbound = radius_upperbound + 1
         endif 
       endif
     endif
   
     vectorsize = radius_upperbound - radius_lowerbound +1
   
     j = 0
   
     if (phase == 'water') then
       do i= radius_lowerbound, radius_upperbound
         j = j+1
         
             call asymptotic_calcu(optical_thickness,                         &
                               asymmetry_water(wave_index,i),                   &
                               singlescattering_water(wave_index,i),            &
                               ak_water(wave_index,i),                          &
                               el_water(wave_index,i),                          &
                               em_water(wave_index,i),                          &
                               en_water(wave_index,i),                          &
                               spherical_albedo_water(number_taus,wave_index,i),&
                               int_reflectance_water(number_taus,wave_index,i), &
                               int_escapefuncwater_sensor(wave_index,i),        &
                               int_escapefuncwater_solar(wave_index,i),         &
                               surface_albedo,                                  &
                               reflectance_vector(j))
       enddo
     else 
       do i= radius_lowerbound, radius_upperbound
         j = j+1
         call asymptotic_calcu(optical_thickness,                         &
                               asymmetry_ice(wave_index,i),                   &
                               singlescattering_ice(wave_index,i),            &
                               ak_ice(wave_index,i),                          &
                               el_ice(wave_index,i),                          &
                               em_ice(wave_index,i),                          &
                               en_ice(wave_index,i),                          &
                               spherical_albedo_ice(number_taus,wave_index,i),&
                               int_reflectance_ice(number_taus,wave_index,i), &
                               int_escapefuncice_sensor(wave_index,i),        & 
                               int_escapefuncice_solar(wave_index,i),         &
                               surface_albedo,                                  &
                               reflectance_vector(j))                     
        enddo
      endif
   
      if(phase == 'water')then
        call spline(vectorsize, &
                    water_radii(radius_lowerbound:radius_upperbound),  &
                    reflectance_vector(1:vectorsize),  &
                    yy2(1:vectorsize))
        reflectance_upper = splint(vectorsize, &
                                   radius_upper,  &
                                   water_radii(radius_lowerbound:radius_upperbound),  &
                                   reflectance_vector(1:vectorsize),  &
                                   yy2(1:vectorsize))
        reflectance_lower = splint(vectorsize, &
                                   radius_lower,  &
                                   water_radii(radius_lowerbound:radius_upperbound),  &
                                   reflectance_vector(1:vectorsize),  &
                                   yy2(1:vectorsize))
      else
        call spline(vectorsize, &
                    ice_radii(radius_lowerbound:radius_upperbound), &
                    reflectance_vector(1:vectorsize), &
                    yy2(1:vectorsize))
        reflectance_upper = splint(vectorsize, &
                                  radius_upper,  &
                                   ice_radii(radius_lowerbound:radius_upperbound),  &
                                   reflectance_vector(1:vectorsize),  &
                                   yy2(1:vectorsize))
        reflectance_lower = splint(vectorsize, &
                                  radius_lower,  &
                                   ice_radii(radius_lowerbound:radius_upperbound),  &
                                   reflectance_vector(1:vectorsize),  &
                                   yy2(1:vectorsize))
      endif
  endif

end subroutine getctref_consttau

subroutine getradiibounds(radiisize, radiidata, radius, upperbound, lowerbound)
  use GeneralAuxType
  implicit none

  integer,      intent(in)  :: radiisize
  real(single), intent(in)  :: radiidata(:), radius
  integer,      intent(out) :: upperbound, lowerbound

  integer :: i

  upperbound = -1
  lowerbound = -1
  do i = 1, radiisize - 1
    if (radius >= radiidata(i).and. radius <= radiidata(i+1)) then
      upperbound = i+1
      lowerbound = i
     exit
    endif
  enddo

end subroutine getradiibounds

end module get_retrieval_uncertainty 
