subroutine local_read_reynsst ( filename, &
                          year, month, day, hour, sst, & 
                          file_format, sst_success ) 

  implicit none

!-----------------------------------------------------------------------
! !F77
!
! !DESCRIPTION:  Retrieves an array of global sea surface temperature  
!                (sst) data from the NCEP Reynolds sst data product.
!                Both formatted (oi.mean.bias) and unformatted (oisst)
!                products are supported.  
!
!
! !INPUT PARAMETERS:
!  Integer       npoints_x   - number of points around a latitude circle 
!  Integer       npoints_y   - number of grid points along a meridian  
!
! !OUTPUT PARAMETERS:
!  character*(*) file_format - one of either 'unk', 'fmt' or 'unf'
!                        
!  Integer       year        - data year 
!  Integer       month       - data month
!  Integer       day         - data day
!  Intger        hour        - set to 0 
!
!  Logical       sst_success - .true. if able to retrieve sst data; 
!                              otherwise .false.
!
!  Real          sst         - global array of REYNSST sea surface
!                              temperature data in degree Celsius 
!
! !REVISION HISTORY:
!
!
! !TEAM-UNIQUE HEADER:
!
!    This software wasdeveloped by the MODIS Science Data Support Team
!    together with MODIS Group, CIMSS/SSEC, UW-Madison for the National
!    Aeronautics and Space Administration, Goddard Space Flight Center,
!    under contract NAS5-32373.
!
! !REFERENCES AND CREDITS:
!
!    Written by Rich Frey, MODIS Group, CIMSS/SSEC, UW-Madison 
!        and
!    Rich Hucek, Decision Systems Technologies, INC 
!    SAIC/GSC MODIS Science Data Support Office
!    7501 Forbes Blvd,  Seabrook MD 20706
!    rhucek@ltpmail.gsfc.nasa.gov
!
!
! !DESIGN NOTES:  
!
!    The NCEP Reynolds sst file format (formatted or unformatted) is
!    identified by comparing the size of the product staged for input 
!    against known, unique and fixed values for the two format types.
!    If a match is made, processing continues.  Otherwise, read_reynsst
!    aborts with a fatal error.
!
!    If a read error occurs, read_reynsst returns control to the 
!    calling program and sets the subroutine logical status flag,
!    sst_success, to .false.  
!
!
!  Externals:
!
!    Functions and Subroutines:
!        modis_io_gen_closef       (MODIS_IO_GEN_CLOSE.f in src_L2)
!        pgs_io_gen_openf          (libPGSTK.a)
!        pgs_pc_getfilesize        (libPGSTK.a)
!
!    Named Constant:
!        LUN_REYNSST               (Atmos_AncData.inc in src_L2)
!        SIZE_OF_REYNSST_FMT       (Atmos_AncData.inc in src_L2)
!        SIZE_OF_REYNSST_UNF       (Atmos_AncData.inc in src_L2)
!        PGS_S_SUCCESS             (PGS_SMF.f)
!
! !END
!-----------------------------------------------------------------------

!-----Declaration of function arguments
      character*(*) , intent(in)   ::  filename
      integer,        intent(out)  ::  year, month, day, hour
      logical,        intent(out)  ::  sst_success
      real,           intent(out)  ::  sst(0:, 0:)
      character*(*),  intent(out)  ::  file_format


!-----Declaration of local variables 
      integer       npoints_x, npoints_y
      character*160 errmsg
      integer       filesize, header(8), i, ios, j, level, lun,  &
                   reclen, status, version , values,blah

!-----------------------------------------------------------------------
!     INITIALIZATION
!-----------------------------------------------------------------------
  npoints_x = size(sst,1)-1
  npoints_y = size(sst,2)-1
  sst_success = .false.
  file_format = 'unk'

!-----------------------------------------------------------------------
!     DETERMINE FILE FORMAT 
!-----------------------------------------------------------------------
  reclen  = 1
  version = 1
  status = 0
  lun=40

  if ( index(filename,'oi.sst') /= 0 .or. index(filename,'oi.mean') /= 0 ) then
    file_format = 'fmt'
    open( unit = lun, &
          file = filename,  &
          form = 'formatted', &
          access = 'sequential',    &
          recl = reclen, &
          status = 'old')

  elseif( index(filename,'oisst') /= 0) then
    file_format = 'unf'
    open( unit = lun, &
          file = filename,  &
          form = 'unformatted', &
          access = 'sequential',    &
!          recl = reclen, &
          status = 'old')
  else
    level   =  2
    status  = -1
  endif

!-----------------------------------------------------------------------
!     READ SST DATA 
!-----------------------------------------------------------------------

  if ( status .eq. 0) then

  if (file_format .eq. 'fmt') then
    read( lun, '( 8i5 )', iostat = ios ) header
  else if(file_format .eq. 'unf') then
    read( lun, iostat = ios ) header
  end if

  if ( ios .ne. 0 ) then
    level = 1
  else
    year  = header( 1 )
    month = header( 2 )
    day   = header( 3 )
    hour  = 0
  endif

  if (file_format .eq. 'fmt') then
    read( lun, '( 20f4.2 )', iostat = ios ) sst
  else if (file_format .eq. 'unf') then
    read( lun, iostat = ios ) ( (sst(i,j), i=0,359), j=0,179)
  end if
  if ( ios .ne. 0 ) then
    level = 1
  else
    sst_success = .true.
  endif
  close(lun)
endif
end subroutine local_read_reynsst
