subroutine mod_pr06od( statistics, errorlevel)

!-----------------------------------------------------------------------
!f90 mod_pr06od
!
!Description:
!
!  retrieve cloud optical and microphysical properties from MODIS radiation
!  measurements
!
!input parameters:
!
!output parameters:
!
!revision history:
!
! v.1 July 2001 Initial work mag gray@climate.gsfc.nasa.gov
!
!team-unique header:
!
! Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA 
!
!references and credits:
!
! Mark Gray
! gray@climate.gsfc.nasa.gov
! EmergentIT 
! Code 913, NASA GSFC 
! Greenbelt, MD 20771
!
! Nakajima, T. and M.D. King, 1990: determination of the
!    optical thickness and effective particle radius of
!    clouds from reflected solar radiation measurements
!    part i: theory. j. atmos. sci., 47, 1878-1893.
!
!design note:
!
!end
!----------------------------------------------------------------------


   use GeneralAuxType
   use core_arrays
   use nonscience_parameters
   use global_model_grids,only:grids_are_read  
   use ancillary_module
   use modis_io_module
   use modis_frontend_module
   use modis_cloudhandler
   use modis_science_module

   implicit none

   include 'mapi.f90.inc'

   real, intent(out)         :: statistics(7)
   integer, intent(out)      :: errorlevel

   real(single)              :: solarzen, sensorzen,relativeazi
   integer, dimension (2)    :: start, edge, stride
   integer, dimension (2)    :: localstart, localedge, localstride
 
   integer                   :: status, i,arguments, tilesize, &
                                number_of_iterations
   integer                   :: debug_start_iteration = 0, debug_stop_iteration = 0
   character(MAX_FILE_NAME_LEN)            :: ice_library, water_library, library_path, directive
   character(MAX_FILE_NAME_LEN)            :: l1b_name, cloudmask_name,&
                                transmittance_library, &
                                filedirective, &
                                geolocation_name, mod06_name , &
                                gdas_name, ncepice_name, sst_name, nise_name, ozone_name

   character(MAX_FILE_NAME_LEN)            :: surfacealbedo_lib_659,      &
                                surfacealbedo_lib_858,      &
                                surfacealbedo_lib_124,      &
                                surfacealbedo_lib_164,      &
                                surfacealbedo_lib_21,       &
                                ecosystem_data_name,        &
                                snowicealbedo_data_name

   character(MAX_FILE_NAME_LEN),allocatable :: commandline(:) 

   real                      :: threshold_solar_zenith,      &
                                threshold_sensor_zenith,     &
                                threshold_relative_azimuth 

   integer                   :: l1b_filedata(MODFILLEN),        &
                                cloudmask_filedata(MODFILLEN),  &
                                geolocation_filedata(MODFILLEN), &
                                mod06_filedata(MODFILLEN)

   logical                   :: debug

   integer, external         :: iargc
   integer :: Npix, Nscan, lst(2)


   status = success
   errorlevel = 0
   statistics(:) = .0

!  get runtime setup parameters, array sizes, lat/long-pixel limits
   if (status == success) then
      call initialize_run (   start, edge, stride,       &
                              tilesize,                  &
                              l1b_name,                   &
                              cloudmask_name,            &
                              mod06_name,                &
                              geolocation_name,          &
                              gdas_name,                 &
                              ncepice_name,              &
                              sst_name,                  &
                              nise_name,                 &
                              ozone_name,                &
                              water_library,             &
                              ice_library,               &
                              transmittance_library,     &
                              surfacealbedo_lib_659,      &
                              surfacealbedo_lib_858,      &
                              surfacealbedo_lib_124,      &
                              surfacealbedo_lib_164,      &
                              surfacealbedo_lib_21,       &
                              ecosystem_data_name,        &
                              snowicealbedo_data_name,    &
                              threshold_solar_zenith,    &
                              threshold_sensor_zenith,   &
                              threshold_relative_azimuth,&
                              debug,                     &
                              status )
     if (status /= success) then
       call local_message_handler('Problem reported in initialize_run. Check previous message/s', status,'mod_pr06od')
     endif
   endif

!  check that all files exist and are openable
!  if not there is a complete algorithm failure
   if( status  == success) then
      call check_datasources(l1b_name,                   &
                             cloudmask_name,            &
                             geolocation_name,          &
                             gdas_name,                 &
                             ncepice_name,              &
                             sst_name,                  &
                             nise_name,                 &
                             ozone_name,                &
                             mod06_name,           &
                             water_library,             &
                             ice_library,               &
                             transmittance_library,     &
                             surfacealbedo_lib_659,     &
                             surfacealbedo_lib_858,     &
                             surfacealbedo_lib_124,     &
                             surfacealbedo_lib_164,     &
                             surfacealbedo_lib_21,      &
                             ecosystem_data_name,       &
                             snowicealbedo_data_name,   & 
                             status)
     if (status /= success) then
       call local_message_handler('Problem reported in check_datasources. Check previous message/s', status,'mod_pr06od')
     endif
   endif


!  open core hdf files for reading and chec 
   if (status == success) then
     call openclose_files (l1b_name,            &
                           cloudmask_name,     &
                           geolocation_name,   &
                           mod06_name,         &
                           'open',             &
                           l1b_filedata,       &
                           cloudmask_filedata, &
                           geolocation_filedata,&
                           mod06_filedata,     &
                           status)
     if (status /= success) then
       call local_message_handler('Problem reported in openclose_files . Check previous message/s', status,'mod_pr06od')
     endif
   endif

!  check the size of the array/s to be processed 
   if (status == success) then
     call check_datasize(l1b_filedata, start, stride, edge, status)
     if (status /= success) then
       call local_message_handler('Problem reported in check_datasize . Check previous message/s', status,'mod_pr06od')
     endif
   endif

!  if the total number of lines to read is greater than the 
!  tilesize (number of lines to process/ iteration) then
!  we need to iterate.  

!     allocate the libraryarrays for storage and interpolation
      if (status == 0 ) then
         call allocate_libraryarrays (transmittance_library, water_library,ice_library, debug, status )
         if (status /= success) then
           call local_message_handler('Problem reported in allocate_libraryarrays',status, 'mod_pr06od')
         endif
      endif

!     read libraries in preparation for later interpolation
      if (status == 0 ) then
         call readlibraries(transmittance_library,water_library,ice_library, debug,status)
         if (status /= success) then
           call local_message_handler('Problem reported in readlibraries',status,'mod_pr06od')
         endif
      endif


! for inventory metadata
   total_number_of_pixels = edge(2)*edge(1)

   number_of_iterations = ceiling(real(edge(2)) / real(tilesize))
   grids_are_read = .false.

   ! Default behavior for setting debug <start/stop> _iteration is to pick only the (10 scan line)
   ! iteration that bounds debugPxlYcoord_1to20L0inclusiv.  Specify additional iterations here. 
   if(debugPxlYcoord_1to20L0inclusiv .ne. 0) then
      debug_start_iteration = floor((real(debugPxlYcoord_1to20L0inclusiv-1)/real(tilesize))) + 1
      debug_stop_iteration = debug_start_iteration
   endif
   debugPRN = .false.

   do iteration = 1, number_of_iterations   
!      print*, "ITERATION: ", iteration
      if(debugPxlYcoord_1to20L0inclusiv .ne. 0) then
         print*, "ITERATION: ", iteration
       endif

      if (iteration == 1) directive = 'open'
      if (iteration >1 .and. iteration  < number_of_iterations) directive = 'no_action'
      if (iteration == number_of_iterations) directive = 'close'
      localstart  = start
      localedge   = edge
      localstride = stride

     if (iteration == 1 .and. number_of_iterations == 1) then
         filedirective = 'openclose' 
      elseif (iteration == 1) then
         filedirective = 'open'
      elseif (iteration == number_of_iterations) then
         filedirective = 'close'
      else
         filedirective = 'inprogress'
      endif

      if (edge(2) > tilesize .and. iteration == 1) then
         localedge(2) = tilesize 
      endif
      if (edge(2) > tilesize .and. iteration > 1 ) then 
         localstart(2)  = start(2) + (iteration-1)*tilesize*stride(2)
         localedge(2)   = tilesize
      endif
          
      if ( edge(2) - iteration*tilesize < 0 ) localedge(2) = edge(2) - (iteration-1)*tilesize

!     Setup and allocate all data arrays
      if (status == success ) then
         call allocate_arrays (localstart, &
                               localstride, &
                               localedge, &
                               debug, &
                               status )
         if (status /= success) then
           call local_message_handler('Failure detected before allocate_modisarrays, check previous messages', &
                                      status, &
                                      'mod_pr06od')
         endif
      endif
     
      call init_qualitydata
 
      ! get data cube to be processed and ancillary data arrays
      if (status == success ) then
         call get_modis_data_cube (l1b_filedata, &
                                   l1b_name, &
                                   geolocation_filedata, &
                                   geolocation_name, &
                                   localstart,  &
                                   localedge,   &
                                   localstride, &
                                   iteration,   &
                                   debug,       &
                                   status )
         if (status /= success) then
           call local_message_handler('Failure detected before get_modis_data_cube check previous messages',&
                                      status, &
                                      'mod_pr06od')
         endif
      endif
   
!     get cloud decision info
      if (status == success ) then
         call modis_cloudprocessing(cloudmask_filedata, &
                                    cloudmask_name,&
                                    iteration,   &
                                    localstart,&
                                    localstride,&
                                    localedge,&
                                    debug,&
                                    status)
         if (status /= success) then
           call local_message_handler('Failure detected before cloudprocessing. Check previous messages/s', &
                                      status, &
                                      'mod_pr06od')
         endif
      endif


!     get ancillary data, right now that means albedos
      if (status == success ) then 
         call set_ancillary(gdas_name,                 &
                            ncepice_name,              &
                            sst_name,                  &
                            nise_name,                 &
                            ozone_name,                &
                            mod06_name,           &
                            mod06_filedata,            &
                            surfacealbedo_lib_659,     &
                            surfacealbedo_lib_858,     &
                            surfacealbedo_lib_124,     &
                            surfacealbedo_lib_164,     &
                            surfacealbedo_lib_21,      &
                            ecosystem_data_name,       &
                            snowicealbedo_data_name,   &
                            localstart, localstride, localedge, &
                            debug,                     &
                            status )
         if (status /= success) then
           call local_message_handler('Problem reported in set_ancillary. Check previous message/s', &
                                      status, &
                                      'mod_pr06od')
         endif
      endif

!     process modis data for cloud optical thickness and cloud top effective
!     particle radius

      if (status == success .and. ( (debug_start_iteration .eq. 0) .or. &
          ( debug_start_iteration .le. iteration .and. iteration .le. debug_stop_iteration ) ) ) then

         call scienceinterface( threshold_solar_zenith,      &
                                threshold_sensor_zenith,     &
                                threshold_relative_azimuth,  &
                                debug,                       &
                                status)

         if (status /= success) then
           call local_message_handler('Problem reported in scienceinterface',status,'mod_pr06od')
         endif
      endif

!     send output retrieval and qa data to a hdf file
      if (status == success ) then
         call output_retrieval(mod06_filedata, &
                               mod06_name,     &
                               iteration,     &
                               start, edge, stride, &
                               status)  
         if (status /= success) then
           call local_message_handler('Problem reported in output_retrieval',status,'mod_pr06od')
         endif
      endif

      if (status == success) then
         !call deallocate_cleanup(status )
         if (status /= success) then
           call local_message_handler('Problem reported in deallocate_cleanup', status,'mod_pr06od')
         endif
      endif
   enddo

   if (status == success) then
     call openclose_files (l1b_name,            &
                           cloudmask_name,     &
                           geolocation_name,   &    
                           mod06_name,         &
                           'close',            &
                           l1b_filedata,       &
                           cloudmask_filedata, &
                           geolocation_filedata,&
                           mod06_filedata,     &
                           status)
     if (status /= success) then
       call local_message_handler('Problem reported in openclose_files', status,'mod_pr06od')
     endif
   endif

! here we do the final assignment of the Inventory metadata statistics

   statistics(1) = real(IM_successful_retrieval_count) / real(total_number_of_pixels) * 100.
   statistics(2) = real(IM_cloudy_count) / real(total_number_of_pixels) * 100.
   statistics(3) = real(IM_water_cloud_count) / real(total_number_of_pixels) * 100.
   statistics(4) = real(IM_ice_cloud_count) / real(total_number_of_pixels) * 100.
   




end subroutine mod_pr06od
