module modis_cloudhandler

private

public :: modis_cloudprocessing

contains

subroutine modis_cloudprocessing(cloudmask_filedata, cloudmask_name, current_scan,start, stride, edge, debug, status)

   use GeneralAuxType, only: integer_fourbyte, integer_onebyte
   use core_arrays
    
   implicit none
   
   character(*), intent(in)                :: cloudmask_name
   integer, intent(in)                                  :: current_scan
   integer(integer_fourbyte), dimension(2), intent(in)  :: start, stride, edge, cloudmask_filedata(:)
   logical, intent(in)                                  :: debug
   integer(integer_fourbyte), intent(out)               :: status


   integer(integer_onebyte), allocatable, dimension(:,:,:) :: packedcloudmask, packedcloudmaskQA
   integer(integer_fourbyte) :: xdimension, ydimension, cloudmask_bits, checkvariable,i,j,cloudmaskQA_bits

   cloudmask_bits = 48
   cloudmaskQA_bits = 80

   xdimension =  edge(1)  
   ydimension =  edge(2) 
 
   allocate (packedcloudmask(xdimension,ydimension,cloudmask_bits), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   allocate (packedcloudmaskQA(xdimension,ydimension,cloudmaskQA_bits), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif


   ! read libraries in preparation for later interpolation
   call modis_readcloudmask( cloudmask_filedata,&
                             cloudmask_name,    &
                             current_scan,      &
                             start,             &
                             edge,              &
                             stride,            &
                             packedcloudmask,   &
                             packedcloudmaskQA, &
                             debug, status)

   call modis_unpackcloudmask ( packedcloudmask, packedcloudmaskQA, debug, status)

   deallocate(packedcloudmask, stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   deallocate(packedcloudmaskQA, stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
 
end subroutine modis_cloudprocessing
!f90 modis_readcloudmask
!
!Description:
!
!  Read MAS 48 bit cloud mask from a hdf file previously opened for reading.
!  Convert individual tests from bits to short integers and pass short integer
!  array out.
!
!input parameters:
!
!output parameters:
!
!revision history:
!
! v.1 September 2001 Initial work mag gray@climate.gsfc.nasa.gov
!
!team-unique header:
!
! Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA
!
!references and credits:
!
! Mark Gray
! gray@climate.gsfc.nasa.gov
! EmergentIT
! Code 913, NASA GSFC
! Greenbelt, MD 20771
!
!----------------------------------------------------------------------

subroutine modis_readcloudmask( mapi_filedata,filename, current_scan,start, edge, stride, data, QAdata, debug,status)
   use nonscience_parameters
   use GeneralAuxType,only: integer_onebyte,  integer_fourbyte, single
   use hdf 
   implicit none
   include 'PGS_MODIS_39500.f'
   include 'mapi.f90.inc'

   integer, intent(in) :: current_scan
   integer(integer_fourbyte),  dimension (2),   intent(in)     :: start,edge,stride,mapi_filedata(:)
   character(*),                   intent(in)                  :: filename
   integer(integer_onebyte) ,  dimension(:,:,:),intent(out)    :: data, QAdata
   integer(integer_fourbyte),                   intent(out)    :: status
   logical, intent(in)                                  :: debug

   integer(integer_fourbyte)                       :: sds_id,sds_index,attr_index, hdfid
   integer(integer_fourbyte),dimension (3)         :: dimsizes, localstart,localedge,localstride
   integer(integer_fourbyte)                       :: rank,  number_type, nattrs
   character(MAX_SDS_NAME_LEN)                                   :: local_sds_name,sds_name
   integer                                         :: xdimension, ydimension , checkvariable, &
                                                      numberofbands, index,cloudmask_bytes
   integer                                         :: bit, byte, i, j, output_bits, cloudmaskQA_bytes
   integer(integer_onebyte), allocatable, dimension(:,:,:)     :: rawbytedata , rawQAbytedata
   
   integer ::  DS_Dim1_Cldmsk, DS_Dim1_Cldmsk_QA,DS_Dim2,DS_Dim3
   logical :: Error_Flag

!   hdfid = sfstart(filename,DFACC_READ)

!  read the cloudmask sds

   xdimension =   edge(1)
   ydimension =   edge(2)


   cloudmask_bytes=6
   localstart  = (/ start(1), start(2), 0/)
   localedge   = (/ edge(1),    edge(2), cloudmask_bytes/)
   localstride = (/ stride(1),  stride(2), 1/)
  
   allocate (rawbytedata(cloudmask_bytes,  xdimension,ydimension ),stat=checkvariable)

!  read the cloudmask QA sds

   cloudmaskQA_bytes = 10

   allocate (rawQAbytedata(cloudmaskQA_bytes, xdimension,ydimension ),stat=checkvariable)

   call read_cldmsk(mapi_filedata, current_scan, &
                    cloudmask_bytes,cloudmaskQA_bytes,xdimension,ydimension, &
                    DS_Dim1_Cldmsk, DS_Dim1_Cldmsk_QA,DS_Dim2, &
                    DS_Dim3, rawbytedata, rawQAbytedata, Error_Flag)
   if(Error_Flag) then
    status = failure
    call local_message_handler('Failure detected in Cloud Mask read. Check previous message/s', status,'modis_readcloudmask')
   endif 

!  convert n x m x 6 byte(48 bit) data to n x m 48 integer array

   data(:,:,:) = 0

   do i = 1, xdimension
     do j = 1, ydimension
       output_bits = 0
       do byte = 1 , cloudmask_bytes
         do bit = 0, 7
            output_bits = output_bits + 1
            if ( btest(rawbytedata(byte, i, j ), bit) ) data(i, j, output_bits) = 1
         enddo   
       enddo
     enddo
   enddo

   QAdata(:,:,:) = 0
   do i = 1, xdimension
     do j = 1, ydimension
       output_bits = 0
       do byte = 1 , cloudmaskQA_bytes
         do bit = 0, 7
            output_bits = output_bits + 1
            if ( btest(rawQAbytedata(byte, i, j ), bit) ) QAdata(i, j, output_bits) = 1
         enddo
       enddo
     enddo
   enddo
   deallocate(rawbytedata, stat = checkvariable)
   deallocate(rawQAbytedata, stat = checkvariable)

end subroutine modis_readcloudmask
subroutine modis_unpackcloudmask(packedcloudmask,packedcloudmaskQA,debug, status)
  use GeneralAuxType, only: integer_onebyte, integer_fourbyte
  use core_arrays, only: cloudmask, processing_information

  implicit none

  integer(integer_fourbyte), intent(out)                  :: status
  integer(integer_onebyte), intent(in), dimension(:,:,:)  :: packedcloudmask, packedcloudmaskQA
  logical, intent(in)                                     :: debug
  integer(integer_fourbyte)                               :: xdimension, ydimension, bytedimension,i,j
  status=0

  xdimension    = size(packedcloudmask,1)
  ydimension    = size(packedcloudmask,2)
  bytedimension = size(packedcloudmask,3)

! top level flags
  cloudmask(:,:)%cloudmask_determined = .false.  
  where(packedcloudmask(:,:,1) == 1) cloudmask(:,:)%cloudmask_determined = .true.

  cloudmask(:,:)%confident_cloudy = .false.
  cloudmask(:,:)%probablyclear_66 = .false.
  cloudmask(:,:)%probablyclear_95 = .false.
  cloudmask(:,:)%probablyclear_99 = .false.
  where(packedcloudmask(:,:,2) == 0 .and. packedcloudmask(:,:,3) ==0) &
           cloudmask(:,:)%confident_cloudy = .true.
  where(packedcloudmask(:,:,2) == 1 .and. packedcloudmask(:,:,3) ==0) &
           cloudmask(:,:)%probablyclear_66 = .true.
  where(packedcloudmask(:,:,2) == 0 .and. packedcloudmask(:,:,3) ==1) &
           cloudmask(:,:)%probablyclear_95 = .true.
  where(packedcloudmask(:,:,2) == 1 .and. packedcloudmask(:,:,3) ==1) &
           cloudmask(:,:)%probablyclear_99 = .true.

! processing path flags
  cloudmask(:,:)%day = .false.
  where(packedcloudmask(:,:,4) == 1) cloudmask(:,:)%day = .true.
 
  cloudmask(:,:)%night = .false.
  where(packedcloudmask(:,:,4) == 0) cloudmask(:,:)%night = .true.

  cloudmask(:,:)%sunglint = .false.
  where(packedcloudmask(:,:,5) == 0) cloudmask(:,:)%sunglint = .true.

  cloudmask(:,:)%snowice_surface = .false.
  where(packedcloudmask(:,:,6) == 0) cloudmask(:,:)%snowice_surface = .true.

  cloudmask(:,:)%water_surface = .false.
  cloudmask(:,:)%coastal_surface = .false.
  cloudmask(:,:)%desert_surface = .false.
  cloudmask(:,:)%land_surface =  .false.

  where(packedcloudmask(:,:,7) == 0 .and. packedcloudmask(:,:,8) == 0) &
           cloudmask(:,:)%water_surface= .true.
  where(packedcloudmask(:,:,7) == 1 .and. packedcloudmask(:,:,8) == 0) &
           cloudmask(:,:)%coastal_surface = .true.
  where(packedcloudmask(:,:,7) == 0 .and. packedcloudmask(:,:,8) == 1) &
           cloudmask(:,:)%desert_surface = .true.
  where(packedcloudmask(:,:,7) == 1 .and. packedcloudmask(:,:,8) == 1) &
           cloudmask(:,:)%land_surface = .true.

! byte 1

  cloudmask(:,:)%noncloud_obstruction =  .false.
  where(packedcloudmask(:,:,9) == 0) cloudmask(:,:)%noncloud_obstruction = .true.

  cloudmask(:,:)%thin_cirrus_solar =  .false.
  where(packedcloudmask(:,:,10) == 0) cloudmask(:,:)%thin_cirrus_solar = .true.

  cloudmask(:,:)%shadow =  .false.
  where(packedcloudmask(:,:,11) == 0) cloudmask(:,:)%shadow= .true.

  cloudmask(:,:)%thin_cirrus_ir =  .false.
  where(packedcloudmask(:,:,12) == 0) cloudmask(:,:)%thin_cirrus_ir = .true.
  
  cloudmask(:,:)%adjacent_cloud =  .false.
  where(packedcloudmask(:,:,13) == 0) cloudmask(:,:)%adjacent_cloud= .true.

  cloudmask(:,:)%test_irthreshold =  .false.
  where(packedcloudmask(:,:,14) == 0) cloudmask(:,:)%test_irthreshold  = .true.

  cloudmask(:,:)%test_CO2 =  .false.
  where(packedcloudmask(:,:,15) == 0) cloudmask(:,:)%test_CO2 = .true.

  cloudmask(:,:)%test_high_67 =  .false.
  where(packedcloudmask(:,:,16) == 0) cloudmask(:,:)%test_high_67= .true.

  cloudmask(:,:)%test_high_138 =  .false.
  where(packedcloudmask(:,:,17) == 0) cloudmask(:,:)%test_high_138  = .true.

  cloudmask(:,:)%test_high_37_12 =  .false.
  where(packedcloudmask(:,:,18) == 0) cloudmask(:,:)%test_high_37_12  = .true.

  cloudmask(:,:)%test_irdiff =  .false.
  where(packedcloudmask(:,:,19) == 0) cloudmask(:,:)%test_irdiff = .true.

  cloudmask(:,:)%test_39_11 =  .false.
  where(packedcloudmask(:,:,20) == 0) cloudmask(:,:)%test_39_11  = .true.

  cloudmask(:,:)%test_visiblereflectance =  .false.
  where(packedcloudmask(:,:,21) == 0) cloudmask(:,:)%test_visiblereflectance  = .true.

  cloudmask(:,:)%test_visible_ratio=  .false.
  where(packedcloudmask(:,:,22) == 0) cloudmask(:,:)%test_visible_ratio= .true.

  cloudmask(:,:)%test_restoral_ndvi =  .false.
  where(packedcloudmask(:,:,23) == 0) cloudmask(:,:)%test_restoral_ndvi= .true.

  cloudmask(:,:)%test_73_11_land_polar_night=  .false.
  where(packedcloudmask(:,:,24) == 0) cloudmask(:,:)%test_73_11_land_polar_night= .true.

  cloudmask(:,:)%test_temporal_consistency =  .false.
  where(packedcloudmask(:,:,25) == 0) cloudmask(:,:)%test_temporal_consistency  = .true.

  cloudmask(:,:)%test_spatial_consistency=  .false.
  where(packedcloudmask(:,:,26) == 0) cloudmask(:,:)%test_spatial_consistency= .true.

  cloudmask(:,:)%test_clearskyrestoral_glint =  .false.
  where(packedcloudmask(:,:,27) == 0) cloudmask(:,:)%test_clearskyrestoral_glint = .true.

  cloudmask(:,:)%test_night_surfacetemp =  .false.
  where(packedcloudmask(:,:,28) == 0) cloudmask(:,:)%test_night_surfacetemp = .true.

  cloudmask(:,:)%test_suspended_dust =  .false.
  where(packedcloudmask(:,:,29) == 0) cloudmask(:,:)%test_suspended_dust = .true.

  cloudmask(:,:)%test_86_73_nightocean =  .false.
  where(packedcloudmask(:,:,30) == 0) cloudmask(:,:)%test_86_73_nightocean = .true.

!  cloudmask(:,:)%test_nightocean_var =  .false.
!  where(packedcloudmask(:,:,31) == 0) cloudmask(:,:)%test_nightocean_var = .true.





  !250m cloud flags
  cloudmask(:,:)%visible_cloudtest_11=  .false.
  where(packedcloudmask(:,:,33) == 0) cloudmask(:,:)%visible_cloudtest_11= .true.
  cloudmask(:,:)%visible_cloudtest_12=  .false.
  where(packedcloudmask(:,:,34) == 0) cloudmask(:,:)%visible_cloudtest_12= .true.
  cloudmask(:,:)%visible_cloudtest_13=  .false.
  where(packedcloudmask(:,:,35) == 0) cloudmask(:,:)%visible_cloudtest_13= .true.
  cloudmask(:,:)%visible_cloudtest_14=  .false.
  where(packedcloudmask(:,:,36) == 0) cloudmask(:,:)%visible_cloudtest_14= .true.

  cloudmask(:,:)%visible_cloudtest_21=  .false.
  where(packedcloudmask(:,:,37) == 0) cloudmask(:,:)%visible_cloudtest_21= .true.
  cloudmask(:,:)%visible_cloudtest_22=  .false.
  where(packedcloudmask(:,:,38) == 0) cloudmask(:,:)%visible_cloudtest_22= .true.
  cloudmask(:,:)%visible_cloudtest_23=  .false.
  where(packedcloudmask(:,:,39) == 0) cloudmask(:,:)%visible_cloudtest_23= .true.
  cloudmask(:,:)%visible_cloudtest_24=  .false.
  where(packedcloudmask(:,:,40) == 0) cloudmask(:,:)%visible_cloudtest_24= .true.

  cloudmask(:,:)%visible_cloudtest_31=  .false.
  where(packedcloudmask(:,:,41) == 0) cloudmask(:,:)%visible_cloudtest_31= .true.
  cloudmask(:,:)%visible_cloudtest_32=  .false.
  where(packedcloudmask(:,:,42) == 0) cloudmask(:,:)%visible_cloudtest_32= .true.
  cloudmask(:,:)%visible_cloudtest_33=  .false.
  where(packedcloudmask(:,:,43) == 0) cloudmask(:,:)%visible_cloudtest_33= .true.
  cloudmask(:,:)%visible_cloudtest_34=  .false.
  where(packedcloudmask(:,:,44) == 0) cloudmask(:,:)%visible_cloudtest_34= .true.

  cloudmask(:,:)%visible_cloudtest_41=  .false.
  where(packedcloudmask(:,:,45) == 0) cloudmask(:,:)%visible_cloudtest_41= .true.
  cloudmask(:,:)%visible_cloudtest_42=  .false.
  where(packedcloudmask(:,:,46) == 0) cloudmask(:,:)%visible_cloudtest_42= .true.
  cloudmask(:,:)%visible_cloudtest_43=  .false.
  where(packedcloudmask(:,:,47) == 0) cloudmask(:,:)%visible_cloudtest_43= .true.
  cloudmask(:,:)%visible_cloudtest_44=  .false.
  where(packedcloudmask(:,:,48) == 0) cloudmask(:,:)%visible_cloudtest_44= .true.

! QA bit fields
  cloudmask(:,:)%cloudmask_useful = .false.
  where(packedcloudmaskQA(:,:,1) == 1) cloudmask(:,:)%cloudmask_useful = .true.
  
  cloudmask(:,:)%lowest_confidence       = .false. 
  where(packedcloudmaskQA(:,:,2) == 0 .and. packedcloudmaskQA(:,:,3) == 0 .and. packedcloudmaskQA(:,:,4) == 0) &
  cloudmask(:,:)%lowest_confidence       = .true.

  cloudmask(:,:)%intermediate_confidence = .false.
  where(packedcloudmaskQA(:,:,2) == 0 .and. packedcloudmaskQA(:,:,3) == 1 .and. packedcloudmaskQA(:,:,4) == 1) &
  cloudmask(:,:)%intermediate_confidence = .true.

  cloudmask(:,:)%high_confidence         = .false.
  where(packedcloudmaskQA(:,:,2) == 1 .and. packedcloudmaskQA(:,:,3) == 1 .and. packedcloudmaskQA(:,:,4) == 0) &
  cloudmask(:,:)%high_confidence = .true.

  cloudmask(:,:)%highest_confidence      = .false.
  where(packedcloudmaskQA(:,:,2) == 1 .and. packedcloudmaskQA(:,:,3) == 1 .and. packedcloudmaskQA(:,:,4) == 1) &
  cloudmask(:,:)%highest_confidence = .true.


  cloudmask(:,:)%applied_nco = .false.
  where(packedcloudmaskQA(:,:,9) == 1) cloudmask(:,:)%applied_nco = .true.

  cloudmask(:,:)%applied_thincirrus_solar = .false.
  where(packedcloudmaskQA(:,:,10) == 1) cloudmask(:,:)%applied_thincirrus_solar= .true.

  cloudmask(:,:)%applied_shadow = .false.
  where(packedcloudmaskQA(:,:,11) == 1) cloudmask(:,:)%applied_shadow= .true.

  cloudmask(:,:)%applied_thincirrus_ir = .false.
  where(packedcloudmaskQA(:,:,12) == 1) cloudmask(:,:)%applied_thincirrus_ir= .true.

  cloudmask(:,:)%applied_cloudadjacent = .false.
  where(packedcloudmaskQA(:,:,13) == 1) cloudmask(:,:)%applied_cloudadjacent= .true.

  cloudmask(:,:)%applied_irthreshold = .false.
  where(packedcloudmaskQA(:,:,14) == 1) cloudmask(:,:)%applied_irthreshold= .true.

  cloudmask(:,:)%applied_highcloudCO2 = .false.
  where(packedcloudmaskQA(:,:,15) == 1) cloudmask(:,:)%applied_highcloudCO2= .true.

  cloudmask(:,:)%applied_highcloud67 = .false.
  where(packedcloudmaskQA(:,:,16) == 1) cloudmask(:,:)%applied_highcloud67= .true.

  cloudmask(:,:)%applied_highcloud138 = .false.
  where(packedcloudmaskQA(:,:,17) == 1) cloudmask(:,:)%applied_highcloud138= .true.

  cloudmask(:,:)%applied_highcloud37_12 = .false.
  where(packedcloudmaskQA(:,:,18) == 1) cloudmask(:,:)%applied_highcloud37_12= .true.

  cloudmask(:,:)%applied_irtempdifference = .false.
  where(packedcloudmaskQA(:,:,19) == 1) cloudmask(:,:)%applied_irtempdifference= .true.

  cloudmask(:,:)%applied_39_11 = .false.
  where(packedcloudmaskQA(:,:,20) == 1) cloudmask(:,:)%applied_39_11= .true.

  cloudmask(:,:)%applied_visiblereflectance = .false.
  where(packedcloudmaskQA(:,:,21) == 1) cloudmask(:,:)%applied_visiblereflectance= .true.

  cloudmask(:,:)%applied_visible_ratio = .false.
  where(packedcloudmaskQA(:,:,22) == 1) cloudmask(:,:)%applied_visible_ratio= .true.

  cloudmask(:,:)%applied_restoral_ndvi= .false.
  where(packedcloudmaskQA(:,:,23) == 1) cloudmask(:,:)%applied_restoral_ndvi= .true.

  cloudmask(:,:)%applied_73_11_land_polar_night= .false.
  where(packedcloudmaskQA(:,:,24) == 1) cloudmask(:,:)%applied_73_11_land_polar_night= .true.

  cloudmask(:,:)%applied_temporal_consistency = .false.
  where(packedcloudmaskQA(:,:,25) == 1) cloudmask(:,:)%applied_temporal_consistency= .true.

  cloudmask(:,:)%applied_spatial_consistency = .false.
  where(packedcloudmaskQA(:,:,26) == 1) cloudmask(:,:)%applied_spatial_consistency= .true.

  cloudmask(:,:)%applied_clearskyrestoral_glint = .false.
  where(packedcloudmaskQA(:,:,27) == 1) cloudmask(:,:)%applied_clearskyrestoral_glint = .true.

  cloudmask(:,:)%applied_night_surfacetemp = .false.
  where(packedcloudmaskQA(:,:,28) == 1) cloudmask(:,:)%applied_night_surfacetemp = .true.

  cloudmask(:,:)%applied_suspended_dust = .false.
  where(packedcloudmaskQA(:,:,29) == 1) cloudmask(:,:)%applied_suspended_dust = .true.

  cloudmask(:,:)%applied_86_73_nightocean = .false.
  where(packedcloudmaskQA(:,:,30) == 1) cloudmask(:,:)%applied_86_73_nightocean = .true.

!  cloudmask(:,:)%applied_nightocean_var= .false.
!  where(packedcloudmaskQA(:,:,31) == 1) cloudmask(:,:)%applied_nightocean_var= .true.


  cloudmask(:,:)%applied_visible_cloudtest_11 = .false.
  where(packedcloudmaskQA(:,:,33) == 1) cloudmask(:,:)%applied_visible_cloudtest_11= .true.

  cloudmask(:,:)%applied_visible_cloudtest_12 = .false.
  where(packedcloudmaskQA(:,:,34) == 1) cloudmask(:,:)%applied_visible_cloudtest_12= .true.

  cloudmask(:,:)%applied_visible_cloudtest_13 = .false.
  where(packedcloudmaskQA(:,:,35) == 1) cloudmask(:,:)%applied_visible_cloudtest_13= .true.

  cloudmask(:,:)%applied_visible_cloudtest_14 = .false.
  where(packedcloudmaskQA(:,:,36) == 1) cloudmask(:,:)%applied_visible_cloudtest_14= .true.

  cloudmask(:,:)%applied_visible_cloudtest_21 = .false.
  where(packedcloudmaskQA(:,:,37) == 1) cloudmask(:,:)%applied_visible_cloudtest_21= .true.

  cloudmask(:,:)%applied_visible_cloudtest_22 = .false.
  where(packedcloudmaskQA(:,:,38) == 1) cloudmask(:,:)%applied_visible_cloudtest_22= .true.

  cloudmask(:,:)%applied_visible_cloudtest_23 = .false.
  where(packedcloudmaskQA(:,:,39) == 1) cloudmask(:,:)%applied_visible_cloudtest_23= .true.

  cloudmask(:,:)%applied_visible_cloudtest_24 = .false.
  where(packedcloudmaskQA(:,:,40) == 1) cloudmask(:,:)%applied_visible_cloudtest_24= .true.

  cloudmask(:,:)%applied_visible_cloudtest_31 = .false.
  where(packedcloudmaskQA(:,:,41) == 1) cloudmask(:,:)%applied_visible_cloudtest_31= .true.

  cloudmask(:,:)%applied_visible_cloudtest_32 = .false.
  where(packedcloudmaskQA(:,:,42) == 1) cloudmask(:,:)%applied_visible_cloudtest_32= .true.

  cloudmask(:,:)%applied_visible_cloudtest_33 = .false.
  where(packedcloudmaskQA(:,:,43) == 1) cloudmask(:,:)%applied_visible_cloudtest_33= .true.

  cloudmask(:,:)%applied_visible_cloudtest_34 = .false.
  where(packedcloudmaskQA(:,:,44) == 1) cloudmask(:,:)%applied_visible_cloudtest_34= .true.

  cloudmask(:,:)%applied_visible_cloudtest_41 = .false.
  where(packedcloudmaskQA(:,:,45) == 1) cloudmask(:,:)%applied_visible_cloudtest_41= .true.

  cloudmask(:,:)%applied_visible_cloudtest_42 = .false.
  where(packedcloudmaskQA(:,:,46) == 1) cloudmask(:,:)%applied_visible_cloudtest_42= .true.

  cloudmask(:,:)%applied_visible_cloudtest_43 = .false.
  where(packedcloudmaskQA(:,:,47) == 1) cloudmask(:,:)%applied_visible_cloudtest_43= .true.

  cloudmask(:,:)%applied_visible_cloudtest_44 = .false.
  where(packedcloudmaskQA(:,:,48) == 1) cloudmask(:,:)%applied_visible_cloudtest_44= .true.



! set cloud mask 1km QA for later storage
  processing_information(:,:)%cloud_mask_usefulness= 0
  where( cloudmask(:,:)%cloudmask_determined) processing_information(:,:)%cloud_mask_usefulness = 1_integer_onebyte
  where( cloudmask(:,:)%probablyclear_66) processing_information(:,:)%cloud_mask_unobs_fov = 1
  where( cloudmask(:,:)%probablyclear_95) processing_information(:,:)%cloud_mask_unobs_fov = 2
  where( cloudmask(:,:)%probablyclear_99) processing_information(:,:)%cloud_mask_unobs_fov = 3

  where( cloudmask(:,:)%day) 
    processing_information(:,:)%day_night = 1
  elsewhere
    processing_information(:,:)%day_night = 0
  endwhere

  where( cloudmask(:,:)%sunglint) 
    processing_information(:,:)%sun_glint = 0
  elsewhere
    processing_information(:,:)%sun_glint = 1
  endwhere

  where( cloudmask(:,:)%snowice_surface) 
    processing_information(:,:)%snow_ice = 0
  elsewhere
    processing_information(:,:)%snow_ice = 1
  endwhere

  processing_information(:,:)%surface_type = 0
  where( cloudmask(:,:)%coastal_surface) processing_information(:,:)%surface_type = 1
  where( cloudmask(:,:)%desert_surface) processing_information(:,:)%surface_type = 2
  where( cloudmask(:,:)%land_surface) processing_information(:,:)%surface_type = 3

  where( cloudmask(:,:)%shadow) 
    processing_information(:,:)%shadow = 0 
  elsewhere
    processing_information(:,:)%shadow = 1
  endwhere

  where(cloudmask(:,:)%thin_cirrus_solar) 
    processing_information(:,:)%thin_cirrus = 0
  elsewhere
    processing_information(:,:)%thin_cirrus = 1
  endwhere

  where( cloudmask(:,:)%noncloud_obstruction) 
    processing_information(:,:)%heavy_aerosol = 0 
  elsewhere
    processing_information(:,:)%heavy_aerosol = 1
  endwhere

end subroutine modis_unpackcloudmask

end module modis_cloudhandler
