 module multi_layer_clouds
!-------------------------------------------------------------------------------
! DESCRIPTION:
!        this module contains the subroutines needed for computation of 
!  the 0.94-based ancillary and the multi-layer map. 
!
!
! PROGRAMMER:
!            Gala Wind (L3 Comm GSI)
!            Climate and Radiation Branch
!            Code 913, NASA Goddard Space Flight Center
!            Greenbelt, Maryland 20771, U.S.A.
!
!            Mark A Gray (L3 Comm GSI)
!            Climate and Radiation Branch
!            Code 913, NASA Goddard Space Flight Center
!            Greenbelt, Maryland 20771, U.S.A.
!------------------------------------------------------------------------------
! REVISION:
! -------------------------------------------------------------------------------------------
!  6.7.04 Initial revision
!    7.04 corrected variable names for MODIS
!         implented logical cloud decision types
!         
!------------------------------------------------------------------------------
 use GeneralAuxType 
 use modis_cloudstructure
 use modis_numerical_module

 implicit none
 private
 
 ! there is only one public routine here
 public :: Compute_Multilayer_Map
 
 
 ! All these variables are private to this module.
 INTEGER, PARAMETER :: NumberOfPressureHeights = 10
 REAL, DIMENSION(NumberOfPressureHeights), PARAMETER :: &
 heightScale = (/ 100., 200., 300., 400., 500., 600., 700., 800., 900., 1000. /)

 INTEGER, PARAMETER :: NumberOfPrecipitableWater = 53
 REAL, DIMENSION(NumberOfPrecipitableWater), PARAMETER :: &
 pwScale = (/ 0.0, 0.02, 0.04, 0.06, 0.08, 0.1, 0.12, 0.14, 0.16, 0.18, 0.2, &
                    0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6,1.8, &
                   2.0, 2.2, 2.4, 2.6, 2.8, 3.0, 3.2, 3.4, 3.6, 3.8, 4.0, &
                   4.2, 4.4, 4.6, 4.8, 5.0, 5.2, 5.4, 5.6, 5.8, 6.0, 6.2, &
                   6.4, 6.6, 6.8, 7.0, 7.2, 7.4, 7.6, 7.8, 8.0, 8.2, 8.4, &
                   8.6 /)

 INTEGER, PARAMETER :: NumberOfMiu = 20 
 REAL, DIMENSION(NumberOfMiu), PARAMETER :: &
 miuscale = (/ 0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, &
              0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, 0.90, &
              0.95, 1.00 /) 

 real modis_bright, modis_planck
 external modis_bright, modis_planck

 
 contains



!=====================================
subroutine Compute_Multilayer_Map(platform_name, &
                                  BigTransTable,  &
                                  refl_65,        &
                                  refl_86,        &
                                  refl_94,        &
                                  refl_12,        &
                                  IIR_data,       &
                                  cloud_phase,    &
                                  Baum_phase,     &
                                  p_ncep,         &
                                  mixR_ncep,      &
                                  t_ncep,         &
                                  MOD06_Pc,       &
                                  MOD06_PW,       &
                                  miu,            &
                                  miu0,           &
                                  tau,            &
                                  mlayer) 
!=====================================

!-----------------------------------------------------------------------
! !f90
!
! !Description:
!     This subroutine computes the multi-layer cloud value for one pixel
!
! !Input Parameters:
!     BigTransTable -- real, dimension(:,:,:,:) -- entire transmittance table
!     refl_86       -- real -- 0.86um reflectance (band 2 in MODIS)
!     refl_94       -- real -- 0.94um reflectance (band 19 in MODIS)
!     cloud_phase   -- logical structure
!     Baum_phase    -- logical structure
!     p_ncep        -- real, dimension(:) -- NCEP pressure profile
!     mixR_ncep     -- real, dimension(:) -- NCEP mixing ratio profile
!     t_ncep        -- real, dimension(:) -- NCEP temperature profile
!     MOD06_Pc      -- real -- Cloud top pressure from MOD06CT
!     MOD06_PW      -- real -- above cloud precip. water derived from MOD06_Pc and NCEP by MOD06OD ancillary
!     IIR_data      -- real -- 11um radiance
!     miu           -- real -- cos of viewing angle
!     miu0          -- real -- cos of solar zenith angle
!     refl_65       -- real -- 0.65 um reflectance (band 1 in MODIS)
!     refl_12       -- real -- 1.2 um reflectance (band 5 in MODIS)
!     tau           -- real -- retrieved cloud optical depth
!
! !Output Parameters:
!     mlayer        -- integer*1 -- multi-layer cloud map for one pixel
!
! !Revision History:
!     2004/06/02 wind: Gala Wind
!     Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!      Written by:
!      Gala Wind
!   L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      wind@climate.gsfc.nasa.gov
!
! !Design Notes:
! NONE
!
! !END
!
!-----------------------------------------------------------------------

! first of all we need to compute the quantities that usually come from the research retrieval 
! code. That would be the total column water and 0.94 pw
  character*(*),intent(in)        ::  platform_name
  real(single), dimension(:,:,:,:), intent(in) :: BigTransTable
  real(single), intent(in)                     :: refl_86, refl_94, MOD06_Pc, MOD06_PW, &
                                                  IIR_data, miu, miu0, refl_65, refl_12, tau
  type(processflag), intent(in)                :: cloud_phase
  type(cloudphase),intent(in)                  :: Baum_phase
  real(single), dimension(:), intent(in)       :: p_ncep, t_ncep, mixR_ncep

  integer(integer_onebyte), intent(out)        :: mlayer


  real :: T, P, PW, TotalPrecipWater, newT, temp, PW_at_900
  logical :: FIRST
  real :: plant_ratio, pw_fraction, pw_fraction_900, ice_ratio
  integer(integer_onebyte) :: PW_test, PW_900_test, tau_test, cloud_phase_test
  
  
  ! Get the initial guess for brightness temperature
  T = modis_bright(platform_name,IIR_data,31,1)
  ! compute the initial pressure
  call Given_T_get_P(P, T, p_ncep, t_ncep)

  ! find the initial amount of precipitable water based on the initial guess 
  ! for the cloud top pressure computed above
  call find_TPW(refl_86, refl_94, P, PW,  BigTransTable, miu, miu0)

    FIRST = .TRUE.
    call find_brightness_T(platform_name,PW, P, newT, TotalPrecipWater, p_ncep, t_ncep, mixR_ncep,&
              IIR_data, BigTransTable, miu, FIRST)


    call Given_T_get_P(P, newT, p_ncep, t_ncep)
    
    ! do another iteration 
    call find_TPW(refl_86, refl_94, P, PW,  BigTransTable, miu, miu0)
    call find_brightness_T(platform_name,PW, P, newT, temp, p_ncep, t_ncep, mixR_ncep,&
              IIR_data, BigTransTable, miu, FIRST)

    T = newT


    ! now we're going to compute PW as if all the clouds were at 900 mb

    call find_TPW(refl_86, refl_94, 900., PW_at_900,  BigTransTable, miu, miu0)


    ! -----------------------------------------------------------------------
    !   NOW ANCILLARY PORTION IS FINISHED. STARTING LAYER DETECTION
    ! -----------------------------------------------------------------------
        
    plant_ratio = refl_86 / refl_65 ! vegetation ratio: 0.86/0.65 um
    pw_fraction = abs(MOD06_PW - PW) / TotalPrecipWater  ! precipitable water ratio
    pw_fraction_900 = abs(MOD06_PW - PW_at_900) / TotalPrecipWater ! additional precip water ratio
    ice_ratio = refl_86 / refl_12 ! ice ratio 0.86/1.2 um
    
    if (pw_fraction > 0.08 .and. MOD06_Pc < 550.0 .and. plant_ratio < 1.25 .and. ice_ratio < 1.3) then 
      PW_test = 1
    else 
      PW_test = 0
    endif 
    
    if (pw_fraction_900 > 0.08 .and. MOD06_Pc < 550.0 .and. plant_ratio < 1.25 .and. ice_ratio < 1.3) then 
      PW_900_test = 1
    else 
      PW_900_test = 0
    endif 

    if (tau < 4. .and. tau >= 0.) then 
      tau_test = 1
    else
      tau_test = 0
    endif
    
    ! cloud phase test: 0 if phases agree, 1 if SWIR-water and IR-ice or IR-mixed,
    ! 2 if SWIR-ice and IR-water
    cloud_phase_test = 0
    if (cloud_phase%watercloud .and. Baum_phase%watercloud ) cloud_phase_test = 0
    if (cloud_phase%icecloud .and. Baum_phase%icecloud) cloud_phase_test = 0

    if (cloud_phase%watercloud .and. &
                    (Baum_phase%mixedcloud .or. Baum_phase%icecloud)) cloud_phase_test = 1
    if (cloud_phase%icecloud .and. Baum_phase%watercloud) cloud_phase_test = 2
    

    mlayer = 1 !to 1
    if ( tau_test == 0) then 
      ! single tests: two PW tests and the cloud phase test alone
      if (PW_test == 1 .and. PW_900_test == 0) mlayer = 3 ! to 3
      if (PW_900_test == 1 .and. PW_test == 0) mlayer = 4 ! to 4
      if (cloud_phase_test > 0 .and. mlayer == 1) mlayer = 2 ! to 2
      
      ! combination tests
      ! both PW tests flagged the cloud
      if (PW_test == 1 .and. PW_900_test == 1 .and. cloud_phase_test == 0) mlayer = 5 !to 5
      ! all tests flagged cloud
      if (cloud_phase_test > 0 .and. PW_test == 1 .and. PW_900_test == 1) mlayer = 8  ! to 8 
      ! phase and PW test flagged
      if (cloud_phase_test > 0 .and. PW_test == 1 .and. PW_900_test == 0) mlayer = 6 ! to 6
      ! phase and PW900 test flagged
      if (cloud_phase_test > 0 .and. PW_900_test == 1 .and. PW_test == 0) mlayer = 7 ! to 7
    
    endif
    

  
  end subroutine Compute_Multilayer_Map
 
 
 !=====================================
    subroutine find_TPW(refl_86, refl_94, P, PW, BigTransTable, miu1, miu0)
!=====================================
!-----------------------------------------------------------------------
! !f90
!
! !Description:
!     This subroutine computes 0.94um-based precipitable water above cloud for a given pixel
!
! !Input Parameters:
!     refl_86 -- real*4 -- 0.86um reflectance (band 2 in MODIS)
!     refl_94 -- real*4 -- 0.94um reflectance (band 9 in MODIS)
!     P -- real*4 -- cloud top pressure in mb
!     BigTransTable -- real*4, dimension(:,:,:,:) -- transmittance table
!     miu1 -- real*4 -- cos of viewing angle
!     miu0 -- real*4 -- cos of solar zenith angle
!
! !Output Parameters:
!     PW -- real*4 -- precipitable water amount in cm
!
! !Revision History:
!     2004/06/02 wind: Gala Wind
!     Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!      Written by:
!      Gala Wind
!    L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      wind@climate.gsfc.nasa.gov
!
! !Design Notes:
!   NONE
!
! !END
!
!-----------------------------------------------------------------------

    implicit none
            
    real, intent(in) :: P, miu1, miu0, refl_86, refl_94
    real, intent(inout) :: PW
    real(single), dimension(:,:,:,:), intent(in) :: BigTransTable
        
    integer :: k
        
    real :: tpw
    real, dimension(:), allocatable :: trans2way


    integer, dimension(4) :: mindex

    integer :: miu_index, pindex, pwindex, miu2index, first_pIndex, second_pIndex
  integer :: bandindexmapsw1(2)

    REAL :: miu, transValue



    real, dimension(53) :: pix086, pix094, ans_diff
    real :: answer, trans_mid1, trans_mid2
    integer :: first_miuindex, second_miuindex
    real :: trans_p_1, trans_p_2 ! these are for the second pIndex
    real :: trans1, trans2

  bandindexmapsw1(1) = 2
  bandindexmapsw1(2) = 3


   pIndex = NINT( (P - heightscale(1)) / 100.0 ) + 1
   if(pIndex < 1) then
     pIndex = 1
     first_pIndex = 1
     second_pIndex = 1
   elseif(pIndex > NumberOfPressureHeights) then
     pIndex = NumberOfPressureHeights
     first_pIndex = pIndex
     second_pIndex = pIndex
   endif

   if (heightscale(pIndex) >= P) then 
     first_pIndex = pIndex-1
     second_pIndex = pIndex
   else
     first_pIndex = pIndex
     second_pIndex = pIndex+1
   endif
 
   if (first_pIndex == 0 ) then 
     first_pIndex = 1
     second_pIndex = 2
   endif

   ! key: compute effective miu for a two way path:

    miu = (miu0*miu1)/(miu0+miu1)

!
!... find array indicies along p, pw and miu dimension:
!
  
        miu2Index = NINT( (miu - miuscale(1)) / 0.05 ) + 1
        if (miu2Index < 1) then
            miu2Index = 1
        elseif(miu2Index > NumberOfMiu) then
            miu2Index = NumberOfMiu
        endif
        

!
!... 2-way case (for shortwave channels upto 3.7 micron band):
!

   mindex(1) = pindex
   mindex(2) = 1
   mindex(3) = miu2Index
   mindex(4) = bandindexmapsw1(1)

        if (miuscale(miu2index) >= miu ) then
                first_miuindex = miu2index -1 
                second_miuindex = miu2index 
        else
                first_miuindex = miu2index
                second_miuindex = miu2index + 1
        endif

    do k=1, numberofprecipitablewater
!     for the first pressure
!      call interpolate(miuscale(first_miuindex), &
!                       bigtransTable(first_pIndex, k, first_miuindex, bandindexmapsw1(1)), &
!                       miuscale(second_miuindex), &
!                       bigtransTable(first_pIndex, k, second_miuindex, bandindexmapsw1(1)), &
!                       miu, trans_mid1)
       trans_mid1 = linearinterpolation ( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
                    (/bigtransTable(first_pIndex, k, first_miuindex, bandindexmapsw1(1)), &
                      bigtransTable(first_pIndex, k, second_miuindex, bandindexmapsw1(1))/), &
                    miu)
      
!      call interpolate(miuscale(first_miuindex), &
!                       bigtransTable(first_pIndex, k, first_miuindex, bandindexmapsw1(2)), &
!                       miuscale(second_miuindex), &
!                       bigtransTable(first_pIndex, k, second_miuindex, bandindexmapsw1(2)), &
!                       miu, trans_mid2)

      trans_mid2 = linearinterpolation( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
                   (/bigtransTable(first_pIndex, k, first_miuindex, bandindexmapsw1(2)), &
                    bigtransTable(first_pIndex, k, second_miuindex, bandindexmapsw1(2))/), &
                    miu) 
!     for the second pressure
!      call interpolate(miuscale(first_miuindex), &
!                       bigtransTable(second_pIndex, k, first_miuindex, bandindexmapsw1(1)), &
!                       miuscale(second_miuindex), &
!                       bigtransTable(second_pIndex, k, second_miuindex, bandindexmapsw1(1)), &
!                       miu, trans_p_1)

      trans_p_1 = linearinterpolation( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
                   (/ bigtransTable(second_pIndex, k, first_miuindex, bandindexmapsw1(1)), &
                      bigtransTable(second_pIndex, k, second_miuindex, bandindexmapsw1(1)) /), &
                      miu)
!      call interpolate(miuscale(first_miuindex), &
!                       bigtransTable(second_pIndex, k, first_miuindex, bandindexmapsw1(2)), &
!                       miuscale(second_miuindex), &
!                       bigtransTable(second_pIndex, k, second_miuindex, bandindexmapsw1(2)), &
!                       miu, trans_p_2)
      trans_p_2 = linearinterpolation( (/ miuscale(first_miuindex),  miuscale(second_miuindex) /), &
                     (/ bigtransTable(second_pIndex, k, first_miuindex, bandindexmapsw1(2)), &
                        bigtransTable(second_pIndex, k, second_miuindex, bandindexmapsw1(2)) /), &
                        miu)
 
!     final interpolation now along the pressure axis
      trans1 = linearinterpolation ((/heightscale(first_pindex), heightscale(second_pindex)/), &
                                    (/trans_mid1, trans_p_1/), &
                                    P)   
      trans2 = linearinterpolation ((/heightscale(first_pindex), heightscale(second_pindex)/), &
                                    (/trans_mid2, trans_p_2/), &  
                                    P)
      if (trans_mid1 < 0. .or. trans_p_1 < 0.) trans1 = -1.0
      if (trans_mid2 < 0. .or. trans_p_2 < 0.) trans2 = -1.0
    
      pix086(k) = refl_86 / trans1
      pix094(k) = refl_94 / trans2
    
      if (trans1 < 0.0) pix086(k) = 20
      if (trans2 < 0.0) pix094(k) = 300
            
    end do

    ans_diff = abs(pix086-pix094)


    answer = minval(ans_diff)
    do k=1, numberofprecipitablewater
      if (ans_diff(k) == answer) then
        answer = pwscale(k)
        exit
      endif    
    end do
        
    PW = answer
    
    end subroutine find_TPW


!=====================================
  subroutine Given_T_get_P(P,T, p_ncep, t_ncep)
!=====================================

!-----------------------------------------------------------------------
! !f90
!
! !Description:
!     This subroutine gets NCEP cloud top pressure for a given 
!     cloud top temperature
!
! !Input Parameters:
!     T -- real*4 -- cloud top temperature in K
!     p_ncep -- real*4, dimension(:) -- NCEP pressure profile
!     t_ncep -- real*4, dimension(:) -- NCEP temperature profile
!
! !Output Parameters:
!     P -- real*4 -- cloud top pressure in mb
!
! !Revision History:
!     2004/06/02 wind: Gala Wind
!     Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!      Written by:
!      Gala Wind
!    L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      wind@climate.gsfc.nasa.gov
!
! !Design Notes:
!   NONE
!
! !END
!
!-----------------------------------------------------------------------
    real, intent(in)  ::  T
    real, intent(inout) :: P
    real, dimension(:), intent(in) :: p_ncep, t_ncep
    
  integer :: k
  real :: factor
   
  do k=1, 16
    if ( T > t_ncep(k)) &
      exit
  end do
    
  if (k == 1) then 
    P = p_ncep(1)
  elseif (k >= 17) then 
    P = p_ncep(16)
  else 
    factor = (T - t_ncep(k)) / (t_ncep(k-1) - t_ncep(k))
    P = p_ncep(k) + factor* (p_ncep(k-1) - p_ncep(k))
  endif

  if (P < 70.) P = 100. 


  end subroutine Given_T_get_P

 
 
!=====================================
    subroutine find_brightness_T(platform_name,PW, P, newT, TotalPrecipWater, p_ncep, t_ncep, mixR_ncep,&
              IIR_data, BigTransTable, miu, FIRST)
!=====================================

!-----------------------------------------------------------------------
! !f90
!
! !Description:
!     This subroutine computes the cloud top temperature for a given pixel
!
! !Input Parameters:
!     P -- real*4 -- cloud top pressure in mb
!     PW -- real*4 -- precipitable water amount in cm
!     p_ncep -- real*4, dimension(:) -- NCEP pressure profile
!     t_ncep -- real*4, dimension(:) -- NCEP temperature profile
!     mixR_ncep -- real*4, dimension(:) -- NCEP mixing ratio profile
!     IIR_data -- real*4 -- 11um radiance
!     BigTransTable -- real*4, dimension(:,:,:,:) -- transmittance table
!     miu -- real*4 -- cosine of viewing angle
!     FIRST -- logical -- flag for integration of TPW
!
! !Output Parameters:
!     newT -- real*4 -- cloud top temperature in K
!     TotalPrecipWater -- real*4 -- integrated NCEP water vapor column amount
!
! !Revision History:
!     2004/06/02 wind: Gala Wind
!     Revision 1.0 Initial Revision
!
! !Team-Unique Header:
!    Developed by the  Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA.
!
! !References and Credits:
!      Written by:
!      Gala Wind
!    L3 GSI
!      Code 913, NASA/GSFC
!      Greenbelt, MD 20771
!      wind@climate.gsfc.nasa.gov
!
! !Design Notes:
!   NONE
!
! !END
!
!-----------------------------------------------------------------------

  implicit NONE
  character*(*),intent(in)        ::  platform_name    
  real,    intent(in)                          :: PW, P, IIR_data, miu
  logical, intent(inout)                       :: FIRST
  real,    intent(inout)                       :: newT, TotalPrecipWater
  real(single), dimension(:), intent(in)       :: p_ncep, t_ncep, mixR_ncep
  real(single), dimension(:,:,:,:), intent(in) :: BigTransTable
        

  integer :: pIndex,  miu2index, pwIndex, first_miuindex, second_miuindex, k
  real ::  tpw, trans_mid1
  integer :: mindex(4), first_pIndex, second_pIndex
  real :: trans_p_1 ! these are for the second pIndex
  real :: trans1
  real :: total_PW, PW_layer, sum_T, slope,  mixR_lower, mixR_upper, T_lower, T_upper, T_layer
  real :: tmeanAbovecloud, tpwAboveCloud
  real :: B_11_crude
  real :: temp
  integer :: bandindexmaplw1(1)

  bandindexmaplw1(1) = 8

  pIndex = NINT( (P - heightscale(1)) / 100.0 ) + 1
  if(pIndex < 1) then
    pIndex = 1
    first_pIndex = 1
    second_pIndex = 1
  elseif(pIndex > NumberOfPressureHeights) then
    pIndex = NumberOfPressureHeights
    first_pIndex = pIndex
    second_pIndex = pIndex
  endif

  if (heightscale(pIndex) >= P) then 
    first_pIndex = pIndex-1
    second_pIndex = pIndex
  else
    first_pIndex = pIndex
    second_pIndex = pIndex+1
  endif

  if (first_pIndex == 0) then
    first_pIndex = 1
    second_pIndex = 2
  endif


!
!... find array indicies along p, pw and miu dimension:
!
   miu2Index = NINT( (miu - miuscale(1)) / 0.05 ) + 1
   if (miu2Index < 1) then
       miu2Index = 1
   elseif(miu2Index > NumberOfMiu) then
       miu2Index = NumberOfMiu
   endif


   if(pw < 0.0) then
       pwIndex = 1
   elseif(pw < 0.2) then
       pwIndex = NINT( (pw- pwScale(1)) / 0.02 ) + 1
   else
       pwIndex = NINT( (pw - pwScale(11)) / 0.20 ) + 11
       if(pwIndex > NumberOfPrecipitableWater) then
          pwIndex = NumberOfPrecipitableWater
       endif
   endif


   mindex(1) = pindex
   mindex(2) = pwIndex
   mindex(3) = miu2Index
   mindex(4) = bandindexmaplw1(1)

   if (miuscale(miu2index) >= miu ) then
     first_miuindex = miu2index -1 
     second_miuindex = miu2index 
   else
     first_miuindex = miu2index
     second_miuindex = miu2index + 1
   endif
!   call interpolate(miuscale(first_miuindex), bigtransTable(first_pIndex, pwindex, first_miuindex, bandindexmaplw1(1)), &
!        miuscale(second_miuindex), bigtransTable(first_pIndex, pwindex, second_miuindex, bandindexmaplw1(1)), &
!        miu, trans_mid1)

   trans_mid1 = linearinterpolation( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
                (/bigtransTable(first_pIndex, pwindex, first_miuindex, bandindexmaplw1(1)), & 
                   bigtransTable(first_pIndex, pwindex, second_miuindex, bandindexmaplw1(1))/), &
                miu) 


!   call interpolate(miuscale(first_miuindex), bigtransTable(second_pindex, pwindex, first_miuindex, bandindexmaplw1(1)), &
!        miuscale(second_miuindex), bigtransTable(second_pindex, pwindex, second_miuindex, bandindexmaplw1(1)), &
!        miu, trans_p_1)
   trans_p_1= linearinterpolation( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
               (/bigtransTable(second_pindex, pwindex, first_miuindex, bandindexmaplw1(1)),  &
                 bigtransTable(second_pindex, pwindex, second_miuindex, bandindexmaplw1(1))/), &
                miu)        


!    call interpolate(heightscale(first_pindex), trans_mid1, heightscale(second_pindex), trans_p_1, P, trans1)

    trans1 = linearinterpolation( (/heightscale(first_pindex), heightscale(second_pindex)/), &
                                  (/trans_mid1, trans_p_1/), &
                                   p)

    if (trans_mid1 >= 0. .and. trans_p_1 < 0.) trans1 = trans_p_1
    if (trans_mid1 < 0. .and. trans_p_1 >= 0.) trans1 = trans_mid1



! at this point we have the 1-way transmittance for the atmosphere at 11 microns and the 11 micron radiance
! stored in IIR_data that came from the read_mas_module.f90

! we can go ahead and find the amount of atmospheric correction for the 11 micron channel. 

 
      total_PW = 0.0
      sum_T  = 0.0


!     sep: start loop at NCEP's TOA level - 1
    
          
! add up the water profile in order to get the column amount later used for multi-layer cloud detection. 
      if (FIRST) then
     do k=1, 15
            PW_layer = (p_ncep(k)-p_ncep(k+1)) * (mixR_ncep(k+1) + mixR_ncep(k))*0.5/ 980.616 
            total_PW = total_PW+PW_layer
        end do
        TotalPrecipWater = total_PW
      endif
          
      
      PW_layer = 0.0
     total_PW = 0.0

      k = 16 - 1
      DO WHILE(p_ncep(k) < P .AND. k > 1)
      
      !     total precipitable water above cloud in g/cm2
      !   sep: dPW = rho*w*dz, dp=abs(rho*g*dz) or dz=dp/rho*g => dPW=w*dp/g
      !            {where w=layer mixing ratio, rho=density}
                        ! mixing ratio must be in g/kg, P in mb 1.22.04

         PW_layer = (p_ncep(k)-p_ncep(k+1)) * (mixR_ncep(k+1) + mixR_ncep(k))*0.5/ 980.616 

         T_layer  = (t_ncep(k)+t_ncep(k+1))*0.5
         total_PW = total_PW + PW_layer
         sum_T = sum_T + PW_layer*T_layer
                   
         k = k - 1
      END DO


      slope = (mixR_ncep(k) - mixR_ncep(k+1)) / (p_ncep(k) - p_ncep(k+1))
      mixR_upper = mixR_ncep(k+1)
      mixR_lower = slope * (P - p_ncep(k+1)) + mixR_upper
      PW_layer = (P - p_ncep(k+1))*(mixR_upper + mixR_lower)*0.5/980.616
      tpwAboveCloud = total_PW + PW_layer 

      slope = (t_ncep(k) - t_ncep(k+1)) / (p_ncep(k) - p_ncep(k+1))
      T_upper = t_ncep(k+1)
      T_lower = slope * (P - p_ncep(k+1)) + T_upper
      T_layer = (T_upper + T_lower)*0.5
      tmeanAbovecloud = (sum_T + PW_layer*T_layer)/tpwAboveCloud

! now we have the mean temperature weighted by the the precip. water
! now we apply the approximation

    B_11_crude = modis_planck(platform_name, tmeanAboveCloud, 31, 1) 
    if (trans1 < -0.0001) trans1 = 1.0
    B_11_crude = B_11_crude * (1. - trans1)   ! scale by 1. - 1waytransmittance

! got the amount of atmospheric correction


! now let's go ahead and subtract the correction from the measured radiance
    temp  = IIR_data - B_11_crude
! now we need to divide this mess by the transmittance that we computed earlier
    temp = temp / trans1

! now all this neat mess needs to be fed into the Planck function inversion
    newT = modis_bright(platform_name,temp,31,1)

    end subroutine find_brightness_T

 end module multi_layer_clouds
