###########################################################################
# !MAKE
#
# !MAKEFILE NAME:
#       MOD_PRDS.mk
#
# !DESCRIPTION:
#       Makefile for the MODIS L1B 1KM Destriping Algorithm (MOD_PRDS).
#
# !VARIABLES:
#       Internal
#        Macros            Definition 
#       ~~~~~~~~           ~~~~~~~~~~
#       ADD_CFLAGS         Process specific C compiler options
#       ADD_F77FLAGS       Process specific FORTRAN 77 compiler options
#       F90                FORTRAN 90 compiler definition in n32 mode
#       HDF5OPT_LIB        Path name of HDF5 dynamically shared library
#       INC                Accumulated include file search path
#       INC_FILE_SCIENCE   Include files developed by science team
#       INC_FILE_SHARED    Atmosphere shared include files 
#       LIB                Accumulated library search path
#       OBJ                Accumulated set of object files
#       PATH_SRC_SDST      Directory path to SDST L2 atmosphere shared source
#       PATH_SRC_UW        Directory path to UW-Madison shared source
#       TARGET             Executable file name
#
# !ENV VARIABLES
#         System
#       Environment 
#        Variables         Definition
#       ~~~~~~~~~~~        ~~~~~~~~~~
#       API_INC            Directory path to M-API include files
#       API_LIB            Directory path to M-API library
#       BRAND              Compiler brand. Current valids are "sgi64/sgi32/linux".
#                          "linux" denotes PGI compilers for FORTRAN code and GNU
#                          compilers for C.
#       CC                 C compiler definition
#       CFLAGS             C compiler options
#       F77                FORTRAN 77 compiler definition
#       F77FLAGS           FORTRAN 77 compiler options
#       HDFEOS_INC         Directory path to HDFEOS include files
#       HDFEOS_LIB         Directory path to HDFEOS library
#       HDFINC             Directory path to HDF include files
#       HDFLIB             Directory path to HDF library
#       HDF5LIB            Directory path to HDF5 libraries
#       PGSINC             Directory path to SDPTK include files
#       PGSLIB             Directory path to SDPTK library
#
#          User
#       Environment
#        Variables         Definition
#       ~~~~~~~~~~~        ~~~~~~~~~~
#       MODIS_STORE        Home directory of MODIS shared source code
#
# !TEAM-UNIQUE HEADER:
#       This software was developed by MODIS Science Data Support Team
#       for the National Aeronautics and Space Administration,
#       Goddard Space Flight Center, under contract NAS5-32373.
#
# !REVISION HISTORY:
#
# !NOTE:
#       1) Must follow MAPI, SDP TK, HDF, and other header/libs order
#       2) HDFLIB must be explicit and in order of -lmfhdf, -ldf, -ljpeg, -lz.
#       3) system environments, such as API_INC, PGSINC, HDFINC, PGSLIB,
#          API_LIB etc. should be defined before executing this makefile.
#       4) Compiler option "-byteswapio" must add to routine modis_ges101.f 
#          for the Linux PGI compiler.
# !END
###########################################################################

#BRAND = linux
#MODIS_STORE   = $(HOME)/OPS/MODIS

PATH_SRC_SDST = $(MODIS_STORE)/shared_src/atmos_src/src_L2
PATH_SRC_UW   = $(MODIS_STORE)/shared_src/atmos_src/src_UW

#---------------------------------------------------------------------------
#                  UW Development Environment
#
#F90 = pgf90
#F90FLAGS = -c -Mextend -Mdclchk
#ADD_F90FLAGS = -O0 -g -Mbounds
#ADD_F90FLAGS = -O2
#
#LINK_FLAGS = -Bstatic
#
# Libraries 
#LIB = -L$(PGSLIB) -lPGSTK -lm \
#      -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz
#
#---------------------------------------------------------------------------

ADD_F90FLAGS = -O2 -c -ansi -r10000
ADD_F77FLAGS = -O2 -c -u -bytereclen -col120 -woff 2271,2272,2274

# Libraries - MOD_PRDS is linked with the HDF5 static libraries instead of 
# dynamic libraries.  This ensures no conflicting usage of the HDF5 shared 
# libraries when switching between the _f90 and _f77 toolkit environments 
# used by PGE03.

LIB = -L$(PGSLIB)  -lPGSTK -lm \
      $(HDF5LIB)/libhdf5.a \
      -L$(HDFLIB)  -lmfhdf -ldf -ljpeg -lz


# Objects
OBJ = \
CONCATENATE.o \
hdf_destripe_new.o \
interp.o \
message.o \
modis_edf_destripe.o \
MODIS_SMF_SETDYNAMICMSG_V2.1.o \
strcompress.o \
STRING_LOC.o \
strlen.o

# Include file search path
INC = -I$(HDFINC) -I$(PGSINC)

# Include files developed by ST
INC_FILES_SCIENCE = MOD_PRDS.inc

# Compiler options are set based on IRIX/LINUX environment. On Linux,
# FORTRAN codes are compiled using LINUX PGI compilers; C codes use the
# GNU compilers 
target :
	case "$(BRAND)" in \
         linux ) \
           $(MAKE) $(TARGET) -f MOD_PRDS.mk \
                   ADD_CFLAGS="   -O2 -c -D_POSIX_SOURCE -funsigned-char" \
                   ADD_F77FLAGS=" -O2 -c -Mstandard" \
                   ADD_F90FLAGS=" -O2 -c -Mstandard" \
           ;; \
         * ) \
           $(MAKE) $(TARGET) -f MOD_PRDS.mk \
                   ADD_CFLAGS="   -O2 -c -D_POSIX_SOURCE" \
                   ADD_F77FLAGS=" -O2 -c -u -bytereclen -woff 2271,2272,2274 -col120" \
                   ADD_F90FLAGS=" -O2 -c -u -bytereclen -woff 2271,2272,2274 -ansi" \
           ;; \
       esac

#target :
#	case "$(BRAND)" in \
#         linux ) \
#           $(MAKE) $(TARGET) -f MOD_PRDS.mk \
#                   ADD_CFLAGS="   -O2 -c -D_POSIX_SOURCE -funsigned-char" \
#                   ADD_F77FLAGS=" -O2 -c -Mstandard" \
#                   ADD_F90FLAGS=" -O2 -c -Mstandard" \
#           ;; \
#         * ) \
#           $(MAKE) $(TARGET) -f MOD_PRDS.mk \
#                   ADD_CFLAGS="   -O2 -c -D_POSIX_SOURCE" \
#                   ADD_F77FLAGS=" -O2 -c -u -bytereclen -col120 -woff 2271,2272,2274" \
#                   ADD_F90FLAGS=" -O2 -c -u -bytereclen -col120 -woff 2271,2272,2274" \
#           ;; \
#       esac

# Define internal macros
TARGET = MOD_PRDS.exe

# Make executable
$(TARGET): $(OBJ)
	$(F90) $(LINK_FLAGS) $(OBJ) $(LIB) -o $@

# Clean up
clean:
	/bin/rm -f *.o *.exe

#---------------------------------------------------------
# Build objects derived from MOD_PRDS science code
#---------------------------------------------------------

hdf_destripe_new.o: hdf_destripe_new.f90 $(INC_FILES_SCIENCE)
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) hdf_destripe_new.f90

modis_edf_destripe.o: modis_edf_destripe.f90 $(INC_FILES_SCIENCE)
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(INC) modis_edf_destripe.f90

#----------------------------------------------------------------------------------
# Build object modules based on UW-Madison shared source library files 
#----------------------------------------------------------------------------------

interp.o: $(PATH_SRC_UW)/interp.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/interp.f

message.o: $(PATH_SRC_UW)/message.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/message.f

strcompress.o: $(PATH_SRC_UW)/strcompress.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strcompress.f

#----------------------------------------------------------------------------
# Build object modules based on SDST shared source library files 
#----------------------------------------------------------------------------

CONCATENATE.o: $(PATH_SRC_SDST)/CONCATENATE.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CONCATENATE.f

MODIS_SMF_SETDYNAMICMSG_V2.1.o: $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f

STRING_LOC.o: $(PATH_SRC_SDST)/STRING_LOC.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/STRING_LOC.f

strlen.o: $(PATH_SRC_SDST)/strlen.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/strlen.f

#********************** END OF MAKEFILE ***************************
