#!/bin/csh -f

#-------------------------------------------------------------------------------
# SETUP AND CHECK ARGUMENTS
#-------------------------------------------------------------------------------

# Check for ATMOS_HOME environment variable
if (! ${?ATMOS_HOME}) then
  echo "Error: Environment variable ATMOS_HOME is not set"
  exit 1
endif

# Set top level variables
set ROOT = $ATMOS_HOME
set PROD = MOD_PR07
set MDIR = $ROOT/$PROD

# Check arguments
if ($#argv != 5) then
  echo "Usage: run_mod07.csh SAT FIL1KM FILGEO FILCM GDAS"
  echo "where"
  echo "SAT    is the satellite name (terra or aqua)"
  echo "FIL1KM is the Level 1B 1000 meter radiance HDF file"
  echo "FILGEO is the Level 1B 1000 meter geolocation HDF file"
  echo "FILCM  is the Level 2 cloud mask HDF file"
  echo "GDAS is the GDAS1 GRIB file"
  exit 1
endif

# Extract arguments
set SAT     = $argv[1]
set FIL1KM  = `basename $argv[2]`
set FILGEO  = `basename $argv[3]`
set FILCM   = `basename $argv[4]`
set FILGDAS = `basename $argv[5]`
set DIR1KM  = `dirname  $argv[2]`
set DIRGEO  = `dirname  $argv[3]`
set DIRCM   = `dirname  $argv[4]`
set DIRGDAS = `dirname  $argv[5]`

# Get platform header (MOD or MYD)
if ($SAT == "terra") then
  set HEADER = "MOD"
else if ($SAT == "aqua") then
  set HEADER = "MYD"
else
  echo "Satellite name not recognized: "$SAT
  exit 1
endif

#-------------------------------------------------------------------------------
# CREATE LINKS TO STATIC INPUT FILES
#-------------------------------------------------------------------------------

ln -f -s $MDIR/coeff/${HEADER}*.mcf .
ln -f -s $MDIR/coeff/thresholds.dat.$SAT.* .
ln -f -s $MDIR/coeff/goge1_2_img.v1 .
ln -f -s $ROOT/toolkit/data/leapsec.dat .
ln -f -s $ROOT/toolkit/data/utcpole.dat .

#-------------------------------------------------------------------------------
# CREATE PCF FILE
#-------------------------------------------------------------------------------

# Get date and time string from L1B 1KM meter file
set DATE_TIME = `echo $FIL1KM:t | cut -d. -f2,3`

# Set current date/time
set CURRENT = `date -u +%Y%j%H%M%S`

# Set name of template file
set TEMPLATE = $MDIR/template/${HEADER}_PR35.pcf.template

# Set name of new PCF file
set FILPCF = ${HEADER}_PR35.$DATE_TIME.pcf

# Set names of output files
set FILOUT_L2 = ${HEADER}35_L2.$DATE_TIME.hdf
set FILOUT_QC = ${HEADER}35_L2.$DATE_TIME.QC
set FILOUT_CS = ${HEADER}CSR_G.$DATE_TIME.hdf

# Set dummy start and stop times for PCF
set START_TIME = '2000-01-01T00:00:00'
set STOP_TIME = `date -u +%Y-%m-%dT%H:%M:%S`

# Set platform name
set PLATFORM_NAME = `uname -a`

# Create new PCF file from the template
sed \
  -e "s?FIL1KM?${FIL1KM}?g" \
  -e "s?FILQKM?${FILQKM}?g" \
  -e "s?FILGEO?${FILGEO}?g" \
  -e "s?FILGDAS?${FILGDAS}?g" \
  -e "s?FILRSST?${FILRSST}?g" \
  -e "s?FILNISE?${FILNISE}?g" \
  -e "s?FILICEC?${FILICEC}?g" \
  -e "s?DIR1KM?${DIR1KM}?g" \
  -e "s?DIRQKM?${DIRQKM}?g" \
  -e "s?DIRGEO?${DIRGEO}?g" \
  -e "s?DIRGDAS?${DIRGDAS}?g" \
  -e "s?DIRRSST?${DIRRSST}?g" \
  -e "s?DIRNISE?${DIRNISE}?g" \
  -e "s?DIRICEC?${DIRICEC}?g" \
  -e "s?FILOUT_L2?${FILOUT_L2}?g" \
  -e "s?FILOUT_QC?${FILOUT_QC}?g" \
  -e "s?FILOUT_CS?${FILOUT_CS}?g" \
  -e "s?START_TIME?${START_TIME}?g" \
  -e "s?STOP_TIME?${STOP_TIME}?g" \
  -e "s?PLATFORM_NAME?${PLATFORM_NAME}?g" \
  $TEMPLATE >! $FILPCF

#-------------------------------------------------------------------------------
# RUN THE ALGORITHM
#-------------------------------------------------------------------------------

# Set toolkit environment variables
setenv PGS_PC_INFO_FILE ./$FILPCF
setenv PGSMSG $ROOT/toolkit/message

# Print start message
echo $PROD "processing started at "`date`

# Run the algorithm
$MDIR/bin/$PROD.exe
set PGE_STATUS = $status

# Clean up
if ($PGE_STATUS == 0) then
  rm -f $FILOUT_L2.met $FILOUT_QC $FILOUT_QC.met
  rm -f $FILPCF ShmMem LogUser LogReport LogStatus GetAttr.temp
  rm -f ${HEADER}*.mcf
  rm -f thresholds.dat.$SAT.*
  rm -f goge1_2_img.v1
  rm -f leapsec.dat utcpole.dat
  rm -f pc*
endif

# Print failure or success message
if ($PGE_STATUS != 0) then
  echo "Error encountered"
  exit 1
else
  echo "Successful completion"
endif

# Print finish message
echo $PROD "processing ended at  "`date`
exit 0
