module Level2FileInfo
  use typeSizes,     only: FourByteIntKind, EightByteRealKind
  use ErrorHandling, only: FatalErrorMessage
  use hdf,           only: SFrnatt, SFfattr, FAIL, MAX_NC_NAME, MAX_VAR_DIMS
  use hdfUtils,      only: getDataSetID, getDataSetDimSizes, endDataSet
  implicit none
  
  private

  public :: getCalibrationInfo, getSpatialSamplingInfo, setSpatialSampling, &
            getGranuleSize
contains
  ! --------------------------------------------------------------------------------
  subroutine getCalibrationInfo(dataSetId, scaleFactor, addOffset)
    integer (kind = FourByteIntKind),   intent( in) :: dataSetID
    real    (kind = EightByteRealKind), intent(out) :: scaleFactor, addOffset
    character (len = 2048)                          :: msgbuffer

    ! !F90
    !
    ! !Description:
    !   Reads the scale_factor and add_offset attributes from a Modis HDF SDS. Both
    !     are assumed to be eight byte reals. Error handling is internal (any mistakes
    !     and execution stops). 
    !
    ! !Input Parameters:
    !   dataSetId: HDF data set ID, as returned by SFstart or getDataSetID
    !
    ! !Output Parameters:
    !   scaleFactor, offset: eight byte real values read from scale_factor and add_offset 
    !     data set attributes. 
    ! 
    ! !Revision History:
    !   $Log: Level2FileInfo.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  16:02:45  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/10  20:14:47  pincus
    ! Removed tabs to ensure ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:44:27  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   Named constants:
    !     FAIL: (hdf.f90)
    !
    ! !END
  
    ! Local variable
    integer :: status

    ! Read scale factor and offset, which are real valued attributes for all variables.
    status = SFrnatt(dataSetID, SFfattr(dataSetID, "scale_factor"), scaleFactor)

    if (status == FAIL) then
      msgbuffer = "Error reading scale factor." &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
      call FatalErrorMessage("getCalibrationInfo", msgbuffer)
    end if

    status = SFrnatt(dataSetID, SFfattr(dataSetID, "add_offset"), addOffset)

    if (status == FAIL) then
      msgbuffer = "Error reading offset" &
  // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
  // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
  // char(10) // "notify SDST. "
      call FatalErrorMessage("getCalibrationInfo", msgbuffer)
    end if

  end subroutine getCalibrationInfo
  ! --------------------------------------------------------------------------------
  subroutine getSpatialSamplingInfo(dataSetID,                                               &
                                CellAcrossFirstPixel, CellAcrossLastPixel, CellAcrossStride, &
                                CellAlongFirstPixel,  CellAlongLastPixel,  CellAlongStride) 
    integer (kind = FourByteIntKind),           intent( in) :: dataSetID
    integer,                          optional, intent(out) ::                               &
                                CellAcrossFirstPixel, CellAcrossLastPixel, CellAcrossStride, &
                                CellAlongFirstPixel,  CellAlongLastPixel,  CellAlongStride   
    character (len = 2048)               :: msgbuffer

  ! !F90
    !
    ! !Description:
    !   Reads the Cell_Across_Swath_Sampling and Cell_Along_Swath_Sampling attributes from 
    !     Modis Level 2 data, returns any or all of the values in the attribute.
    !  
    ! !Input Parameters:
    !   dataSetID: HDF data set ID, as obtained from SFselect of getDataSetID.
    !
    ! !Output Parameters:
    !   Note: all output parmaters are optional. 
    !   CellAcrossFirstPixel, CellAcrossLastPixel, CellAcrossStride: the first, second, and 
    !     third elements of the size 3, rank 1 integer array stored in the 
    !     Cell_Across_Swath_Sampling attribute. 
    !   CellAlongFirstPixel,  CellAlongLastPixel,  CellAlongStride: the first, second, and
    !     third elements of the size 3, rank 1 integer array stored in the
    !     Cell_Along_Swath_Sampling attribute.
    ! 
    ! !Revision History:
    !   $Log: Level2FileInfo.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  16:02:45  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/10  20:14:47  pincus
    ! Removed tabs to ensure ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:44:27  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   If the HDF attributes are ever changed this subroutine won't work any more. 
    !
    ! !END
    
    ! Local variables
    integer                                        :: status
    integer (kind = FourByteIntKind), dimension(3) :: CellAcrossSampling, CellAlongSampling

    status = SFrnatt(dataSetID, &
                     SFfattr(dataSetID, "Cell_Across_Swath_Sampling"), CellAcrossSampling)

    if (status == FAIL) then
      msgbuffer = "Error reading Cell Across Swath Sampling" &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
      call FatalErrorMessage("getSpatialSamplingInfo", msgbuffer)
    end if

    if(present(CellAcrossFirstPixel)) CellAcrossFirstPixel = CellAcrossSampling(1)
    if(present(CellAcrossLastPixel )) CellAcrossLastPixel  = CellAcrossSampling(2)
    if(present(CellAcrossStride    )) CellAcrossStride     = CellAcrossSampling(3)

    status = SFrnatt(dataSetID, &
                     SFfattr(dataSetID, "Cell_Along_Swath_Sampling"), CellAlongSampling)

    if (status == FAIL) then
      msgbuffer = "Error reading Cell Along Swath Sampling" &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
      call FatalErrorMessage("getSpatialSamplingInfo", msgbuffer)
    end if

    if(present(CellAlongFirstPixel )) CellAlongFirstPixel  = CellAlongSampling(1)
    if(present(CellAlongLastPixel  )) CellAlongLastPixel   = CellAlongSampling(2)
    if(present(CellAlongStride     )) CellAlongStride      = CellAlongSampling(3)

  end subroutine getSpatialSamplingInfo
  ! --------------------------------------------------------------------------------
  subroutine setSpatialSampling(GeorefCellAcrossStride, GeorefCellAlongStride, &
                                CellAcrossStride, CellAlongStride, start, stride, dataSetName)
    integer, intent( in) :: GeorefCellAcrossStride, GeorefCellAlongStride, &
                                  CellAcrossStride,       CellAlongStride
    integer (kind = FourByteIntKind),                                      &
                     dimension(2), intent(out) :: start, stride
    character (len = *), optional, intent( in) :: dataSetName
    ! !F90
    !
    ! !Description:
    !    Given the sampling density of geolocation data and a second parameter, sets
    !      the values of start(:) and stride(:) (arrays which define the 
    !      way the HDF SDS gets read) such that the second parameter will be read
    !      at the same spatial resolution as the geolocation. Edge(:), which is also 
    !      needed by the HDF calls, has to be computed in the calling program (it can 
    !      be set to the size of the geolocation arrays). 
    !  
    ! !Input Parameters:
    !    GeorefCellAcrossStride, CellAcrossStride: spatial resolution of the geolocation and 
    !      second parameter in the first dimension. 
    !    GeorefCellAlongStride, CellAlongStride: spatial resolution of the geolocation and 
    !      second parameter in the second dimension.
    !    dataSetName: character string with the data set name, used to make error messages
    !      more informative. Optional. 
    !
    ! !Output Parameters:
    !    start: Follows HDF conventions. The array element from which to begin reading the 
    !      HDF array. Zero based. 
    !    stride: Follows HDF conventions. The frequency with which data should be read in 
    !      each dimension. Stride 1 means read every element. 
    ! 
    ! !Revision History:
    !   $Log: Level2FileInfo.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  16:02:45  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/10  20:14:47  pincus
    ! Removed tabs to ensure ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:44:27  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    The second parameter must have spatial resolution greater than 
    !      or equal to the geolocation. 
    !
    ! !END
  
    ! Local variables
    character (len = MAX_NC_NAME) :: thisDataSetName = ""
    character (len = 2048)               :: msgbuffer

    if(present(dataSetName)) thisDataSetName = dataSetName

    if(all( (/ GeorefCellAcrossStride, GeorefCellAlongStride /) == &
             (/      CellAcrossStride,       CellAlongStride /) ) ) then
      start (:2) = 0
      stride(:2) = 1
    else 
      ! Sanity checks on the spatial sampling - the sampling density for this
      !   dataset must be an integer multiple of the geolocation. 
      !   That is, datasets are allowed to be more densely sampled than 
      !   geolocation, but not less densely. 

      if( (CellAcrossStride > GeorefCellAcrossStride) .or. &
          (CellAlongStride  > GeorefCellAlongStride )) then
          msgbuffer = "Sampling density for dataset " // &
                       trim(thisDataSetName) // &
                       " greater than geolocation." &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
        call FatalErrorMessage("setSpatialSampling", msgbuffer)
       end if
  
      if(mod(GeorefCellAcrossStride, CellAcrossStride) /= 0 .or.  &
         mod(GeorefCellAlongStride,  CellAlongStride ) /= 0 ) then
         msgbuffer = "Sampling density for dataset " // &
                      trim(thisDataSetName)           // &
                      " not an integer divisor of geolocation." &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
         call FatalErrorMessage("setSpatialSampling", msgbuffer)
       end if

      stride(:2) = (/ GeorefCellAcrossStride/CellAcrossStride, &
                      GeorefCellAlongStride /CellAlongStride /)
      start (:2) = stride(:2) / 2
    end if
  end subroutine setSpatialSampling
! --------------------------------------------------------------------------------
  function getGranuleSize(fileID, fileName)
    integer (kind = FourByteIntKind), intent( in) :: fileID
    character (len = *),              intent( in) :: fileName
    integer, dimension(2)                         :: getGranuleSize
    ! !F90
    !
    ! !Description:
    !   Returns a two element array containing the sizes of the first two dimensions
    !     of the geolocation arrays in a Level 2 granule file.
    !  
    ! !Input Parameters:
    !   fileID: HDF file ID, as opened in the calling procedure
    !   fileName: the name of the file associated with fileID; used for error reporting.
    !
    ! !Output Parameters:
    !    The function returns an integer vector of length 2 containing the sizes of the
    !      first two dimensions of the "Latitude" data set in the file. 
    ! 
    ! !Revision History:
    !   $Log: Level2FileInfo.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  16:02:45  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/10  20:14:47  pincus
    ! Removed tabs to ensure ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:44:27  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
      
    ! Local variables
    integer (kind = FourByteIntKind) :: LatID
    integer, dimension(MAX_VAR_DIMS) :: dimensionSizes
    integer                          :: status

  
    LatID = getDataSetID(fileID, "Latitude")
    dimensionSizes(:) = getDataSetDimSizes(dataSetId   = LatID,           &
                                           fileName    = trim(fileName),  &
                                           dataSetName = "Latitude" )
    call             endDataSet(LatID)
    getGranuleSize = dimensionSizes(1:2)
  
  end function getGranuleSize
  ! --------------------------------------------------------------------------------
end module Level2FileInfo
