module Level2Read  
  ! This module contains arrays and functions needed to read Modis Level 2 
  !   data from an HDF file. 
  use typeSizes
  use ErrorHandling,  only: FatalErrorMessage
  use hdf,            only: SFrdata, SFrnatt, SFfattr, SFginfo, SFgdinfo, SFdimid, &
                            DFNT_INT8, DFNT_INT16, DFNT_INT32, DFNT_FLOAT32,       &
                            FAIL, MAX_VAR_DIMS, MAX_NC_NAME
  use hdfUtils,       only: getDataSetType, getDataSetDimSizes
  use Level2FileInfo, only: getCalibrationInfo
  implicit none
  
  private

  ! Define a value to be used throughout the Atmospheres Daily processing code
  !   to represent missing values. It needs to be small enough to be represented as 
  !   a default integer. 
  integer                 :: defaultInteger
  real, public, parameter :: ModisL3FillValue = -huge(defaultInteger)

  !  Arrays for Level 2 data. They will get allocated to the maximum size needed, 
  !  as specified by the file attributes. 
  integer (kind =  OneByteIntKind ), dimension(:, :), allocatable, target ::  oneByteIntL2Data
  integer (kind =  TwoByteIntKind ), dimension(:, :), allocatable, target ::  twoByteIntL2Data
  integer (kind = FourByteIntKind ), dimension(:, :), allocatable, target :: fourByteIntL2Data
  real    (kind = FourByteRealKind), dimension(:, :), allocatable, target :: fourByteRealL2Data

  ! Arrays for subsampled Level 2 data. 
  integer (kind =  OneByteIntKind ), dimension(:, :), pointer ::  oneByteIntL2DataSample
  integer (kind =  TwoByteIntKind ), dimension(:, :), pointer ::  twoByteIntL2DataSample
  integer (kind = FourByteIntKind ), dimension(:, :), pointer :: fourByteIntL2DataSample
  real    (kind = FourByteRealKind), dimension(:, :), pointer :: fourByteRealL2DataSample
  !
  integer (kind =  OneByteIntKind ) ::  oneByteIntFillValue
  integer (kind =  TwoByteIntKind ) ::  twoByteIntFillValue
  integer (kind = FourByteIntKind ) :: fourByteIntFillValue
  real    (kind = FourByteRealKind) :: fourByteRealFillValue
  
  public :: readOneL2DataSet, readOneL2IntDataSet, readOneL2BitField
  
contains
  ! ----------------------------------------------------------------------------------
  function calibrate(inputArray, offset, scaleFactor) result(outputArray)
    real, dimension(:, :), intent( in) :: inputArray
    real (kind = EightByteRealKind), &
                           intent( in) :: offset, scaleFactor
    real, dimension(size(inputArray, 1), &
                    size(inputArray, 2)) :: outputArray
    ! !F90
    !
    ! !Description:
    !   Applys scale factor and offset correction to a real-valued array (as
    !   would be read from an HDF file) to return a correctly calibrated array.
    !  
    ! !Input Parameters:
    !    inputArray: real valued rank two array. Typically this contains real values 
    !      as read from an HDF file. 
    !    offset: offset, as would be read from an HDF SDS attribute. 
    !    scaleFactor: linear scaling factor, as would be read from an HDF SDS attribute.  
    !
    ! !Output Parameters:
    !   The function returns an array of rank two of the same size as inputArray, with 
    !     the scale factor and offset applied. 
    !
    ! !Revision History:
    !   $Log: Level2Read.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.4  1997/12/30  16:02:36  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.3  1997/11/13  15:15:51  pincus
    ! Added support for reading bit field values from a two dimensional SDS.
    ! (Previously, all bit field SDSs were supposed to have three dimensions.)
    !
    ! Revision 1.2  1997/11/10  20:33:22  pincus
    ! Removed tabs for ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:46:07  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   Note that this definition of how to apply the calibration information 
    !     is consistent with the HDF convention. 
    !  Both offset and scaleFactor are optional arguments. 
    !   
    !
    ! !END
    outputArray(:, :) = inputArray(:, :)
    if (abs(offset) > tiny(offset)) &
      outputArray = outputArray - offset
    if (abs(scaleFactor - 1.0) > tiny(scaleFactor)) &
      outputArray = outputArray * scaleFactor
  end function calibrate
  ! ----------------------------------------------------------------------------------
  function readOneL2DataSet(dataSetID, fileName, dataSetName, &
                            start, stride, edge, plane)
    integer (kind = FourByteIntKind), intent( in) :: dataSetID
    character (len = *),              intent( in) :: fileName, dataSetName
    integer (kind = FourByteIntKind), dimension(2), &
                                      intent( in) :: start, stride, edge
    integer (kind = FourByteIntKind), intent( in) :: plane
    real (kind = FourByteRealKind), &
                      dimension(edge(1), edge(2)) :: readOneL2DataSet

    ! !F90
    !
    ! !Description:
    !   Read a two dimensional matrix from an HDF file, convert to real, and replace
    !     fill values with ModisLevel3FillValue. Variables aren't allowed to have more 
    !     than three dimensions. We read along the first two, with the third dimension 
    !     specified by the "plane" input argument. 
    !  
    ! !Input Parameters:
    !   dataSetID: HDF data set ID, as provided by getDataSetID or SFselect
    !   fileName, dataSetName: character strings containing the name of the dataset
    !     being read and the file from which it is being read; used to make error
    !     reports more useful. 
    !    start: Follows HDF conventions. The array element from which to begin reading the
    !      HDF array. Zero based.
    !    stride: Follows HDF conventions. The frequency with which data should be read in
    !      each dimension. Stride 1 means read every element.
    !    edge: Follows HDF conventions. The number of elements to read in each dimension. If
    !      start(:) == 0 and stride(:) == 1, setting edge equal to the shape of the underlying
    !      array reads all elements. 
    !    plane: integer determining which (single) array section in the third dimension to read.
    !
    ! !Output Parameters:
    !   Function returns a real valued array of shape (edge(1), edge(2)) containing
    !     Level 2 data, calibrated to physical units, with all missing values to to the
    !     parameter ModisL3FillValue. 
    ! 
    ! !Revision History:
    !   $Log: Level2Read.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.4  1997/12/30  16:02:36  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.3  1997/11/13  15:15:51  pincus
    ! Added support for reading bit field values from a two dimensional SDS.
    ! (Previously, all bit field SDSs were supposed to have three dimensions.)
    !
    ! Revision 1.2  1997/11/10  20:33:22  pincus
    ! Removed tabs for ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:46:07  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   This procedure makes extensive use of allocatable arrays declared in the 
    !     data portion of the module. We discover the type of the uncalibrated 
    !     data, allocate the appropriate array, read in the data, calibrate it, 
    !     and deallocate the temporary array. 
    !   Rather than read discontiguous sections of the arrays from the HDF files, 
    !     which is painfully slow, we read all the data and extract the needed 
    !     values from within memory. We know that we'll always read a single plane in 
    !     the third dimension. 
    !
    ! !END

    ! Local variables
    integer :: status
    real    (kind = EightByteRealKind) :: scaleFactor, addOffset
    integer (kind = FourByteIntKind  ), &
               dimension(MAX_VAR_DIMS) :: hdfStart, hdfStride, hdfEdge
    character (len = 2048)             :: msgbuffer

    call getCalibrationInfo(dataSetId, scaleFactor = scaleFactor, addOffset = addOffset)
    hdfStart (:2) = 0;
    hdfStart ( 3) = plane
    hdfStride(:3) = 1
    hdfEdge  (: ) = getDataSetDimSizes(dataSetID)
    hdfEdge  ( 3) = 1

    ! Read data. Need to read different types and normalize to real values with 
    !   Level 3 fill values where appropriate. 
    select case (getDataSetType(dataSetID, fileName, dataSetName))
      case(DFNT_INT8)
        ! Allocate memory for all values and read them.
        allocate(oneByteIntL2Data(hdfEdge(1), hdfEdge(2)))
        status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, oneByteIntL2Data)

        if (status == FAIL) then 
          msgbuffer = "Error reading data from dataset " // &
                 trim(dataSetname) // " in file " // trim(fileName) &
                 // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
                 // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                 // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2DataSet", msgbuffer)
        end if

        ! Read the Fill Value attribute.
        status = SFrnatt(dataSetID, SFfattr(dataSetID, "_FillValue"), oneByteIntFillValue)

        if (status == FAIL) then
          msgbuffer = "Error reading fill value from dataset " // &
               trim(dataSetname) // " in file " // trim(fileName) &
               // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
               // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
               // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2DataSet", msgbuffer)
        end if

        ! Identify the subsampled data. 
        oneByteIntL2DataSample => oneByteIntL2Data((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                                   (start(2) + 1):(edge(2) * stride(2)):stride(2) )
        ! Calibrate the subsampled data, and apply the Level 3 Fill Value. 
        readOneL2DataSet = calibrate(inputArray  = real(oneByteIntL2DataSample), &
                                     offset      = addOffset,                    &
                                     scaleFactor = scaleFactor)
        where (oneByteIntL2DataSample == oneByteIntFillValue) &
          readOneL2DataSet = ModisL3FillValue
        nullify(oneByteIntL2DataSample)
        deallocate(oneByteIntL2Data)

      case(DFNT_INT16)
        allocate(twoByteIntL2Data(hdfEdge(1), hdfEdge(2)))
        status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, twoByteIntL2Data)

        if (status == FAIL) then
          msgbuffer = "Error reading data from dataset " // &
             trim(dataSetname) // " in file " // trim(fileName) &
            // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
            // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
            // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2DataSet", msgbuffer)
        end if

        status = SFrnatt(dataSetID, SFfattr(dataSetID, "_FillValue"), twoByteIntFillValue)

        if (status == FAIL) then
          msgbuffer = "Error reading fill value from dataset " // &
                trim(dataSetname) // " in file " // trim(fileName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2DataSet", msgbuffer)
        end if

        twoByteIntL2DataSample => twoByteIntL2Data((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                                   (start(2) + 1):(edge(2) * stride(2)):stride(2) )
        readOneL2DataSet = calibrate(inputArray  = real(twoByteIntL2DataSample), &
                                     offset      = addOffset,                    &
                                     scaleFactor = scaleFactor)
        where (twoByteIntL2DataSample == twoByteIntFillValue) &
          readOneL2DataSet = ModisL3FillValue
        nullify(twoByteIntL2DataSample)
        deallocate(twoByteIntL2Data)

      case(DFNT_INT32)
        allocate(fourByteIntL2Data(hdfEdge(1), hdfEdge(2)))
        status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, fourByteIntL2Data)

        if (status == FAIL) then
          msgbuffer = "Error reading data from dataset " // &
            trim(dataSetname) // " in file " // trim(fileName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2DataSet", msgbuffer)
        end if 

        status = SFrnatt(dataSetID, SFfattr(dataSetID, "_FillValue"), fourByteIntFillValue)

         if (status == FAIL) then
           msgbuffer = "Error reading fill value from dataset " // &
               trim(dataSetname) // " in file " // trim(fileName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
           call FatalErrorMessage("readOneL2DataSet", msgbuffer)
         end if

        fourByteIntL2DataSample => fourByteIntL2Data((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                                     (start(2) + 1):(edge(2) * stride(2)):stride(2) )
        readOneL2DataSet = calibrate(inputArray  = real(fourByteIntL2DataSample), &
                                     offset      = addOffset,                     &
                                     scaleFactor = scaleFactor)
        where (fourByteIntL2Data == fourByteIntFillValue) &
          readOneL2DataSet = ModisL3FillValue
        nullify(twoByteIntL2DataSample)
        deallocate(fourByteIntL2Data)

      case(DFNT_FLOAT32)
        allocate(fourByteRealL2Data(hdfEdge(1), hdfEdge(2)))
        status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, fourByteRealL2Data)

        if (status == FAIL) then
          msgbuffer = "Error reading data from dataset " // &
             trim(dataSetname) // " in file " // trim(fileName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2DataSet", msgbuffer)
        end if

        status = SFrnatt(dataSetID, SFfattr(dataSetID, "_FillValue"), fourByteRealFillValue)

        if (status == FAIL) then
          msgbuffer = "Error reading fill value from dataset " // &
            trim(dataSetname) // " in file " // trim(fileName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2DataSet", msgbuffer)
        end if

        fourByteRealL2DataSample => fourByteRealL2Data((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                                       (start(2) + 1):(edge(2) * stride(2)):stride(2) )
        readOneL2DataSet = calibrate(inputArray  = fourByteRealL2DataSample, &
                                     offset      = addOffset,                &
                                     scaleFactor = scaleFactor)
        where (abs(readOneL2DataSet - fourByteRealFillValue) <= &
               spacing(fourByteRealFillValue))                 &
          readOneL2DataSet = ModisL3FillValue
        nullify(fourByteRealL2DataSample)
        deallocate(fourByteRealL2Data)

      case default
        msgbuffer = "Unrecognized data type for parameter " // &
          trim(dataSetname) // " in file " // trim(fileName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
        call FatalErrorMessage("ReadOneL2DataSet", msgbuffer)
    end select
  end function readOneL2DataSet
  ! ----------------------------------------------------------------------------------
  function readOneL2IntDataSet(dataSetID, fileName, dataSetName, &
                               start, stride, edge, plane)
    integer (kind = FourByteIntKind), intent( in) :: dataSetID, plane
    character (len = *),              intent( in) :: fileName, dataSetName
    integer (kind = FourByteIntKind), dimension(2), &
                                      intent( in) :: start, stride, edge
    integer, dimension(edge(1), edge(2))          :: readOneL2IntDataSet

    ! !F90
    !
    ! !Description:
    !   Read a two dimensional matrix of integers from an HDF file and replace
    !     fill values with ModisLevel3FillValue. Variables aren't allowed to have more
    !     than three dimensions. We read along the first two, with the third dimension
    !     specified by the "plane" input argument.
    
    ! !Input Parameters:
    !   dataSetID: HDF data set ID, as provided by getDataSetID or SFselect
    !   fileName, dataSetName: character strings containing the name of the dataset
    !     being read and the file from which it is being read; used to make error
    !     reports more useful. 
    !    start: Follows HDF conventions. The array element from which to begin reading the
    !      HDF array. Zero based.
    !    stride: Follows HDF conventions. The frequency with which data should be read in
    !      each dimension. Stride 1 means read every element.
    !    edge: Follows HDF conventions. The number of elements to read in each dimension. If
    !      start(:) == 0 and stride(:) == 1, setting edge equal to the shape of the underlying
    !      array reads all elements. 
    !    plane: integer determining which (single) array section in the third dimension to read.
    !
    ! !Output Parameters:
    !   Function returns an real integer array of shape (edge(1), edge(2)) containing
    !     Level 2 data, calibrated to physical units, with all missing values to to the
    !     parameter ModisL3FillValue. 
    ! 
    ! !Revision History:
    !   $Log: Level2Read.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.4  1997/12/30  16:02:36  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.3  1997/11/13  15:15:51  pincus
    ! Added support for reading bit field values from a two dimensional SDS.
    ! (Previously, all bit field SDSs were supposed to have three dimensions.)
    !
    ! Revision 1.2  1997/11/10  20:33:22  pincus
    ! Removed tabs for ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:46:07  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   This procedure makes extensive use of allocatable arrays declared in the 
    !     data portion of the module. We discover the type of the uncalibrated 
    !     data, allocate the appropriate array, read in the data, calibrate it, 
    !     and deallocate the temporary array. 
    !
    ! !END
                                      
    ! Local variables
    integer                                 :: status
    integer (kind = FourByteIntKind  ), &
                    dimension(MAX_VAR_DIMS) :: hdfStart, hdfStride, hdfEdge
    character (len = 2048)                  :: msgbuffer

    hdfStart (:2) = 0;
    hdfStart ( 3) = plane
    hdfStride(:3) = 1
    hdfEdge  (: ) = getDataSetDimSizes(dataSetID)
    hdfEdge  ( 3) = 1

    ! Read data. Need to read different types and normalize to real values with 
    !   Level 3 fill values where appropriate. 
    select case (getDataSetType(dataSetID, fileName, dataSetName))
      case(DFNT_INT8)
        allocate(oneByteIntL2Data(hdfEdge(1), hdfEdge(2)) )
        status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, oneByteIntL2Data)

        if (status == FAIL) then
          msgbuffer = "Error reading data from dataset " //  &
             trim(dataSetname) // " in file " // trim(fileName) &
            // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
            // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
            // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2IntDataSet", msgbuffer)
         end if

        status = SFrnatt(dataSetID, SFfattr(dataSetID, "_FillValue"), &
                         oneByteIntFillValue)
        if (status == FAIL) then
              msgbuffer = "Error reading fill value from dataset " //  &
                           trim(dataSetname) // " in file " // trim(fileName) &
              // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
              // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
              // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2IntDataSet", msgbuffer)
        end if

        oneByteIntL2DataSample => oneByteIntL2Data((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                                   (start(2) + 1):(edge(2) * stride(2)):stride(2))

        readOneL2IntDataSet = int(oneByteIntL2DataSample)
        where (oneByteIntL2DataSample == oneByteIntFillValue) &
          readOneL2IntDataSet = int(ModisL3FillValue)
        nullify(oneByteIntL2DataSample)
        deallocate(oneByteIntL2Data)

      case(DFNT_INT16)
        allocate(twoByteIntL2Data(hdfEdge(1), hdfEdge(2)))
        status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, twoByteIntL2Data)

        if (status == FAIL) then
          msgbuffer = "Error reading data from dataset " // &
            trim(dataSetname) // " in file " // trim(fileName) &
            // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
            // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
            // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2IntDataSet", msgbuffer)
        end if

        status = SFrnatt(dataSetID, SFfattr(dataSetID, "_FillValue"), &
                         twoByteIntFillValue)

        if (status == FAIL) then
           msgbuffer = "Error reading fill value from dataset " //  &
             trim(dataSetname) // " in file " // trim(fileName) &
             // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
             // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
             // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2IntDataSet", msgbuffer)
        end if

        twoByteIntL2DataSample => twoByteIntL2Data((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                                   (start(2) + 1):(edge(2) * stride(2)):stride(2))
        readOneL2IntDataSet = int(twoByteIntL2DataSample)
        where (twoByteIntL2DataSample == twoByteIntFillValue) &
          readOneL2IntDataSet = int(ModisL3FillValue)
        nullify(twoByteIntL2DataSample)
        deallocate(twoByteIntL2Data)

      case(DFNT_INT32)
        allocate(fourByteIntL2Data(hdfEdge(1), hdfEdge(2)))
        status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, fourByteIntL2Data)

        if (status == FAIL) then
          msgbuffer = "Error reading data from dataset " // &
             trim(dataSetname) // " in file " // trim(fileName) &
             // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
             // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
             // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2IntDataSet", msgbuffer)
        end if

        status = SFrnatt(dataSetID, SFfattr(dataSetID, "_FillValue"), &
                         fourByteIntFillValue)

        if (status == FAIL) then
          msgbuffer = "Error reading fill value from dataset " // &
              trim(dataSetname) // " in file " // trim(fileName) &
              // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
              // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
              // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2IntDataSet", msgbuffer)
        end if

        fourByteIntL2DataSample => fourByteIntL2Data((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                                     (start(2) + 1):(edge(2) * stride(2)):stride(2))
        readOneL2IntDataSet = int(fourByteIntL2DataSample)
        where (fourByteIntL2DataSample == fourByteIntFillValue) &
          readOneL2IntDataSet = int(ModisL3FillValue)
        deallocate(fourByteIntL2Data)

      case default
        msgbuffer = "Non-integer data type for parameter " //  &
           trim(dataSetname) // " in file " // trim(fileName) &
           // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
           // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
           // char(10) // "notify SDST. "
        call FatalErrorMessage("ReadOneL2DataSet", msgbuffer)
    end select
  end function readOneL2IntDataSet
  ! ----------------------------------------------------------------------------------
  function readOneL2BitField(dataSetID, fileName, BitFieldName, &
                            start, stride, edge, plane, startBit, numBits) result(Bitvalues)
    integer (kind = FourByteIntKind), intent( in) :: dataSetID
    character (len = *),              intent( in) :: fileName, BitFieldName
    integer (kind = FourByteIntKind), dimension(2), &
                                      intent( in) :: start, stride, edge
    integer (kind = FourByteIntKind), intent( in) :: plane
    integer, optional,                intent( in) :: startBit, NumBits
    integer (kind = OneByteIntKind), &
                      dimension(edge(1), edge(2)) :: Bitvalues

    ! !F90
    !
    ! !Description:
    !   Read a two dimensional matrix of one byte values from an HDF file and extract 
    !     the relevant bits. We read along the first two dimensions, with the third 
    !     dimension specified by the "plane" input argument. 
    !  
    ! !Input Parameters:
    !   dataSetID: HDF data set ID, as provided by getDataSetID or SFselect
    !   fileName, BitFieldName: character strings containing the name of the dataset
    !     being read and the file from which it is being read; used to make error
    !     reports more useful. 
    !    start: Follows HDF conventions. The array element from which to begin reading the
    !      HDF array. Zero based.
    !    stride: Follows HDF conventions. The frequency with which data should be read in
    !      each dimension. Stride 1 means read every element.
    !    edge: Follows HDF conventions. The number of elements to read in each dimension. If
    !      start(:) == 0 and stride(:) == 1, setting edge equal to the shape of the underlying
    !      array reads all elements. 
    !    plane: integer determining which individual byte is to be read. The dimensions
    !      of the bit field in the file can be either (Longitude, Latitude, Byte) or 
    !      (Byte, Longitude, Latitude); this is determined within this procedure. 
    !    startBit: the starting location of the bit sequence to be extracted from the 
    !      selected byte. Zero based (Fortran 90 standard model). 
    !    numBits: number of bits to be extracted from the bytes read from the HDF file. 
    !
    ! !Output Parameters:
    !   Function returns an array of one byte integers with shape (edge(1), edge(2)) 
    !    containing the selected bits from the Level 2 Bit fields. 
    ! 
    ! !Revision History:
    !   $Log: Level2Read.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.4  1997/12/30  16:02:36  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.3  1997/11/13  15:15:51  pincus
    ! Added support for reading bit field values from a two dimensional SDS.
    ! (Previously, all bit field SDSs were supposed to have three dimensions.)
    !
    ! Revision 1.2  1997/11/10  20:33:22  pincus
    ! Removed tabs for ANSI compliance.
    !
    ! Revision 1.1  1997/10/30  17:46:07  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    In many QA fields, Byte is the most rapidly varying index. Since we only read
    !      one byte, this forces HDF to make many reads in discontiguous parts of the file, 
    !      which is painfully inefficient. In this procedure, we read the entire byte field, 
    !      then extract the required section from within memory. 
    !     Note that HDF arrays start at 0, while Fortran arrays start at 1. 
    !
    ! !END
    
    ! Local variables
    integer                              :: status
    integer (kind = FourByteIntKind)     :: numDimensions, dataSetType, numAttributes, &
                                            dimCount
    integer (kind = FourByteIntKind), &
                 dimension(MAX_VAR_DIMS) :: dimensionIDs
    character (len = MAX_NC_NAME)        :: dataSetName, dimensionName
    integer (kind = FourByteIntKind), &
                 dimension(MAX_VAR_DIMS) :: hdfStart, hdfStride, hdfEdge
    integer (kind = OneByteIntKind), allocatable, &
                  dimension(:, :, :)     :: allBitValues
    integer (kind = OneByteIntKind), allocatable, &
                  dimension(:, :)        :: planeOfBitValues
    character (len = 2048)               :: msgbuffer

    ! Check to be sure the SDS is of type byte. 
   if(getDataSetType(dataSetID = dataSetID) /= DFNT_INT8) then
      msgbuffer = "Dataset " // trim(BitFieldName) // " isn't a byte field." &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
      call FatalErrorMessage("readOneL2BitField", msgbuffer)
    end if

    ! The QA information is in a 3D byte array, but the QA parameter can be stored as 
    !   either the first (most rapidly varying) or last (most slowly varying) dimension. 
    !   What a pain. 
    ! First get the dimension IDs. Make sure there are only three dimensions, which 
    !   should be CellAcross, CellAlong, and the QA parameter. 
    status = SFginfo(dataSetID, dataSetName,      &
                     numDimensions, dimensionIDs, &
                     dataSetType, numAttributes)
    if(status == FAIL) then
          msgbuffer = "Can't get data set info for byte field" &
                        // trim(BitFieldName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
      call FatalErrorMessage("readOneL2BitField", msgbuffer)
    end if

    if(numDimensions /= 3 .and. plane > 0) then
       msgbuffer = "Byte field " // trim(BitFieldName) //  &
            " doesn't have 3 dimensions, but numDimensions " &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
      call FatalErrorMessage("readOneL2BitField", msgbuffer)
    end if

    ! Read all the values, then extract the plane we need later. This is much faster 
    !   then using stride /= 1, especially when the stride is in the first 
    !   (most rapdily varying) dimension. 
    hdfStart (:3) = 0
    hdfStride(:3) = 1
    hdfEdge  (: ) = getDataSetDimSizes(dataSetID)
    threeDimensionalSDS: if(hdfEdge(3) > 0) then 
      ! Must read third dimension, and decide which is non-geographic dimension
      allocate(allBitValues(hdfEdge(1), hdfEdge(2), hdfEdge(3)), stat = status)


      if(status /= 0) then
         msgbuffer = "Error allocating temporary memory." &
   // char(10) // "Operator Action:  Suspect system problem.  If a fault is " &
   // char(10) // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
        call FatalErrorMessage("readOneL2BitField", msgbuffer)
      end if

      status = SFrdata(dataSetID, hdfStart, hdfStride, hdfEdge, allBitValues)

      if (status == FAIL) then
        msgbuffer = "Error reading byte field " // trim(BitFieldName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
        call FatalErrorMessage("readOneL2BitField", msgbuffer)
      end if

      ! This is really clunky. Examine the dimension names of the first and last (third) 
      !   dimension. If the first four characters are "Cell", then this is not the QA 
      !   parameter dimension. (Can we be sure the dimension name for the QA parameter
      !   begins with "QA" ?) Extract the byte values for the single array plane specified
      !   by the plane arguement. 
      status = SFgdinfo(SFdimid(dataSetID, 1 - 1), dimensionName, &
                        dimCount, dataSetType, numAttributes)

      if(status == FAIL) then
        msgbuffer = "Can't get dimension info for byte field " // trim(BitFieldName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
        call FatalErrorMessage("readOneL2BitField", msgbuffer)
      end if

      if (dimensionName(1:4) /= "Cell") then
        Bitvalues(:, :) = allBitValues(plane + 1,                                      &
                                     (start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                     (start(2) + 1):(edge(2) * stride(2)):stride(2) )
      else
        status = SFgdinfo(SFdimid(dataSetID, 3 - 1), dimensionName, &
                          dimCount, dataSetType, numAttributes)

        if(status == FAIL) then
          msgbuffer = "Can't get dimension info for byte field " // trim(BitFieldName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2BitField", msgbuffer)
        end if

        if (dimensionName(1:4) /= "Cell") then
          Bitvalues(:, :) = allBitValues((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                         (start(2) + 1):(edge(2) * stride(2)):stride(2), &
                                         plane + 1)
        else
          msgbuffer = "QA parameter is not stored in either first or last dimension." &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
          call FatalErrorMessage("readOneL2BitField", msgbuffer)
        end if
      end if
      deallocate(allBitValues, stat = status)

      if (status /= 0) then
        msgbuffer = "Error deallocating temporary memory." &
   // char(10) // "Operator Action:  Suspect system problem.  If a fault is " &
   // char(10) // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
        call FatalErrorMessage("readOneL2BitField", msgbuffer)
      end if

    else ! only two dimensions in SDS. 
      allocate(planeOfBitValues(hdfEdge(1), hdfEdge(2)), stat = status)

      if(status /= 0) then
        msgbuffer = "Error allocating temporary memory." &
   // char(10) // "Operator Action:  Suspect system problem.  If a fault is " &
   // char(10) // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
        call FatalErrorMessage("readOneL2BitField", msgbuffer)
      end if
        
      status = SFrdata(dataSetID, hdfStart(:2), hdfStride(:2), hdfEdge(:2), planeOfBitValues)

      if (status == FAIL) then
        msgbuffer = "Error reading byte field " // trim(BitFieldName) &
   // char(10) // "Operator Action:  Check for valid PCF file.  If wrong or " &
   // char(10) // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
   // char(10) // "notify SDST. "
        call FatalErrorMessage("readOneL2BitField", msgbuffer)
      end if

      Bitvalues(:, :) = planeOfBitValues((start(1) + 1):(edge(1) * stride(1)):stride(1), &
                                         (start(2) + 1):(edge(2) * stride(2)):stride(2) )
      deallocate(planeOfBitValues, stat = status)

      if (status /= 0) then
         msgbuffer = "Error deallocating temporary memory." &
   // char(10) // "Operator Action:  Suspect system problem.  If a fault is " &
   // char(10) // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
         call FatalErrorMessage("readOneL2BitField", msgbuffer)
      end if

    end if threeDimensionalSDS
    
    if(present(startBit) .and. present(numBits) ) &
       Bitvalues = ibits(i   = Bitvalues, &
                         pos = startBit,  &
                         len = numBits )
  end function readOneL2BitField
  ! ----------------------------------------------------------------------------------
end module Level2Read
