module StatisticsDescription
  ! Module revision history
  !   $Log: StatisticsDescription.f90,v $
  !   Revision 1.1  2006/02/07 22:47:44  gumley
  !   Initial version
  !
  !
  ! Revision 1.9  2005/02/25  15:30:00  phubanks
  ! Modified NeedsJointParameter routine to test for "LogUncertainty" statistic type.
  ! Declare the default LogUncertaintySuffix (Log_Mean_Uncertainty) in the "Public data" section.
  ! Add logic for alternate UncertaintySuffix (QA_Log_Mean_Uncertainty) in the
  !  buildProcessingDatabase subroutine.
  !
  ! Revision 1.8  2005/02/08  15:30:00  phubanks
  ! Modified NeedsJointParameter routine to test for "Uncertainty" statistic type.
  ! Declare the default UncertaintySuffix (Mean_Uncertainty) in the "Public data" section.
  ! Add logic for alternate UncertaintySuffix (QA_Mean_Uncertainty) in the
  !  buildProcessingDatabase subroutine.
  !
  ! Revision 1.7  1999/08/19  14:30:56  egmoody
  ! Created a procedure called getRegressionInfo, which is used to read the
  ! fill value, and bounding range from output HDF metadata for regression
  ! statistics.
  !
  ! Revision 1.6  1998/05/05  18:38:32  pincus
  ! Removed memory leaks found by SDST; added provision for freeing memory
  ! from the processing database.
  !
  ! Revision 1.5  1998/03/18  16:50:48  pincus
  ! All one-dimensional output datasets are now ignored when building
  ! the processing database. If output products can't be matched with
  ! input data products, the error is now non-fatal, allowing us to
  ! continue processing even if one or more types of input products
  ! aren't available.
  !
  ! Revision 1.4  1997/12/30  15:55:32  pincus
  ! netCDF constants are now taken from the hdf module; removed reference
  ! to module netcdf. Names of the constants changed.
  !
  ! Revision 1.3  1997/11/18  20:38:35  pincus
  ! Added support for multiple datasets for output data set types
  ! "Area Fraction" and "Logarithmic".
  !
  ! Revision 1.2  1997/11/06  22:49:34  pincus
  ! Added functions getFractionInfo, getBitStringInfo
  !
  ! Revision 1.1  1997/11/06  21:39:47  pincus
  ! Initial revision
  
  use typeSizes,     only: oneByteIntKind, TwoByteIntKind, FourByteIntKind, &
                           FourByteRealKind
  use hdf,           only: SFstart, SFend, FAIL, DFACC_READ, MAX_NC_NAME, MAX_VAR_DIMS
  use hdfUtils,      only: getDataSetID, endDataSet, getNumDataSets, getDataSetList, &
                           getDataSetDimSizes, getAttributeLength, readAttribute
  use ErrorHandling, only: ErrorMessage, FatalErrorMessage
  use InputProducts, only: createInputProducts, addInputProductInfo, getInputProductInfo, &
                           getInputProductNames, getInputProductTypes,                    &
                           InputProductInfo
  implicit none
  private
   
  ! Public procedures
  ! File level procedures - how many input and output data sets, and what are their names?
  public :: getNumInputDataSets,  getInputDataSetName,  &
            getNumOutputDataSets, getOutputDataSetName, &
            buildProcessingDatabase, deleteProcessingDatabase
  ! Procedures for getting information about Level 2 datasets.
  public :: useNightTimeData, needsQA, readL2PixelsAs, getQAInfo, getBitStringInfo
  ! Procedures for Level 3 datasets.
  public :: usesAggregation, getAggregationInfo, getStatisticType, &
            needsJointParameter, getJointParameterInfo, &
            getHistogramInfo, getJointPDFInfo, getFractionInfo

  ! Public data         
  ! Simple statistics and regressions are represented by a single output dataset
  !   name, although more than one dataset is written. Here are the suffixes of
  !   the dataset names we keep.
  character (len = *), parameter, public :: simpleSuffix         = "_Minimum"
  character (len = *), parameter, public :: uncertaintySuffix    = "_Mean_Uncertainty"
  character (len = *), parameter, public :: loguncertaintySuffix = "_Log_Mean_Uncertainty"
  character (len = *), parameter, public :: logarithmicSuffix    = "_Log_Mean"
  character (len = *), parameter, public :: regressionSuffix     = "_Regression_Slope"
  character (len = *), parameter, public :: fractionSuffix       = "_Fraction"

  ! Private module data - used for building database of relationships between 
  !   input and output datasets
  character (len = MAX_NC_NAME), dimension(:), allocatable, save :: outputDataSetNames
  integer,                       dimension(:), allocatable, save :: inputProductKey
  type(InputProductInfo),        dimension(:), allocatable, save :: inputDataSetInfos
  
  character, parameter :: newLine = char(10)
contains
  ! ----------------------------------------------------------- 
  !  Procedures related to the input datasets: how many input parameters in 
  !    an HDF file are needed, and what is the name of the next input parameter?
  ! ----------------------------------------------------------- 
  function getNumInputDataSets(desiredInputProductType)
    integer, intent( in) :: desiredInputProductType
    integer              :: getNumInputDataSets
    ! !F90
    !
    ! !Description:
    !    Determine how many input data sets are contained in this input product.
    !
    ! !Input Parameters:
    !   desiredInputProductType (integer): the index of the input product type. 
    !     When the processing database is built at the beginning of execution, the 
    !     number of different input product types (e.g. MOD04, MOD05) is determined from 
    !     the runtime parameters. There should be 4 (MOD 04, 05, 06, and 07), though they
    !     aren't required to be in any particular order. desiredInputProductType is the  
    !     index of the current input product.
    !
    ! !Output Parameters:
    !   Function value is the number of input data sets that should be found in this 
    !     granule type. 
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    getNumInputDataSets = count(getInputProductTypes(inputDataSetInfos) == desiredInputProductType)
  end function getNumInputDataSets
  ! ----------------------------------------------------------- 
  function getInputDataSetName(inputProductType, dataSetIndex)
    integer, intent( in)          :: inputProductType, dataSetIndex
    character (len = MAX_NC_NAME) :: getInputDataSetName
    ! !F90
    !
    ! !Description:
    !   Returns the names of the input datasets that should be found in a given 
    !     graunle type. 
    !  
    ! !Input Parameters:
    !   inputProductType: as for function getNumInputDataSets, the index of the current 
    !     granule type, as determined from the runtime information. 
    !   dataSetIndex: the index of the desired input dataset within this input granule type. 
    !     Valid range is 1 to getNumInputDataSets(inputProductType)
    !
    ! !Output Parameters:
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variables
    character (len = MAX_NC_NAME), dimension(:), allocatable :: usefulNames
    logical,                       dimension(:), allocatable :: mask
    integer                                                  :: status
    character (len = 2048)                                   :: msgbuffer
    
    allocate(usefulNames(size(inputDataSetInfos)), stat = status)

     if(status /= 0) then
       msgbuffer = "Can't allocate memory for names of datasets." &
                    // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                    // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
       call FatalErrorMessage("getInputDataSetName", msgbuffer)
     end if

    allocate(mask       (size(inputDataSetInfos)), stat = status)
     if(status /= 0) then
       msgbuffer = "Can't allocate memory for mask. " &
                    // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                    // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
       call FatalErrorMessage("getInputDataSetName", msgbuffer)
     end if 

    mask = inputProductType == getInputProductTypes(inputDataSetInfos)
    usefulNames = pack(array = getInputProductNames(inputDataSetInfos), mask = mask)
    usefulNames(count(mask) + 1:) = ""

    if(dataSetIndex < 1 .or. dataSetIndex > count(mask)) then
      msgbuffer = "Invalid data set index: dataSetIndex. " &
                    // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                    // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                    // newLine // "notify SDST. "
      call FatalErrorMessage("getInputDataSetNames", msgbuffer)
    end if

    getInputDataSetName = usefulNames(dataSetIndex)
    deallocate(usefulNames)
    deallocate(mask)
    
  end function getInputDataSetName
  ! ----------------------------------------------------------- 
  !  Procedures for getting information about output datasets given the name
  !    of an input dataset. How many output parameters come from this input 
  !    parameter? What is the name of the next one? 
  ! ----------------------------------------------------------- 
  function inputDataSetNameToIndex(inputDataSetName)
    character (len = *), intent( in) :: inputDataSetName
    integer                          :: inputDataSetNameToIndex
    ! !F90
    !
    ! !Description:
    !    Determines which element of the database corresponds to a given 
    !      input data set name; this element is the element of inputDataSetInfos
    !      to be used to get QA info, etc.
    !  
    ! !Input Parameters:
    !    inputDataSetName: the name of the desired input data set. 
    !
    ! !Output Parameters:
    !    Function value is an integer index into the inputDataSetInfos(:) array. 
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END

    ! Local variables
    integer                            :: i, status
    integer, dimension(:), allocatable :: counter
    integer, dimension(1)              :: temp
    character (len = 2048)             :: msgbuffer
    
    allocate(counter(size(inputDataSetInfos)), stat = status)

    if(status /= 0) then
      msgbuffer = "Can't allocate space. " &
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
      call FatalErrorMessage("inputDataSetNameToIndex", msgbuffer)
    end if

    counter = (/ (i, i = 1, size(counter)) /)
    temp    = 0
    ! We're looking for the element of getInputProductNames(inputDataSetInfos)
    !   that matches inputDataSetName exactly; this should happen only once.
    select case(count(getInputProductNames(inputDataSetInfos) == inputDataSetName))
      case(:0)
        msgbuffer = "Input dataset name " // trim(inputDataSetName) // &
                    " does not appear in database. " &
                     // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                     // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                     // newLine // "notify SDST. "
        call FatalErrorMessage("inputDataSetNameNameToIndex", msgbuffer)
      case(1)
        temp = pack(array = counter, &
                    mask  = getInputProductNames(inputDataSetInfos) == inputDataSetName)
        inputDataSetNameToIndex = temp(1)
      case(2:)
        msgbuffer = "Input dataset name " // trim(inputDataSetName) // &
                    " appears more than once in database." &
                     // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                     // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                     // newLine // "notify SDST. "
        call FatalErrorMessage("inputDataSetNameNameToIndex", msgbuffer)
    end select
    
    deallocate(counter)
    
  end function inputDataSetNameToIndex
  ! ----------------------------------------------------------- 
  function getNumOutputDataSets(inputDataSetName)
    character (len = *), intent( in) :: inputDataSetName
    integer                          :: getNumOutputDataSets
    ! !F90
    !
    ! !Description:
    !   Determine how many Level 3 output datasets are derived from a given 
    !     input data set.
    !  
    ! !Input Parameters:
    !   inputDataSetName: the name of a parameter within a Level 2 granule file.
    !
    ! !Output Parameters:
    !   Function value is an integer indicating how many Level 3 datasets are derived from
    !     the named Level 2 dataset. 
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    getNumOutputDataSets = count(inputProductKey(:) == &
                                 inputDataSetNameToIndex(inputDataSetName))
  end function getNumOutputDataSets
  ! ----------------------------------------------------------- 
  function getOutputDataSetName(inputDataSetName, outputDataSetIndex)
    character (len = *), intent( in) :: inputDataSetName
    integer,             intent( in) :: outputDataSetIndex
    character (len = MAX_NC_NAME)    :: getOutputDataSetName
    ! !F90
    !
    ! !Description:
    !   Return each of the names of the output data sets that are derived from 
    !    an input data set.
    !  
    ! !Input Parameters:
    !  inputDataSetName: the name of the input data set
    !  outputDataSetIndex: the number of the output data set name to get. 
    !    Valid range is from 1 to getNumOutputDataSets(inputDataSetName); ordering
    !    is arbitrary and determined at runtime. 
    !
    ! !Output Parameters:
    !   Function value is the name of the outputDataSetIndex'th output data set 
    !     associated with the given input data set.
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variables
    character (len = MAX_NC_NAME), dimension(:), allocatable :: usefulNames
    logical,                       dimension(:), allocatable :: mask
    integer                                                  :: status
    character (len = 2048)                                   :: msgbuffer

    allocate(usefulNames(count(outputDataSetNames /= "")),  stat = status)

    if(status /= 0) then
      msgbuffer = "Can't allocate memory for names of datasets." &
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
      call FatalErrorMessage("getOutputDataSetName", msgbuffer)
    end if

    allocate(mask       (count(outputDataSetNames /= "")),  stat = status)

    if(status /= 0) then
      msgbuffer = "Can't allocate memory for mask. " &
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
      call FatalErrorMessage("getOutputDataSetName", msgbuffer)
    end if

    mask = inputProductKey(:) == inputDataSetNameToIndex(inputDataSetName)
    usefulNames = pack(array = outputDataSetNames, mask = mask)
    usefulNames(count(mask) + 1:) = ""
    getOutputDataSetName = usefulNames(outputDataSetIndex)
    deallocate(usefulNames); deallocate(mask)
    
  end function getOutputDataSetName
  ! ----------------------------------------------------------- 
  ! Functions needed to read in Level 2 granules. 
  ! ----------------------------------------------------------- 
  function useNightTimeData(inputDataSetName)
    character (len = *), intent( in) :: inputDataSetName
    logical                          :: useNightTimeData
    ! !F90
    !
    ! !Description:
    !   Determine whether granules collected at night need to be processed for 
    !     this input data set.
    !  
    ! !Input Parameters:
    !   inputDataSetName: the name of a parameter within a Level 2 granule file.
    !
    ! !Output Parameters:
    !   Function value is a logical indicating whether data collected at night should
    !     be processed.
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                             useNighttimeData = useNightTimeData)
  end function useNightTimeData
  ! ----------------------------------------------------------- 
  function needsQA(inputDataSetName)
    character (len = *), intent( in) :: inputDataSetName
    logical                          :: needsQA
    ! !F90
    !
    ! !Description:
    !   Determine whether this input dataset has any Quality Assurance (QA) 
    !     information associated with it.  
    !  
    ! !Input Parameters:
    !   inputDataSetName: the name of a parameter within a Level 2 granule file.
    !
    ! !Output Parameters:
    !   Function value is a logical indicating whether QA data are provided.
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variables
    character (len = MAX_NC_NAME) :: QADataSetName
    
    call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                             QADataSetName = QADataSetName)
    needsQA = .not. (QADataSetName == "None" .or. QADataSetName == "")
  end function needsQA
  ! ----------------------------------------------------------- 
  function readL2PixelsAs(inputDataSetName)
    character (len = *), intent( in) :: inputDataSetName
    character (len = MAX_NC_NAME)    :: readL2PixelsAs
    ! !F90
    !
    ! !Description:
    !   Determine whether this input dataset has any Quality Assurance (QA) 
    !     information associated with it.  
    !  
    ! !Input Parameters:
    !   inputDataSetName: the name of a parameter within a Level 2 granule file.
    !
    ! !Output Parameters:
    !   Function value is a logical indicating whether QA data are provided.
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                             readL2PixelsAs = readL2PixelsAs)
  end function readL2PixelsAs
  ! ----------------------------------------------------------- 
  subroutine getBitStringInfo(outputFileID, outputDataSetName, Byte, StartBit, NumBits)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character (len = *),              intent( in) :: outputDataSetName
    integer,                optional, intent(out) :: Byte, StartBit, NumBits
    !F90
    !
    ! !Description:
    !   Obtain the information needed to read a bit string input data set.
    !   This is done for each output dataset
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !    Byte: the (zero-based)index into the non-geographic dimension of the dataset
    !    StartBit: the (zero-based) number of the bit at which to begin reading.
    !    NumBits: the number of bits to read
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   The information describing how to read a bit string is stored in both the QA and  
    !     Aggregation sections of each output data set. This routine serves mainly a 
    !     a wrapper. 
    !
    ! !END
    integer (kind = FourByteIntKind) :: dataSetID
    
    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    
    if(present(Byte     )) &
      call readAttribute(dataSetID, "Level_2_Byte",      Byte)
    if(present(StartBit )) &
      call readAttribute(dataSetID, "Level_2_Start_Bit", StartBit)
    if(present(NumBits  )) &
      call readAttribute(dataSetID, "Level_2_Num_Bits",  NumBits)
      
    call endDataSet(dataSetID)
    
  end subroutine getBitStringInfo
  ! ----------------------------------------------------------- 
  subroutine getQAInfo(inputDataSetName,                     &
                       QADataSetName, QAByte, UsefulFlagBit, &
                       QAValueStartBit, QAValueNumBits)
    character (len = *),           intent( in) :: inputDataSetName
    character (len = *), optional, intent(out) :: QADataSetName
    integer,             optional, intent(out) :: QAByte, UsefulFlagBit, &
                                                  QAValueStartBit, QAValueNumBits
    !F90
    !
    ! !Description:
    !   Obtain the information needed to read the Quality Assurance (QA) information 
    !     for a given input data set. 
    !  
    ! !Input Parameters:
    !    inputDataSetName: the name of the desired input data set.
    !
    ! !Output Parameters:
    !    QADataSetName: the name of the HDF SDS in which the QA information is stored
    !    QAByte: the index into the non-geographic dimension of the QA dataset
    !    UsefulFlagBit: the (zero-based) number of the bit with the useful/not useful flag
    !    QAValueStartBit: the (zero-based) starting bit of the QA information
    !    QAValueNumBits: the number of bits to read.
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    if(present(QADataSetName   )) &
      call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                               QADataSetName   = QADataSetName)
                               
    if(present(QAByte          )) &
      call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                               QAByte          = QAByte)
                               
    if(present(UsefulFlagBit   )) &
      call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                               UsefulFlagBit   = UsefulFlagBit)
                               
    if(present(QAValueStartBit )) &
      call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                               QAValueStartBit = QAValueStartBit)
                               
    if(present(QAValueNumBits  )) &
      call getInputProductInfo(inputDataSetInfos(inputDataSetNameToIndex(inputDataSetName)), &
                               QAValueNumBits  = QAValueNumBits)

  end subroutine getQAInfo
  ! ----------------------------------------------------------- 
  ! Functions needed to read in information needed to compute specific Level 3 datasets
  !   from a given Level 2 data set. 
  ! ----------------------------------------------------------- 
  function usesAggregation(outputFileID, outputDataSetName)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character(len = *),               intent( in) :: outputDataSetName
    logical                                       :: usesAggregation
    ! !F90
    !
    ! !Description:
    !   Determine whether this output dataset is composed of an aggregated 
    !     subset of the Level 2 observations. 
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !   Function value is a logical indicating whether this output dataset
    !     is an aggregated subset of the original observations.  
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variables
    integer (kind = FourByteIntKind) :: dataSetID
    character (len = MAX_NC_NAME)    :: AggregationDataSetName
    
    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    call readAttribute(dataSetID, "Aggregation_Data_Set", AggregationDataSetName)
    usesAggregation = .not. (AggregationDataSetName == "None" .or. AggregationDataSetName == "")
    call endDataSet(dataSetID)
    
  end function usesAggregation
  ! ----------------------------------------------------------- 
  subroutine getAggregationInfo(outputFileID, outputDataSetName,      &
                       AggregationName, AggregationByte,              & 
                       AggregationStartBit, AggregationNumBits,       &
                       numValidCategories,       ValidCategories,     &
                       numAggregationCategories, AggregationCategories)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character (len = *),              intent( in) :: outputDataSetName
    character (len = *),    optional, intent(out) :: AggregationName
    integer,                optional, intent(out) :: AggregationByte,     &
                                                     AggregationStartBit, &
                                                     AggregationNumBits
    integer,                optional, intent(out) :: numValidCategories
    integer, dimension(:),  optional, intent(out) :: validCategories
    integer,                optional, intent(out) :: numAggregationCategories
    integer, dimension(:),  optional, intent(out) :: aggregationCategories
    ! !F90
    !
    ! !Description:
    !   Returns information necessary to extract an aggregated subset of observations
    !     from the original observations based on the values of a bit string. 
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !    AggregationName:       name of the HDF SDS containing the aggregation bit string
    !    AggregationByte:       index into the non-geographic dimension of the aggregation dataset
    !    AggregationStartBit:   (zero-based) starting bit of the aggregation information
    !    AggregationNumBits:    number of bits to read.
    !    numValidCategories:    the number of possible valid values of the aggregation bit string.
    !    validCategories:       a list of valid values for the aggregation bit string. These include
    !                           all values indicating a result, but not those indicating missing data.
    !    numAggregationCategories: the number of aggregation values used to extract the subset of 
    !                           observations for the current output dataset.
    !    aggregationCategories: list of the values of the aggregation string that go into the 
    !                           current output dataset. 
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    Some Level 3 data sets are statistics based on a subset of the original Level 2
    !      observations. The subset is determined by the value of a bit string in a QA flag. 
    !      For example, MOD06 products contain a flag indicating whether a pixel is  
    !      thought to contain, ice cloud, water cloud, mixed phase cloud, or clear air. 
    !      Suppose that the values of this QA falg were understood to be:
    !        0 - Missing data
    !        1 - Clear air
    !        2 - Water cloud
    !        3 - Ice cloud
    !        4 - Mixed phase cloud
    !      When constructing statistics for water clouds, this function would return
    !        numValidCategories = 4
    !        validCategories = (/ 1, 2, 3, 4 /)
    !        numAggregationCategories = 1
    !        aggregationCategories = (/ 2 /) 
    !
    ! !END

    ! Local variables
    integer (kind = FourByteIntKind) :: dataSetID
    
    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    
    if(present(AggregationName         )) &
      call readAttribute(dataSetID, "Aggregation_Data_Set", AggregationName)
    
    if(present(AggregationByte         )) &
      call readAttribute(dataSetID, "Aggregation_Byte", AggregationByte)
    
    if(present(AggregationStartBit     )) &
      call readAttribute(dataSetID, "Aggregation_Value_Start_Bit", AggregationStartBit)
    
    if(present(AggregationNumBits      )) &
      call readAttribute(dataSetID, "Aggregation_Value_Num_Bits", AggregationNumBits)
    
    if(present(numValidCategories      )) &
      numValidCategories = getAttributeLength(dataSetID, "Aggregation_Valid_Category_Values")
    
    if(present(validCategories         )) &
      call readAttribute(dataSetID, "Aggregation_Valid_Category_Values", validCategories)
    
    if(present(numAggregationCategories)) &
      numAggregationCategories = getAttributeLength(dataSetID, "Aggregation_Category_Values")
    
    if(present(aggregationCategories   )) &
      call readAttribute(dataSetID, "Aggregation_Category_Values", aggregationCategories)
    
    call endDataSet(dataSetID)

  end subroutine getAggregationInfo
  ! ----------------------------------------------------------- 
  subroutine getFractionInfo(outputFileID,           outputDataSetName, &
                             numValidCategories,       ValidCategories, &
                             numFractionCategories, fractionCategories)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character (len = *),    optional, intent( in) :: outputDataSetName
    integer,                optional, intent(out) :: numValidCategories
    integer, dimension(:),  optional, intent(out) :: validCategories
    integer,                optional, intent(out) :: numFractionCategories
    integer, dimension(:),  optional, intent(out) :: fractionCategories
    ! !F90
    !
    ! !Description:
    !   Returns information necessary to compute the fraction of valid pixels 
    !     which meet certain conditions. 
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !    numValidCategories:    the number of possible valid values of the fraction bit string.
    !    validCategories:       a list of valid values for the fraction bit string. These include
    !                           all values indicating a result, but not those indicating missing data.
    !    numFractionCategories: the number of values used to compute the area fraction for the 
    !                           current output dataset.
    !    fractionCategories: list of the values of the bit string that go into the computed 
    !                        area fraction. 
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    See the design note for getAggregationInfo. The fraction info is held in the 
    !      Aggregation local attributes, so this function is mostly a wrapper to 
    !      getAggregationInfo.
    !
    ! !END
    
    integer (kind = FourByteIntKind) :: dataSetID
    
    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    
    if(present(numValidCategories      )) &
      numValidCategories = getAttributeLength(dataSetID, "Fraction_Valid_Category_Values")
    if(present(validCategories         )) &
      call readAttribute(dataSetID, "Fraction_Valid_Category_Values", validCategories)
    if(present(numFractionCategories)) &
      numFractionCategories = getAttributeLength(dataSetID, "Fraction_Category_Values")
    if(present(fractionCategories   )) &
      call readAttribute(dataSetID, "Fraction_Category_Values", fractionCategories)
    
    call endDataSet(dataSetID)
        
  end subroutine getFractionInfo
  ! ----------------------------------------------------------- 
  function getStatisticType(outputFileID, outputDataSetName)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character(len = *),               intent( in) :: outputDataSetName
    character(len = MAX_NC_NAME)                  :: getStatisticType
    ! !F90
    !
    ! !Description:
    !   Determine the class of statistic from given output data set. Class here
    !     means, for example, "Histogram" or "Regression".  
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !   The function value is a character string of length MAX_NC_NAME containing
    !      the value of the Statistic_Type local attribute.
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END

    ! Local variables
    integer (kind = FourByteIntKind) :: dataSetID
    
    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    call readAttribute(dataSetID, "Statistic_Type", getStatisticType)
    call endDataSet(dataSetID)
    
  end function getStatisticType
  ! ----------------------------------------------------------- 
  subroutine getHistogramInfo(outputFileID, outputDataSetName, &
                              numBins, binBoundaries, numCategories, categories)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character (len = *),              intent( in) :: outputDataSetName
    integer,               optional,  intent(out) :: numBins
    real,    dimension(:), optional,  intent(out) :: binBoundaries
    integer,               optional,  intent(out) :: numCategories
    integer, dimension(:), optional,  intent(out) :: categories
    ! !F90
    !
    ! !Description:
    !   Determine the information needed to compute a histogram. 
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !   numBins: number of bins in the histogram of real values
    !   binBoundaries: the boundaries of the bins for a histogram of real values
    !   numCategories: the number of discrete values to count for a histogram of 
    !     integer values
    !   categories: a list of the discrete values for a histogram of integer values
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   If the parameter being accumulated is being read as an integer, we would use
    !     numCategories and categories; if it is real-valued we use numBins 
    !     and binBoundaries.
    !
    ! !END
 
    ! Local variables
    integer(kind = FourByteIntKind) :: dataSetID

    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    
    if(present(numBins      )) &
      numBins       = getAttributeLength(dataSetID, "Histogram_Bin_Boundaries") - 1

    if(present(binBoundaries)) &
      call readAttribute(dataSetID, "Histogram_Bin_Boundaries", binBoundaries)
      
    if(present(numCategories)) &
      numCategories = getAttributeLength(dataSetID, "Histogram_Categories")
    
    if(present(categories   )) &
      call readAttribute(dataSetID, "Histogram_Categories", categories)
    
    call endDataSet(dataSetID)
     
  end subroutine getHistogramInfo
  ! ----------------------------------------------------------- 
  function needsJointParameter(outputFileID, outputDataSetName)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character(len = *),               intent( in) :: outputDataSetName
    logical                                       :: needsJointParameter
    ! !F90
    !
    ! !Description:
    !   Determine whether or not a given output dataset requires us to read an 
    !     additional ("joint") parameter.  
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !   The function value is logical value indicating whether the given output 
    !     dataset requires more than one input data set.
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   Currently we only test to see if the statistic typs is either "Regression"
    !     or "Joint_Histogram". This code would need to be changed of other output 
    !     data set types were added that required joint parameters. 
    !
    ! !END

    ! Local variables
    integer (kind = FourByteIntKind) :: dataSetID
    character (len = MAX_NC_NAME)    :: statisticType
    
    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    statisticType = getStatisticType(outputFileID, outputDataSetName)

    if(statisticType == "Regression"  .or.       &
       statisticType == "Uncertainty" .or.       &
       statisticType == "LogUncertainty" .or.       &
       statisticType == "Joint_Histogram") then
      needsJointParameter = .true.
    else
      needsJointParameter = .false.
    end if

    call endDataSet(dataSetID)
    
  end function needsJointParameter
  ! ----------------------------------------------------------- 
  subroutine getJointParameterInfo(outputFileID, outputDataSetName, &
                                   jointInputParameterName,  &
                                   jointParameterWaveBand,   &
                                   jointParameterQAName,     &
                                   jointParameterQAByte,     &
                                   jointParameterQAStartBit, & 
                                   jointParameterQANumBits)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character (len = *),           intent( in) :: outputDataSetName
    character (len = *), optional, intent(out) :: jointInputParameterName
    integer,             optional, intent(out) :: jointParameterWaveBand
    character (len = *), optional, intent(out) :: jointParameterQAName
    integer,             optional, intent(out) :: jointParameterQAByte,     &
                                                  jointParameterQAStartBit, & 
                                                  jointParameterQANumBits                           
    ! !F90
    !
    ! !Description:
    !   Determine the information needed to read a joint parameter 
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !   jointInputParameterName: name of the SDS containing the joint parameter
    !   jointParameterWaveBand:  which waveband of the joint parameter to read
    !   jointParameterQAName, jointParameterQAByte, 
    !     jointParameterQAStartBit, jointParameterQANumBits: information required to
    !     read QA information for the joint parameter; exactly analagous to the 
    !     arguments of getQAInfo. 
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variables
    integer(kind = FourByteIntKind) :: dataSetID

    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    
    if(present(jointInputParameterName ))                                         &
      call readAttribute(dataSetID, "Joint_Parameter_Data_Set",                   &
                         jointInputParameterName)
                         
    if(present(jointParameterWaveBand  ))                                         &
      call readAttribute(dataSetID, "Joint_Parameter_Wave_Band",                  &
                         jointParameterWaveBand)
                         
    if(present(jointParameterQAName    ))                                         &
      call readAttribute(dataSetID, "Joint_Parameter_Quality_Assurance_Data_Set", &
                         jointParameterQAName)
                         
    if(present(jointParameterQAByte    ))                                         &
      call readAttribute(dataSetID, "Joint_Parameter_QA_Byte",                    &
                         jointParameterQAByte)
                         
    if(present(jointParameterQAStartBit))                                         &
      call readAttribute(dataSetID, "Joint_Parameter_QA_Value_Start_Bit",         &
                         jointParameterQAStartBit)
                         
    if(present(jointParameterQANumBits  ))                                        &
      call readAttribute(dataSetID, "Joint_Parameter_QA_Value_Num_Bits ",         &
                         jointParameterQANumBits)
                         
    call endDataSet(dataSetID)    
    
  end subroutine getJointParameterInfo
! ----------------------------------------------------------- 
  subroutine getJointPDFInfo(outputFileID, outputDataSetName,   &
                             jointWithName,                     &
                             bandNum, jointWithBand,            &
                             numBins, numJointBins,             &
                             binBoundaries, jointBinBoundaries)
    integer (kind = FourByteIntKind), intent( in) :: outputFileID
    character (len = *),              intent( in) :: outputDataSetName
    character (len = *),    optional, intent(out) :: jointWithName
    integer,                optional, intent(out) :: bandNum, jointWithBand
    integer,                optional, intent(out) :: numBins, numJointBins
    real, dimension(:),     optional, intent(out) :: binBoundaries, jointBinBoundaries
    ! !F90
    !
    ! !Description:
    !   Determine the information needed to compute a joint histogram. 
    !  
    ! !Input Parameters:
    !   outputFileID: the HDF ID number of the output file
    !   outputDataSetName: the name of the output data set
    !
    ! !Output Parameters:
    !   jointWithName: name of the HDF SDS containing the second parameter for the 
    !                  joint histogram
    !   bandNum:       index into the wavelength-dependent dimension of the first  parameter 
    !   jointWithBand: index into the wavelength-dependent dimension of the second parameter
    !   numBins:       number of bins in first  dimension of the joint histogram
    !   numJointBins:  number of bins in second dimension of the joint histogram
    !   binBoundaries:      boundaries of the bins in first  dimension
    !   jointBinBoundaries: boundaries of the bins in second dimension
    !
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   In some instances, joint histograms will be computed from the values of a single
    !     parameter at different wavelengths. In that case, jointWIthName would be the 
    !     same as the name of the input data set, but bandNum and jointWithBand would
    !     differ. 
    !
    ! !END

    ! Local variables
    integer(kind = FourByteIntKind) :: dataSetID

    dataSetID = getDataSetID(outputFileID, outputDataSetName)
    
    if(present(jointWithName     )) &
      call readAttribute(dataSetID, "Joint_Parameter_Data_Set", jointWithName)
      
    if(present(bandNum           )) &
      call readAttribute(dataSetID, "Wave_Band", bandNum)
      
    if(present(jointWithBand     )) &
      call readAttribute(dataSetID, "Joint_Parameter_Wave_Band", jointWithBand)
      
    if(present(numBins           )) &
      numBins      = getAttributeLength(dataSetID, "Histogram_Bin_Boundaries") - 1

    if(present(numJointBins      ))                &
      numJointBins = getAttributeLength(dataSetID, &
                                        "Joint_Parameter_Histogram_Bin_Boundaries") - 1

    if(present(binBoundaries     )) &
      call readAttribute(dataSetID, "Histogram_Bin_Boundaries", binBoundaries)
      
    if(present(jointBinBoundaries))                                             &
      call readAttribute(dataSetID, "Joint_Parameter_Histogram_Bin_Boundaries", &
                         jointBinBoundaries)
      
    call endDataSet(dataSetID)    
    
  end subroutine getJointPDFInfo 
  ! ----------------------------------------------------------- 
  subroutine buildProcessingDatabase(Level2GranuleFileNames, Level3DailyFileName)
    character(len = *), dimension(:), intent( in) :: Level2GranuleFileNames
    character(len = *),               intent( in) :: Level3DailyFileName
    ! !F90
    !
    ! !Description:
    !   Given a single Level 3 output file and a list of Level 2 graunle files 
    !     (one per each product type, i.e. MOD04, 05, ...), builds a database 
    !     with information about each input and output parameter.
    !  
    ! !Input Parameters:
    !   Level2GranuleFileNames: a list of Level 2 granule file names. One file 
    !     from each product needs to be supplied; only one file per product is necessary.
    !   Level3DailyFileName: the name of the Level 3 output file to be filled in. 
    !
    ! !Output Parameters:
    !   none.
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !   The database consists of a vector of derived type InputProductInfo, a vector 
    !     of output data set names, and a vector of keys indicating which input dataset
    !     is associated with each output dataset name. These vectors are held privately 
    !     in module level memory with the SAVE attribute, and are accessed by the 
    !     various procedures in this module. 
    !   This procedure must be called before any other procedure in the module is useful.
    !   Note that the variable inputDataSetNames plays two roles. Each output dataset knows the
    !     name of the input data set from which it is derived; when the variable inputDataSetNames
    !     is first allocated it is filled with a list of these names, which are used to initialize
    !     the database. That memory is then released, and reallocated once for each kind of
    !     input product. In this incarnation it holds a list of the data set names in the
    !     granule type being processed.
    !
    ! !END
  
    ! Local variables 
    integer                       :: numInputDataSets, numOutputDataSets
    character (len = MAX_NC_NAME), dimension(:), allocatable &
                                  :: inputDataSetNames
    integer                       :: dataSet, inputProductType
    character (len = MAX_NC_NAME) :: rootName
    character (len = MAX_NC_NAME) :: dataSetName
    integer, dimension(3)         :: status
    character (len = 2048)        :: msgbuffer

    ! Variables associated with each input product. 
    logical                       :: useNighttimeData
    character(len = MAX_NC_NAME)  :: nighttimeString, readL2PixelsAs, QADataSetName
    integer                       :: QAByte, UsefulFlagBit, &
                                     QAValueStartBit, QAValueNumBits
    
    ! Variables used in HDF calls. 
    integer(kind = FourByteIntKind) :: outputFileID, inputFileID, dataSetID
    integer(kind = FourByteIntKind), dimension(MAX_VAR_DIMS) &
                                    :: dimensionSizes
    ! -----------------
    ! Read names of the output datasets from Level 3 daily file.
    !   outputDataSetNames(:) is module-level data. 
    outputFileID = SFstart(trim(Level3DailyFileName), DFACC_READ)
    if(outputFileID == FAIL) then
      msgbuffer = "Can't open Level 3 file " // trim(Level3DailyFileName) &
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
      call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
    end if

    numOutputDataSets = getNumDataSets(outputFileID)
    allocate(outputDataSetNames(numOutputDataSets), stat = status(1))
    allocate( inputDataSetNames(numOutputDataSets), stat = status(2))
    allocate( inputProductKey  (numOutputDataSets), stat = status(3))
    if(any(status(:) /= 0)) then
        msgbuffer = "Can't allocate memory for output products. " &
                    // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                    // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
        call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
    end if

    outputDataSetNames(:) = ""
     inputDataSetNames(:) = ""
     inputProductKey  (:) = 0
    call getDataSetList(fileID = outputFileID, dataSetNames = outputDataSetNames)
    
    ! We remove any one-dimensional SDSs from the list of output data sets. 
    remove1D_SDSs: do dataSet = 1, numOutputDataSets
      dataSetID = getDataSetID(outputFileID, outputDataSetNames(dataSet))
      dimensionSizes = getDataSetDimSizes(dataSetID)
      if (dimensionSizes(2) < 1) outputDataSetNames(dataSet) = ""
      call endDataSet(dataSetID)
    end do remove1D_SDSs
           
    ! There are five sets of output datasets which are always computed together. They are
    !
    !   "Simple": _Mean, _Standard_Deviation, _Minimum, _Maximum
    !        if QA is used, also:      _QA_Mean,  _QA_Standard_Deviation
    !        if no QA is used, also:   _Pixel_Counts
    !
    !   "Uncertainty": _Mean_Uncertainty, _QA_Mean_Uncertainty
    !
    !   "LogUncertainty": _Log_Mean_Uncertainty, _QA_Log_Mean_Uncertainty
    !
    !   "Logarithmic": _Log_Mean, _Log_Standard_Deviation, _QA_Log_Mean, 
    !                  _QA_Log_Standard_Deviation
    !
    !   "Regression": _Regression_Slope, _Regression_Intercept, _Regression_R_squared, 
    !                 _Regression_Mean_Square_Error
    !
    !   "Fraction": *_Fraction_* and *_Fraction_*_Pixel_Counts
    !
    !   We'll leave only one of the output data set names in each set, then compute
    !   and write out the entire class of statistics when we come to the single dataset.

    removeExtraNames: do dataSet = 1, numOutputDataSets

      if(index(outputDataSetNames(dataSet), simpleSuffix) > 0) then
        rootName = outputDataSetNames(dataSet)
        rootName = rootName(1:(len_trim(rootName) - len_trim(simpleSuffix)))
        where(outputDataSetNames(:) == trim(rootName) // "_Mean"                 .or. &
              outputDataSetNames(:) == trim(rootName) // "_Standard_Deviation"   .or. &
              outputDataSetNames(:) == trim(rootName) // "_Maximum"              .or. &
              outputDataSetNames(:) == trim(rootName) // "_Pixel_Counts"         .or. &
              outputDataSetNames(:) == trim(rootName) // "_QA_Mean"              .or. &
              outputDataSetNames(:) == trim(rootName) // "_QA_Standard_Deviation")    &
          outputDataSetNames(:) = ""
      end if




      if(index(outputDataSetNames(dataSet), uncertaintySuffix) > 0) then
        rootName = outputDataSetNames(dataSet)
        rootName = rootName(1:(len_trim(rootName) - len_trim(uncertaintySuffix)))
        where(outputDataSetNames(:) == trim(rootName) // "_QA_Mean_Uncertainty")     &
          outputDataSetNames(:) = ""
      end if




      if(index(outputDataSetNames(dataSet), loguncertaintySuffix) > 0) then
        rootName = outputDataSetNames(dataSet)
        rootName = rootName(1:(len_trim(rootName) - len_trim(loguncertaintySuffix)))
        where(outputDataSetNames(:) == trim(rootName) // "_QA_Log_Mean_Uncertainty")     &
          outputDataSetNames(:) = ""
      end if




      if(index(outputDataSetNames(dataSet), logarithmicSuffix) > 0) then
        rootName = outputDataSetNames(dataSet)
        rootName = rootName(1:(len_trim(rootName) - len_trim(logarithmicSuffix)))
        where(outputDataSetNames(:) == trim(rootName) // "_Log_Standard_Deviation" .or. &
              outputDataSetNames(:) == trim(rootName) // "_QA_Log_Mean"            .or. &
              outputDataSetNames(:) == trim(rootName) // "_QA_Log_Standard_Deviation")  &
          outputDataSetNames(:) = ""
      end if

      if(index(outputDataSetNames(dataSet), regressionSuffix) > 0) then
        rootName = outputDataSetNames(dataSet)
        rootName = rootName(1:(len_trim(rootName) - len_trim(regressionSuffix)))
        where(outputDataSetNames(:) == trim(rootName) // "_Regression_Intercept"     .or. &
              outputDataSetNames(:) == trim(rootName) // "_Regression_R_squared"     .or. &
              outputDataSetNames(:) == trim(rootName) // "_Regression_Mean_Square_Error") &
          outputDataSetNames(:) = ""
      end if

      if(index(outputDataSetNames(dataSet), fractionSuffix) > 0) then
        rootName = outputDataSetNames(dataSet)
        if(index(outputDataSetNames(dataSet), "_Pixel_Counts") > 0) &
          rootName = rootName(1:(len_trim(rootName) - len("_Pixel_Counts")))
        where(outputDataSetNames(:) == trim(rootName) // "_Pixel_Counts")  &
          outputDataSetNames(:) = ""
       end if

    end do removeExtraNames

    ! Use numInputDataSets as a temporary variable to count non-blank output dataset names. 
    numInputDataSets = count(outputDataSetNames(:numOutputDataSets) /= "")
    outputDataSetNames = pack(array = outputDataSetNames(:numOutputDataSets), &
                              mask  = outputDataSetNames(:numOutputDataSets) /= "")
    numOutputDataSets = numInputDataSets
    outputDataSetNames(numOutputDataSets + 1:) = ""
    
    ! Get the names of the input parameters. Note that the names of the input
    !   parameters should appear only once. 
    numInputDataSets = 0
    do dataSet = 1, numOutputDataSets
      ! Get Derived_From_Level_2_Data_Set attribute into dataSetName
      dataSetID = getDataSetID(outputFileID, outputDataSetNames(dataSet))
      call readAttribute(dataSetID, "Derived_From_Level_2_Data_Set", dataSetName)
      call        endDataSet(dataSetID)
      ! Have we encountered the name of this input dataset before ?
      if(.not. any(inputDataSetNames(:numInputDataSets) == dataSetName)) then
        numInputDataSets = numInputDataSets + 1
        inputDataSetNames(numInputDataSets) = trim(dataSetName)
      end if
    end do
    
    ! Allocate space for the database of input parameters; initialize it. 
    allocate(inputDataSetInfos(numInputDataSets), stat = status(1))
    if(status(1) /= 0) then
      msgbuffer = "Can't allocate memory for data base. " &
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
      call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
    end if

    inputDataSetInfos = createInputProducts(inputDataSetNames(:numInputDataSets))
    ! We're done with the names of the input parameters for now. 
    deallocate(inputDataSetNames, stat = status(1))
    if(status(1) /= 0) then
      msgbuffer = "Can't deallocate memory. Continuing anyway... " &
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
      call ErrorMessage("buildProcessingDatabase", msgbuffer)
    end if

    ! Now go back through the output parameters and figure out which input parameter 
    !   to point to. 
    do dataSet = 1, numOutputDataSets
      dataSetID = getDataSetID(outputFileID, outputDataSetNames(dataSet))
      call readAttribute(dataSetID, "Derived_From_Level_2_Data_Set", dataSetName)
      call endDataSet(dataSetID)
      ! Choose the key into the array of input products. 
      inputProductKey(dataSet) = inputDataSetNameToIndex(dataSetName)
    end do
    
    ! Did we find an input parameter for every output parameter?
      if(any(inputProductKey(:numOutputDataSets) <= 0 .or. &
             inputProductKey(:numOutputDataSets) > numInputDataSets)) then         
         msgbuffer = "Couldn't find input parameters for some output parameters. " &
                     // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                     // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                     // newLine // "notify SDST. "
         call ErrorMessage("buildProcessingDatabase", msgbuffer)
      end if

   ! Fill in the information associated with each input data set.
    granuleTypes: do inputProductType = 1, size(Level2GranuleFileNames)
      inputFileID = SFstart(trim(Level2GranuleFileNames(inputProductType)), DFACC_READ)
      if(inputFileID == FAIL) then
        msgbuffer = "Can't open Level 2 file " // &
                     trim(Level2GranuleFileNames(inputProductType)) &
                     // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                     // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                     // newLine // "notify SDST. "
        call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
      end if

      numInputDataSets = getNumDataSets(inputFileID)
      allocate( inputDataSetNames(numInputDataSets), stat = status(1))
      if(status(1) /= 0) then
          msgbuffer = "Can't allocate memory. " &
                      // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                      // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
          call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
      end if

      call getDataSetList(fileID = inputFileID, dataSetNames = inputDataSetNames)

      inputDataSets: do dataSet = 1, numInputDataSets
        if(.not. any(inputDataSetNames(dataSet) ==              &
                     getInputProductNames(inputDataSetInfos)) ) &
            ! The input parameter in question is not found in the current input product file. 
            cycle inputDataSets
        ! We'll read the information associated with this input product 
        !   from the first of its output products. 
        dataSetName = getOutputDataSetName(inputDataSetNames(dataSet), 1)
        dataSetID = getDataSetID(outputFileID, dataSetName) 
        
        ! Use data collected at night?
        call readAttribute(dataSetID, "Included_Level_2_Nighttime_Data", nighttimeString)
        select case(trim(nighttimeString))
          case("FALSE", "False", "false")
            useNighttimeData = .false.
          case("TRUE",  "True",  "true")
            useNighttimeData = .true.
          case default
           msgbuffer = "Illegal value for attribute Included_Level_2_Nightime_Data " // &
               "in parameter " // trim(dataSetName) // ": " // trim(nighttimeString) &
               // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
               // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
               // newLine // "notify SDST. "
           call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
        end select
            
        ! How to read the Level 2 pixel values?
        call readAttribute(dataSetID, "Level_2_Pixel_Values_Read_As", readL2PixelsAs)
        ! QA Info
        call readAttribute(dataSetID, "Quality_Assurance_Data_Set", QADataSetName)
        if(trim(QADataSetName) == "None" ) then
          ! No QA information is used. 
          call addInputProductInfo(inputProduct     = &
             inputDataSetInfos(inputDataSetNameToIndex(inputDataSetNames(dataSet))), &
                                   productType      = inputProductType,       &
                                   readL2PixelsAs   = readL2PixelsAs,         &
                                   useNighttimeData = useNighttimeData)
        else 
          ! Read the QA information.
          call readAttribute(dataSetID, "QA_Byte",            QAByte)
          call readAttribute(dataSetID, "QA_Useful_Flag_Bit", UsefulFlagBit)
          call readAttribute(dataSetID, "QA_Value_Start_Bit", QAValueStartBit)
          call readAttribute(dataSetID, "QA_Value_Num_Bits",  QAValueNumBits)
          call addInputProductInfo(inputProduct     = &
             inputDataSetInfos(inputDataSetNameToIndex(inputDataSetNames(dataSet))), &
                                   productType      = inputProductType,    &
                                   readL2PixelsAs   = readL2PixelsAs,      &
                                   useNighttimeData = useNighttimeData,    &
                                   QADataSetName    = trim(QADataSetName), &
                                   QAByte           = QAByte,              &
                                   UsefulFlagBit    = UsefulFlagBit,       &
                                   QAValueStartBit  = QAValueStartBit,     &
                                   QAValueNumBits   = QAValueNumBits)
        end if 
        call endDataSet(dataSetID)
      end do inputDataSets
      
      deallocate(inputDataSetNames, stat = status(1))
      if(status(1) /= 0) then
        msgbuffer = "Can't deallocate memory. Continuing anyway ... " &
                     // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                     // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
        call ErrorMessage("buildProcessingDatabase", msgbuffer)
      end if
      
      status(1) = SFend(inputFileID)
      if (status(1) == FAIL) then
        msgbuffer = "Error closing Level 2 file " // &
                     trim(Level2GranuleFileNames(inputProductType)) &
                     // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                     // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
        call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
      end if
    end do granuleTypes
    
    status(1) = SFend(outputFileID)
    if (status(1) == FAIL) then
      msgbuffer = "Error closing Level 3 file " // trim(Level3DailyFileName) &
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
      call FatalErrorMessage("buildProcessingDatabase", msgbuffer)
    end if
    
    ! Did we find all the input parameters in one of the different input products?
    if(any(getInputProductTypes(inputDataSetInfos) <= 0 )) then
       msgbuffer = "Dataset information is unavailable " // &
                   "for one or more requested input parameters." &
                   // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                   // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                   // newLine // "notify SDST. "
       call ErrorMessage("buildProcessingDatabase", msgbuffer)
    end if
       
    if(any(getInputProductTypes(inputDataSetInfos) > size(Level2GranuleFileNames))) then
      msgbuffer = "Invalid input file type found for one or more input parameters. " &
                   // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                   // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                   // newLine // "notify SDST. "
      call ErrorMessage("buildProcessingDatabase", msgbuffer)
    end if

  end subroutine buildProcessingDatabase
  ! ----------------------------------------------------------- 
  subroutine deleteProcessingDatabase
    ! !F90
    !
    ! !Description:
    !  Frees module memory, allocated in subroutine buildProcessingDatabase, used for 
    !    the processing database. 
    !
    ! !Input Parameters:
    !   None. 
    !
    ! !Output Parameters:
    !   none.
    ! 
    ! !Revision History:
    !   See module revision history at top of file. 
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variable 
    integer, dimension(3)                :: status
    character (len = 2048)               :: msgbuffer

    if(allocated(outputDataSetNames)) deallocate(outputDataSetNames, stat = status(1))
    if(allocated(inputProductKey   )) deallocate(inputProductKey,    stat = status(2))
    if(allocated(inputDataSetInfos )) deallocate(inputDataSetInfos,  stat = status(3))
    
    if(any(status(:) /= 0)) then
      msgbuffer = "Error freeing database memory. Continuing. " & 
                  // newLine // "Operator Action:  Suspect system problem.  If a fault is " &
                  // newLine // "identified, correct and rerun PGE.  Otherwise, notify SDST." 
                      call ErrorMessage("deleteProcessingDatabase", msgbuffer)
    end if

  end subroutine deleteProcessingDatabase

end module StatisticsDescription
